Package: mcmc
Version: 0.9-8
Date: 2023-11-14
Title: Markov Chain Monte Carlo
Author: Charles J. Geyer <geyer@umn.edu> and Leif T. Johnson
     <ltjohnson@google.com>
Maintainer: Charles J. Geyer <geyer@umn.edu>
Depends: R (>= 3.6.0)
Imports: stats
Suggests: xtable, Iso
ByteCompile: TRUE
Description: Simulates continuous distributions of random vectors using
    Markov chain Monte Carlo (MCMC).  Users specify the distribution by an
    R function that evaluates the log unnormalized density.  Algorithms
    are random walk Metropolis algorithm (function metrop), simulated
    tempering (function temper), and morphometric random walk Metropolis
    (Johnson and Geyer, 2012, <doi:10.1214/12-AOS1048>,
    function morph.metrop),
    which achieves geometric ergodicity by change of variable.
License: MIT + file LICENSE
URL: http://www.stat.umn.edu/geyer/mcmc/,
        https://github.com/cjgeyer/mcmc
NeedsCompilation: yes
Packaged: 2023-11-15 05:41:20 UTC; geyer
Repository: CRAN
Date/Publication: 2023-11-16 22:20:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 16:02:45 UTC; unix
Archs: mcmc.so.dSYM

Package: mcmc
Version: 0.9-8
Date: 2023-11-14
Title: Markov Chain Monte Carlo
Author: Charles J. Geyer <geyer@umn.edu> and Leif T. Johnson
     <ltjohnson@google.com>
Maintainer: Charles J. Geyer <geyer@umn.edu>
Depends: R (>= 3.6.0)
Imports: stats
Suggests: xtable, Iso
ByteCompile: TRUE
Description: Simulates continuous distributions of random vectors using
    Markov chain Monte Carlo (MCMC).  Users specify the distribution by an
    R function that evaluates the log unnormalized density.  Algorithms
    are random walk Metropolis algorithm (function metrop), simulated
    tempering (function temper), and morphometric random walk Metropolis
    (Johnson and Geyer, 2012, <doi:10.1214/12-AOS1048>,
    function morph.metrop),
    which achieves geometric ergodicity by change of variable.
License: MIT + file LICENSE
URL: http://www.stat.umn.edu/geyer/mcmc/,
        https://github.com/cjgeyer/mcmc
NeedsCompilation: yes
Packaged: 2023-11-15 05:41:20 UTC; geyer
Repository: CRAN
Date/Publication: 2023-11-16 22:20:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 09:46:53 UTC; unix
Archs: mcmc.so.dSYM

useDynLib(mcmc, .registration = TRUE, .fixes = "C_")

export(metrop)
export(morph)
export(olbm)
export(initseq)
export(temper)
export(morph.metrop)
export(morph)
export(morph.identity)

S3method(metrop, metropolis)
S3method(metrop, "function")
S3method(morph.metrop, morph.metropolis)
S3method(morph.metrop, "function")
S3method(temper, tempering)
S3method(temper, "function")

importFrom(stats, runif)

euclid.norm <- function(x) {
  sqrt(sum(x * x))
}

isotropic <- function(f) {
  force(f)
  function(x) {
    x.norm <- euclid.norm(x)
    if (x.norm == 0)
      rep(0, length(x))
    else
      f(x.norm) * x / x.norm
  }
}

isotropic.logjacobian <- function(f, d.f) {
  force(f)
  force(d.f)
  function(x) {
    x.norm <- euclid.norm(x)
    k <- length(x)
    if (x.norm == 0) {
      k * log(d.f(x.norm))
    } else {
      log(d.f(x.norm)) + (k - 1) * (log(f(x.norm)) - log(x.norm))
    }
  }
}

# rearranged by charlie
# add starting point so we can use a better starting point for
#     the current actual use
# also do one more Newton step after error is < sqrt(machine.eps)
#     so ultimate error is about machine.eps
newton.raphson <- function(f, df, x, r, x0 = 2 * r) {
  f.err <- function(cur) f(cur) - x
  step <- function(cur, err=NULL)
    cur - ifelse(is.null(err), f.err(cur), err) / df(cur)
  err <- f.err(x0)
  while(err >= sqrt(.Machine$double.eps)) {
    x0 <- step(x0, err)
    err <- f.err(x0)
  }
  # if you don't want to use an extra-step, replace this return with
  # return(x0)
  return(step(x0, err))
}

subexponential <- function(b=1) {
  if (missing(b) | is.null(b)) b <- 1
  stopifnot(b > 0)
  force(b)
  f.inv <- function(x) ifelse(x > 1/b,
                              exp(b * x) - exp(1)/3,
                              (x * b)^3 * exp(1) / 6 + x * b * exp(1) / 2)
  d.f.inv <- function(x) ifelse(x > 1/b,
                                b * exp(b * x),
                                b * (x * b)^2 * exp(1) / 2 + b * exp(1) / 2)
  f <- function(x) {
    # x > exp(b * 1 / b) - exp(1) / 3
    if (x > 2 * exp(1) / 3) {
      log(x + exp(1)/3) / b
    } else {
      poly.inv <- exp(1/3) *
        (sqrt(b^12 * (9 * x^2 + exp(2))) - 3 * b^6 * x)^(-1/3)
      poly.inv * b - 1 / (poly.inv * b^3)
    }
  }
  return(list(f=f, f.inv=f.inv, d.f.inv=d.f.inv))
}

exponential <- function(r=1, p=3) {
  if (missing(p) || is.null(p)) p <- 3
  if (missing(r) || is.null(r)) r <- 0
  stopifnot(p > 2)
  stopifnot(r >= 0)
  f.inv <- function(x) ifelse(x <= r, x, x + (x-r)^p)
  d.f.inv <- function(x) ifelse(x <= r, 1, 1 + p * (x-r)^(p-1))
  if (p == 3) {
    g <- function(x) {
      n <- sqrt((27*r-27*x)^2 + 108) + 27 * (r - x)
      r + (2/n)^(1/3) - (n/2)^(1/3)/3
    }
    f <- function(x) ifelse(x < r, x, g(x))
  } else {
    # No general closed form solution exists.  However, since the
    # transformation has polynomial form, using the Newton-Raphson method
    # should work well.
    f <- function(x) ifelse(x < r, x,
        newton.raphson(f.inv, d.f.inv, x, r, x0 = r + x^(1 / p)))
  }
  return(list(f=f, f.inv=f.inv, d.f.inv=d.f.inv))
}

.make.outfun <- function(out) {
  force(out)
  function(f) {
    force(f)
    if (is.null(f))
      return(out$inverse)
    else if (is.function(f))
      return(function(state, ...) f(out$inverse(state), ...))
    else
      return(function(state) out$inverse(state)[f])
  }
}

identity.func <- function(x) x
morph.identity <- function() {
  out <- list(transform=identity.func,
              inverse=identity.func,
              lud=function(f) function(x, ...) f(x, ...),
              log.jacobian=function(x) 0,
              center=0,
              f=identity.func,
              f.inv=identity.func)
  out$outfun <- .make.outfun(out)
  return(out)
}

morph <- function(b, r, p, center) {
  if (all(missing(b), missing(r), missing(p), missing(center)))
    return(morph.identity())
  if (missing(center)) center <- 0
  use.subexpo <- !missing(b)
  use.expo <- !(missing(r) && missing(p))
  
  if (!use.expo && !use.subexpo) {
    f <- function(x) x
    f.inv <- function(x) x
    log.jacobian <- function(x) 0
  } else {
    if (use.expo && !use.subexpo) {
      expo <- exponential(r, p)
      
      f <- expo$f
      f.inv <- expo$f.inv
      d.f.inv <- expo$d.f.inv
    } else if (!use.expo && use.subexpo) {
      subexpo <- subexponential(b)
      
      f <- subexpo$f
      f.inv <- subexpo$f.inv
      d.f.inv <- subexpo$d.f.inv
    } else { #use.expo && use.subexpo
      expo <- exponential(r, p)
      subexpo <- subexponential(b)
      
      f <- function(x) expo$f(subexpo$f(x))
      f.inv <- function(x) subexpo$f.inv(expo$f.inv(x))
      d.f.inv <- function(x) expo$d.f.inv(x) * subexpo$d.f.inv(expo$f.inv(x))
    }
    
    f <- isotropic(f)
    f.inv <- isotropic(f.inv)
    log.jacobian <- isotropic.logjacobian(f.inv, d.f.inv)
  }

  out <- list(f=f, f.inv=f.inv, log.jacobian=log.jacobian,
              center=center)
  out$transform <- function(state) out$f(state - out$center)
  out$inverse <- function(state) out$f.inv(state) + out$center
  
  out$outfun <- .make.outfun(out)

  out$lud <- function(lud) {
    force(lud)
    function(state, ...) {
      foo <- lud(out$inverse(state), ...)
      if (length(foo) != 1)
          stop("log unnormalized density function returned vector not scalar")
      if (is.na(foo))
          stop("log unnormalized density function returned NA or NaN")
      if (foo == -Inf) return(foo)
      if (! is.finite(foo))
          stop("log unnormalized density function returned +Inf")
      foo + out$log.jacobian(state)
    }
  }

  return(out)
}

 # test outfun (function)

 epsilon <- 1e-15

 library(mcmc)

 RNGkind("Marsaglia-Multicarry")
 set.seed(42)

 n <- 100
 rho <- 0.5
 beta0 <- 0.25
 beta1 <- 1
 beta2 <- 0.5

 x1 <- rnorm(n)
 x2 <- rho * x1 + sqrt(1 - rho^2) * rnorm(n)
 eta <- beta0 + beta1 * x1 + beta2 * x2
 p <- 1 / (1 + exp(- eta))
 y <- as.numeric(runif(n) < p)

 out <- glm(y ~ x1 + x2, family = binomial())

 logl <- function(beta) {
     if (length(beta) != 3) stop("length(beta) != 3")
     beta0 <- beta[1]
     beta1 <- beta[2]
     beta2 <- beta[3]
     eta <- beta0 + beta1 * x1 + beta2 * x2
     p <- exp(eta) / (1 + exp(eta))
     return(sum(log(p[y == 1])) + sum(log(1 - p[y == 0])))
 }

 out.metro <- metrop(logl, coefficients(out), 1e3, scale = 0.01)
 out.metro$accept

 out.metro <- metrop(out.metro, scale = 0.1)
 out.metro$accept

 out.metro <- metrop(out.metro, scale = 0.5)
 out.metro$accept

 apply(out.metro$batch, 2, mean)

 out.metro <- metrop(logl, as.numeric(coefficients(out)), 1e2,
     scale = 0.5, debug = TRUE, outfun = function(x) c(x, x^2))

 niter <- out.metro$nbatch * out.metro$blen * out.metro$nspac
 niter == nrow(out.metro$current)
 niter == nrow(out.metro$proposal)
 all(out.metro$current[1, ] == out.metro$initial)
 all(out.metro$current[niter, ] == out.metro$final) |
     all(out.metro$proposal[niter, ] == out.metro$final)

 .Random.seed <- out.metro$initial.seed
 d <- ncol(out.metro$proposal)
 n <- nrow(out.metro$proposal)
 my.proposal <- matrix(NA, n, d)
 my.u <- double(n)
 ska <- out.metro$scale
 for (i in 1:n) {
     my.proposal[i, ] <- out.metro$current[i, ] + ska * rnorm(d)
     if (is.na(out.metro$u[i])) {
         my.u[i] <- NA
     } else {
         my.u[i] <- runif(1)
     }
 }
 max(abs(out.metro$proposal - my.proposal)) < epsilon
 all(is.na(out.metro$u) == is.na(my.u))
 all(out.metro$u[!is.na(out.metro$u)] == my.u[!is.na(my.u)])

 my.curr.log.green <- apply(out.metro$current, 1, logl)
 my.prop.log.green <- apply(out.metro$proposal, 1, logl)
 all(is.na(out.metro$u) == (my.prop.log.green > my.curr.log.green))
 foo <- my.prop.log.green - my.curr.log.green
 max(abs(foo - out.metro$log.green)) < epsilon

 my.accept <- is.na(my.u) | my.u < exp(foo)
 sum(my.accept) == round(n * out.metro$accept)
 if (my.accept[niter]) {
     all(out.metro$proposal[niter, ] == out.metro$final)
 } else {
     all(out.metro$current[niter, ] == out.metro$final)
 }

 my.current <- out.metro$current
 my.current[my.accept, ] <- my.proposal[my.accept, ]
 my.current <- rbind(out.metro$initial, my.current[- niter, ])
 max(abs(out.metro$current - my.current)) < epsilon

 my.path <- matrix(NA, n, d)
 my.path[my.accept, ] <- out.metro$proposal[my.accept, ]
 my.path[! my.accept, ] <- out.metro$current[! my.accept, ]
 nspac <- out.metro$nspac

 my.path <- my.path[seq(nspac, niter, by = nspac), ]

 fred <- t(apply(my.path, 1, out.metro$outfun))
 k <- ncol(fred)

 foom <- array(as.vector(t(fred)), c(k, out.metro$blen, out.metro$nbatch))
 boom <- t(apply(foom, c(1, 3), mean))

 all(dim(boom) == dim(out.metro$batch))
 max(abs(boom - out.metro$batch)) < epsilon

 goom <- cbind(my.path, my.path^2)
 all(dim(goom) == dim(out.metro$batch))
 max(abs(goom - out.metro$batch)) < epsilon

metrop <- function(obj, initial, nbatch, blen = 1,
    nspac = 1, scale = 1, outfun, debug = FALSE, ...)
UseMethod("metrop")

metrop.metropolis <- function(obj, initial, nbatch, blen = 1,
    nspac = 1, scale = 1, outfun, debug = FALSE, ...)
{
    if (missing(nbatch)) nbatch <- obj$nbatch
    if (missing(blen)) blen <- obj$blen
    if (missing(nspac)) nspac <- obj$nspac
    if (missing(scale)) scale <- obj$scale
    if (missing(debug)) debug <- obj$debug
    assign(".Random.seed", obj$final.seed, .GlobalEnv)
    if (missing(outfun)) {
        if (is.null(obj$outfun)) {
            metrop.function(obj$lud, obj$final, nbatch, blen,
                nspac, scale, debug = debug, ...)
        } else {
            metrop.function(obj$lud, obj$final, nbatch, blen,
                nspac, scale, obj$outfun, debug = debug, ...)
        }
    } else {
        metrop.function(obj$lud, obj$final, nbatch, blen,
            nspac, scale, outfun, debug, ...)
    }
}

metrop.function <- function(obj, initial, nbatch, blen = 1,
    nspac = 1, scale = 1, outfun, debug = FALSE, ...)
{
    if (! exists(".Random.seed")) runif(1)
    saveseed <- .Random.seed
    func1 <- function(state) obj(state, ...)
    env1 <- environment(fun = func1)
    if (missing(outfun)) {
        func2 <- NULL
        env2 <- NULL
        outfun <- NULL
    } else if (is.function(outfun)) {
        func2 <- function(state) outfun(state, ...)
        env2 <- environment(fun = func2)
    } else {
        func2 <- outfun
        env2 <- NULL
    }

    out.time <- system.time(
    out <- .Call(C_metrop, func1, initial, nbatch, blen, nspac,
        scale, func2, debug, env1, env2)
    )
    out$initial.seed <- saveseed
    out$final.seed <- .Random.seed
    out$time <- out.time
    out$lud <- obj
    out$nbatch <- nbatch
    out$blen <- blen
    out$nspac <- nspac
    out$scale <- scale
    out$outfun <- outfun
    out$batch <- t(out$batch)
    out$debug <- debug
    if (! is.null(out$current)) out$current <- t(out$current)
    if (! is.null(out$proposal)) out$proposal <- t(out$proposal)
    if (! is.null(out$z)) out$z <- t(out$z)
    class(out) <- c("mcmc", "metropolis")
    return(out)
}

 library(mcmc)

 set.seed(42)

 data(foo)
 attach(foo)

 out <- glm(y ~ x1 + x2 + x3, family = binomial, x = TRUE)
 summary(out)

 modmat <- out$x

 models <- cbind(rep(0:1, each = 4), rep(rep(0:1, times = 2), each = 2),
               rep(0:1, times = 4))

 exes <- paste("x", 1:3, sep = "")
 betas <- NULL
 for (i in 1:nrow(models)) {
     inies <- as.logical(models[i, ])
     foo <- exes[inies]
     bar <- paste("y ~", paste(foo, collapse = " + "))
     if (! any(inies)) bar <- paste(bar, "1")
     baz <- glm(as.formula(bar), family = binomial)
     beta <- rep(0, 4)
     beta[c(TRUE, inies)] <- baz$coef
     betas <- rbind(betas, beta)
 }

 neighbors <- matrix(FALSE, nrow(models), nrow(models))
 for (i in 1:nrow(neighbors)) {
     for (j in 1:ncol(neighbors)) {
         foo <- models[i, ]
         bar <- models[j, ]
         if (sum(foo != bar) == 1) neighbors[i, j] <- TRUE
     }
 }

 ludfun <- function(state, ...) {
     stopifnot(is.numeric(state))
     stopifnot(length(state) == ncol(models) + 2)
     stopifnot(length(state) == ncol(models) + 2)
     icomp <- state[1]
     stopifnot(icomp == as.integer(icomp))
     stopifnot(1 <= icomp && icomp <= nrow(models))
     beta <- state[-1]
     inies <- c(TRUE, as.logical(models[icomp, ]))
     beta.logl <- beta
     beta.logl[! inies] <- 0
     eta <- as.numeric(modmat %*% beta.logl)
     logp <- ifelse(eta < 0, eta - log1p(exp(eta)), - log1p(exp(- eta)))
     logq <- ifelse(eta < 0, - log1p(exp(eta)), - eta - log1p(exp(- eta)))
     logl <- sum(logp[y == 1]) + sum(logq[y == 0])
     val <- logl - sum(beta^2) / 2
     return(val)
 }

 ludval <- NULL
 for (i in 1:nrow(models)) ludval <- c(ludval, ludfun(c(i, betas[i, ])))
 all(is.finite(ludval))


 out <- temper(ludfun, initial = betas, neighbors = neighbors, nbatch = 20,
     blen = 10, nspac = 5, scale = 0.56789, parallel = TRUE, debug = TRUE)

 names(out)

 ### check decision about within-component or jump/swap

 identical(out$unif.which < 0.5, out$which)

 identical(out$which, out$proposal[ , 1] == out$coproposal[ , 1])

 ### check proposal and coproposal are actually current state or part thereof

 prop <- out$proposal
 coprop <- out$coproposal
 prop.i <- prop[ , 1]
 coprop.i <- coprop[ , 1]
 alt.prop <- prop
 alt.coprop <- coprop
 for (i in 1:nrow(prop)) {
     alt.prop[i, ] <- c(prop.i[i], out$state[i, prop.i[i], ])
     alt.coprop[i, ] <- c(coprop.i[i], out$state[i, coprop.i[i], ])
 }
 identical(coprop, alt.coprop)
 identical(prop[! out$which, ], alt.prop[! out$which, ])
 identical(prop[out$which, 1], alt.prop[out$which, 1])

 ### check hastings ratio calculated correctly

 foo <- apply(prop, 1, ludfun)
 fooco <- apply(coprop, 1, ludfun)
 prop[ , 1] <- out$coproposal[ , 1]
 coprop[ , 1] <- out$proposal[ , 1]
 foo.swap <- apply(prop, 1, ludfun)
 fooco.swap <- apply(coprop, 1, ludfun)
 log.haste <- ifelse(out$which, foo - fooco,
     foo.swap + fooco.swap - foo - fooco)
 all.equal(log.haste, out$log.hastings)

 ### check hastings rejection decided correctly

 identical(out$log.hastings >= 0, is.na(out$unif.hastings))
 all(out$log.hastings < 0 | out$acceptd)
 identical(out$acceptd,
     out$log.hastings >= 0 | out$unif.hastings < exp(out$log.hastings))

 ### check acceptance carried out or not (according to decision) correctly

 before <- out$state
 after <- before
 after[- dim(after)[1], , ] <- before[-1, , ]
 after[dim(after)[1], , ] <- out$final
 my.after <- before
 for (i in 1:length(out$acceptd)) {
     if (out$acceptd[i]) {
         if (out$which[i]) {
             j <- out$proposal[i, 1]
             my.after[i, j, ] <- out$proposal[i, -1]
         } else {
             j <- out$proposal[i, 1]
             k <- out$coproposal[i, 1]
             my.after[i, j, ] <- out$coproposal[i, -1]
             my.after[i, k, ] <- out$proposal[i, -1]
         }
     }
 }
 identical(after, my.after)

 ### check within-component proposal

 my.coproposal.within <- out$coproposal[out$which, ]
 proposal.within <- out$proposal[out$which, ]
 my.z <- out$norm[out$which, ]
 my.proposal.within <- my.coproposal.within
 my.proposal.within[ , -1] <- my.coproposal.within[ , -1] + out$scale * my.z
 all.equal(proposal.within, my.proposal.within)

 my.unif.choose <- out$unif.choose[out$which, 1]
 my.i <- floor(nrow(models) * my.unif.choose) + 1
 all(1 <= my.i & my.i <= nrow(models))
 identical(my.i, my.coproposal.within[ , 1])

 ### check swap proposal

 coproposal.swap <- out$coproposal[! out$which, ]
 proposal.swap <- out$proposal[! out$which, ]
 unif.choose.swap <- out$unif.choose[! out$which, ]
 my.i <- floor(nrow(models) * unif.choose.swap[ , 1]) + 1
 nneighbors <- apply(out$neighbors, 1, sum)
 my.nneighbors <- nneighbors[my.i]
 my.k <- floor(my.nneighbors * unif.choose.swap[ , 2]) + 1
 my.j <- my.k
 foo <- seq(1, ncol(out$neighbors))
 for (i in seq(along = my.j)) {
     my.j[i] <- (foo[out$neighbors[my.i[i], ]])[my.k[i]]
 }
 identical(coproposal.swap[ , 1], my.i)
 identical(proposal.swap[ , 1], my.j)

 ### check standard normal and uniform random numbers are as purported

 save.Random.seed <- .Random.seed
 .Random.seed <- out$initial.seed

 nx <- ncol(out$initial)
 niter <- out$nbatch * out$blen * out$nspac
 my.norm <- matrix(NA, nrow = nrow(out$norm), ncol = ncol(out$norm))
 my.unif.which <- rep(NA, niter)
 my.unif.hastings <- rep(NA, niter)
 my.unif.choose <- matrix(NA, niter, 2)
 for (iiter in 1:niter) {
     my.unif.which[iiter] <- runif(1)
     if (out$which[iiter]) {
         my.unif.choose[iiter, 1] <- runif(1)
         my.norm[iiter, ] <- rnorm(nx)
         if (out$log.hastings[iiter] < 0) my.unif.hastings[iiter] <- runif(1) 
     } else {
         my.unif.choose[iiter, ] <- runif(2)
         if (out$log.hastings[iiter] < 0) my.unif.hastings[iiter] <- runif(1) 
     }
 }
 identical(my.norm, out$norm)
 identical(my.unif.which, out$unif.which)
 identical(my.unif.hastings, out$unif.hastings)
 identical(my.unif.choose, out$unif.choose)

 .Random.seed <- save.Random.seed

 ### check batch means

 foo <- after[seq(1, niter) %% out$nspac == 0, , ]
 foo <- array(as.vector(foo), dim = c(out$blen, out$nbatch, dim(foo)[2:3]))
 foo <- apply(foo, c(2, 3, 4), mean)
 all.equal(foo, out$batch)

 ### check acceptance rates

 accept.within <- out$acceptd[out$which]
 my.i.within <- out$coproposal[out$which, 1]
 my.acceptx <- as.vector(sapply(split(accept.within, my.i.within), mean))
 identical(my.acceptx, out$acceptx)

 accept.swap <- out$acceptd[! out$which]
 my.i.swap <- out$coproposal[! out$which, 1]
 my.j.swap <- out$proposal[! out$which, 1]
 nmodel <- nrow(out$neighbors)
 my.accepti <- matrix(NA, nmodel, nmodel)
 for (i in 1:nmodel) {
     for (j in 1:nmodel) {
         if (out$neighbors[i, j]) {
             my.accepti[i, j] <-
                 mean(accept.swap[my.i.swap == i & my.j.swap == j])
         }
     }
 }
 identical(my.accepti, out$accepti)

 ### check scale vector

 nx <- ncol(models) + 1
 newscale <- rnorm(nx, 0.5, 0.1)

 out <- temper(out, scale = newscale)

 my.coproposal.within <- out$coproposal[out$which, ]
 proposal.within <- out$proposal[out$which, ]
 my.z <- out$norm[out$which, ]
 my.proposal.within <- my.coproposal.within
 my.proposal.within[ , -1] <- my.coproposal.within[ , -1] +
     sweep(my.z, 2, out$scale, "*")
 all.equal(proposal.within, my.proposal.within)

 ### check scale matrix

 matscale <- matrix(rnorm(nx * nx, 0.0, 0.1), nx, nx)
 diag(matscale) <- 0.56789

 out <- temper(out, scale = matscale)

 my.coproposal.within <- out$coproposal[out$which, ]
 proposal.within <- out$proposal[out$which, ]
 my.z <- out$norm[out$which, ]
 my.proposal.within <- my.coproposal.within
 my.proposal.within[ , -1] <- my.coproposal.within[ , -1] +
     my.z %*% t(out$scale)
 all.equal(proposal.within, my.proposal.within)

 ### check scale list

 lisztscale <- list(0.56789, newscale, matscale, matscale, newscale,
     0.98765, 0.98765, newscale)

 out <- temper(out, scale = lisztscale)

 my.coproposal.within <- out$coproposal[out$which, ]
 proposal.within <- out$proposal[out$which, ]
 my.z <- out$norm[out$which, ]
 my.proposal.within <- my.coproposal.within
 for (iiter in 1:nrow(my.z)) {
     my.i <- my.coproposal.within[iiter, 1]
     my.scale <- out$scale[[my.i]]
     if (is.matrix(my.scale)) {
         my.proposal.within[iiter, -1] <- my.coproposal.within[iiter, -1] +
             my.z[iiter, , drop = FALSE] %*% t(my.scale)
     } else {
         my.proposal.within[iiter, -1] <- my.coproposal.within[iiter, -1] +
             my.z[iiter, ] * my.scale
     }
 }
 all.equal(proposal.within, my.proposal.within)

 ### check outfun

 outfun <- function(state, icomp, ...) {
     stopifnot(is.matrix(state))
     stopifnot(is.numeric(state))
     nx <- ncol(betas)
     ncomp <- nrow(betas)
     stopifnot(ncol(state) == nx)
     stopifnot(nrow(state) == ncomp)
     stopifnot(1 <= icomp && icomp <= ncomp)
     foo <- state[icomp, ]
     bar <- foo^2
     return(c(foo, bar))
 }

 out <- temper(out, outfun = outfun, icomp = 4)

 before <- out$state
 after <- before
 after[- dim(after)[1], , ] <- before[-1, , ]
 after[dim(after)[1], , ] <- out$final
 outies <- apply(after, 1, outfun, icomp = 4)
 outies <- t(outies)

 foo <- outies[seq(1, niter) %% out$nspac == 0, ]
 foo <- array(as.vector(foo), dim = c(out$blen, out$nbatch, dim(foo)[2]))
 foo <- apply(foo, c(2, 3), mean)
 all.equal(foo, out$batch)

 library(mcmc)

 set.seed(42)

 h <- function(x) if (all(x >= 0) && sum(x) <= 1) return(1) else return(-Inf)
 out <- metrop(h, initial = rep(0, 5), nbatch = 100, blen = 17, nspac = 3,
     scale = 0.1)

 save.seed <- .Random.seed

 out1 <- metrop(out)
 out2 <- metrop(out1)
 out3 <- metrop(out, nbatch = 2 * out$nbatch)

 fred <- rbind(out1$batch, out2$batch)
 identical(fred, out3$batch)

 epsilon <- 1e-15

 library(mcmc)

 RNGkind("Marsaglia-Multicarry")
 set.seed(42)

 d <- 5

 logh <- function(x) {
     if (! is.numeric(x)) stop("x not numeric")
     if (length(x) != d) stop("length(x) != d")
     fred <- 1 - sum(x^2)
     if (fred > 0) return(log(fred)) else return(-Inf)
 }

 out.metro <- metrop(logh, rep(0, d), 1e3, scale = 0.01)
 out.metro$accept

 out.metro <- metrop(out.metro, scale = 0.1)
 out.metro$accept

 out.metro <- metrop(out.metro, scale = 0.5)
 out.metro$accept

 out.metro <- metrop(out.metro, scale = 0.4)
 out.metro$accept

 out.metro <- metrop(out.metro, nbatch = 1e2, debug = TRUE)

 all(out.metro$batch[- out.metro$nbatch, ] == out.metro$current[- 1, ])
 all(out.metro$current[1, ] == out.metro$initial)
 all(out.metro$batch[out.metro$nbatch, ] == out.metro$final)

 .Random.seed <- out.metro$initial.seed
 d <- ncol(out.metro$proposal)
 n <- nrow(out.metro$proposal)
 my.proposal <- matrix(NA, n, d)
 my.u <- double(n)
 ska <- out.metro$scale
 for (i in 1:n) {
     my.proposal[i, ] <- out.metro$current[i, ] + ska * rnorm(d)
     if (is.na(out.metro$u[i])) {
         my.u[i] <- NA
     } else {
         my.u[i] <- runif(1)
     }
 }
 max(abs(out.metro$proposal - my.proposal)) < epsilon
 all(is.na(out.metro$u) == is.na(my.u))
 all(out.metro$u[!is.na(out.metro$u)] == my.u[!is.na(my.u)])

 my.curr.log.green <- apply(out.metro$current, 1, logh)
 my.prop.log.green <- apply(out.metro$proposal, 1, logh)
 all(is.na(out.metro$u) == ((my.prop.log.green == -Inf) |
     (my.prop.log.green > my.curr.log.green)))
 foo <- my.prop.log.green - my.curr.log.green
 blurfle <- foo - out.metro$log.green
 blurfle[foo == -Inf & out.metro$log.green == -Inf] <- 0
 max(blurfle) < epsilon

 my.accept <- (my.prop.log.green > -Inf) & (is.na(my.u) | my.u < exp(foo))
 sum(my.accept) == round(n * out.metro$accept)

 my.path <- matrix(NA, n, d)
 my.path[my.accept, ] <- out.metro$proposal[my.accept, ]
 my.path[! my.accept, ] <- out.metro$current[! my.accept, ]

 all(my.path == out.metro$batch)

# new feature batching acceptance rates

 set.seed(42)

 library(mcmc)

 h <- function(x) if (all(x >= 0) && sum(x) <= 1) return(1) else return(-Inf)
 out <- metrop(h, rep(0, 5), nbatch = 100, blen = 100, scale = 0.1,
     debug = TRUE)

 all.equal(out$accept, mean(out$accept.batch))

 foo <- matrix(out$debug.accept, nrow = out$blen)
 bar <- colMeans(foo)
 all.equal(out$accept.batch, bar)

 options(digits = 4) # try to keep insanity of computer arithmetic under control

 out$accept
 t.test(out$accept.batch)$conf.int

 library(mcmc)

 set.seed(42)

 n <- 1e5
 rho <- 0.99

 x <- arima.sim(model = list(ar = rho), n = n)
 gamma <- acf(x, lag.max = 1999, type = "covariance",
     plot = FALSE)$acf
 k <- seq(along = gamma)
 Gamma <- gamma[k %% 2 == 1] + gamma[k %% 2 == 0]
 k <- min(seq(along = Gamma)[Gamma < 0])
 Gamma <- Gamma[1:k]
 Gamma[k] < 0
 Gamma[k] <- 0

 out <- .Call(mcmc:::C_initseq, x - mean(x))
 names(out)

 all.equal(gamma[1], out$gamma0)

 length(out$Gamma.pos) == length(Gamma)
 all.equal(out$Gamma.pos, Gamma)

 Gamma.dec <- cummin(Gamma)
 all.equal(out$Gamma.dec, Gamma.dec)
 
## IGNORE_RDIFF_BEGIN
library(Iso)
## IGNORE_RDIFF_END
 Gamma.con <- Gamma.dec[1] + cumsum(c(0, pava(diff(Gamma.dec))))
 all.equal(out$Gamma.con, Gamma.con)

 all.equal(0, min(out$Gamma.pos - out$Gamma.dec))
 max(diff(out$Gamma.dec)) < sqrt(.Machine$double.eps)

 all.equal(0, min(out$Gamma.dec - out$Gamma.con))
 min(diff(diff(out$Gamma.con))) > (- sqrt(.Machine$double.eps))

 all.equal(2 * sum(out$Gamma.pos) - out$gamma0, out$var.pos)
 all.equal(2 * sum(out$Gamma.dec) - out$gamma0, out$var.dec)
 all.equal(2 * sum(out$Gamma.con) - out$gamma0, out$var.con)

 rev(out$Gamma.pos)[1] == 0
 rev(out$Gamma.dec)[1] == 0
 all.equal(rev(out$Gamma.con)[1], 0)

 # test outfun (function)

 epsilon <- 1e-15

 library(mcmc)

 RNGkind("Marsaglia-Multicarry")
 set.seed(42)

 n <- 100
 rho <- 0.5
 beta0 <- 0.25
 beta1 <- 1
 beta2 <- 0.5

 x1 <- rnorm(n)
 x2 <- rho * x1 + sqrt(1 - rho^2) * rnorm(n)
 eta <- beta0 + beta1 * x1 + beta2 * x2
 p <- 1 / (1 + exp(- eta))
 y <- as.numeric(runif(n) < p)

 out <- glm(y ~ x1 + x2, family = binomial())

 logl <- function(beta) {
     if (length(beta) != 3) stop("length(beta) != 3")
     beta0 <- beta[1]
     beta1 <- beta[2]
     beta2 <- beta[3]
     eta <- beta0 + beta1 * x1 + beta2 * x2
     p <- exp(eta) / (1 + exp(eta))
     return(sum(log(p[y == 1])) + sum(log(1 - p[y == 0])))
 }

 out.metro <- metrop(logl, coefficients(out), 1e3, scale = 0.01)
 out.metro$accept

 out.metro <- metrop(out.metro, scale = 0.1)
 out.metro$accept

 out.metro <- metrop(out.metro, scale = 0.5)
 out.metro$accept

 apply(out.metro$batch, 2, mean)

 out.metro <- metrop(logl, as.numeric(coefficients(out)), 1e2,
     scale = 0.5, debug = TRUE, outfun = function(x) c(x, x^2))

 out.metro <- metrop(out.metro)
 out.metro$outfun
 dim(out.metro$batch)

 logl <- function(beta, x1, x2, y) {
     if (length(beta) != 3) stop("length(beta) != 3")
     beta0 <- beta[1]
     beta1 <- beta[2]
     beta2 <- beta[3]
     eta <- beta0 + beta1 * x1 + beta2 * x2
     p <- exp(eta) / (1 + exp(eta))
     return(sum(log(p[y == 1])) + sum(log(1 - p[y == 0])))
 }

 out.metro <- metrop(logl, as.numeric(coefficients(out)), 1e2,
     scale = 0.5, debug = TRUE, x1 = x1, x2 = x2, y = y)
 out.metro$lud
 out.metro <- metrop(out.metro, x1 = x1, x2 = x2, y = y)
 out.metro$lud

olbm <- function(x, batch.length, demean = TRUE) {
    x <- as.matrix(x)
    n <- nrow(x)
    p <- ncol(x)
    storage.mode(x) <- "double"
    if (batch.length > n) stop("batch.length must be <= nrow(x)")
    if (demean) {
    	mean <- apply(x, 2, mean)
    	no.calc.mean <- TRUE
    } else {
    	mean <- double(p)
    	no.calc.mean <- FALSE
    }
    out <- .C(C_olbm,
    	x=x,
    	n=as.integer(n),
    	p=as.integer(p),
    	batch.length=as.integer(batch.length),
    	mean=as.double(mean),
    	var=matrix(as.double(0), p, p),
    	no.calc.mean=as.logical(no.calc.mean))
    return(out$var)
}

library(mcmc)

.morph.unmorph <- mcmc:::.morph.unmorph

###########################################################################
# basic functionality check, can morph.metro run?  Can we change the
# transformation?
set.seed(42)
obj <- morph.metrop(function(x) dt(x, df=3, log=TRUE),
                    100, 100, morph=morph(b=3))
obj <- morph.metrop(obj, morph=morph(b=1))

obj <- morph.metrop(function(x) prod(dt(x, df=3, log=TRUE)),
                    rep(100, 3), 100, morph=morph(p=3, b=1))
obj <- morph.metrop(obj, morph=morph(r=1, p=3, b=1))

all.equal(class(obj), c("mcmc", "morph.metropolis"))

###########################################################################
# check .morph.unmorph
obj <- list(final=10)
outfun <- function(x) x
m <- morph(p=3)
obj <- .morph.unmorph(obj, m, outfun)
all.equal(class(obj), c("mcmc", "morph.metropolis"))
all.equal(sort(names(obj)),
          sort(c("final", "morph", "morph.final", "outfun")))
all.equal(c(obj$final, obj$morph.final), c(m$inverse(10), 10))
all.equal(obj$outfun, outfun)
all.equal(obj$morph, m)

 library(mcmc)

 set.seed(42)

 d <- 3
 witch.which <- 1 - (1 / 2)^(1 / d) * (1 / 4)^(seq(0, 5) / d)
 witch.which

 ncomp <- length(witch.which)

 neighbors <- matrix(FALSE, ncomp, ncomp)
 neighbors[row(neighbors) == col(neighbors) + 1] <- TRUE
 neighbors[row(neighbors) == col(neighbors) - 1] <- TRUE
 neighbors[row(neighbors) == col(neighbors) + 2] <- TRUE
 neighbors[row(neighbors) == col(neighbors) - 2] <- TRUE

 ludfun <- function(state) {
     stopifnot(is.numeric(state))
     stopifnot(length(state) == d + 1)
     icomp <- state[1]
     stopifnot(icomp == as.integer(icomp))
     stopifnot(1 <= icomp && icomp <= ncomp)
     theta <- state[-1]
     if (any(abs(theta) > 1.0)) return(-Inf)
     bnd <- witch.which[icomp]
     if(bnd >= 1.0)
         stop(sprintf("witch.which[%d] >= 1.0", icomp))
     if(bnd <= 0.0)
         stop(sprintf("witch.which[%d] <= 0.0", icomp))
     if (all(abs(theta) > bnd))
         return(- (d + 1) * log(2) - d * log(1 - bnd))
     return(- (d + 1) * log(2) - log1p(- (1 - bnd)^d))
 }

 initial <- c(1, rep(0, d))

 out <- temper(ludfun, initial = initial, neighbors = neighbors,
     nbatch = 50, blen = 13, nspac = 7, scale = 0.3456789)

 names(out)

 out$acceptx

 out$accepti

 colMeans(out$ibatch)

 ### check that have prob 1 / 2 for corners

 outfun <- function(state) {
     stopifnot(is.numeric(state))
     icomp <- state[1]
     stopifnot(icomp == as.integer(icomp))
     stopifnot(1 <= icomp && icomp <= length(witch.which))
     theta <- state[-1]
     foo <- all(abs(theta) > witch.which[icomp])
     bar <- rep(0, length(witch.which))
     baz <- rep(0, length(witch.which))
     bar[icomp] <- as.numeric(foo)
     baz[icomp] <- 1
     return(c(bar, baz))
 }

 out <- temper(out, blen = 103, outfun = outfun, debug = TRUE)

 eta.batch <- out$batch[ , seq(1, ncomp)]
 noo.batch <- out$batch[ , seq(ncomp + 1, ncomp + ncomp)]
 eta <- colMeans(eta.batch)
 noo <- colMeans(noo.batch)
 mu <- eta / noo
 eta
 noo
 mu

 eta.batch.rel <- sweep(eta.batch, 2, eta, "/")
 noo.batch.rel <- sweep(noo.batch, 2, noo, "/")
 mu.batch.rel <- eta.batch.rel - noo.batch.rel

 mu.mcse.rel <- apply(mu.batch.rel, 2, sd) / sqrt(out$nbatch)
 mu.mcse.rel

 foo <- cbind(mu, mu * mu.mcse.rel)
 colnames(foo) <- c("means", "MCSE")
 foo

 ### check decision about within-component or jump/swap

 identical(out$unif.which < 0.5, out$which)

 identical(out$which, out$proposal[ , 1] == out$state[ , 1])

 ### check hastings ratio calculated correctly

 n <- apply(neighbors, 1, sum)
 i <- out$state[ , 1]
 istar <- out$proposal[ , 1]
 foo <- apply(out$state, 1, ludfun)
 bar <- apply(out$proposal, 1, ludfun)
 my.log.hastings <- bar - foo - log(n[istar]) + log(n[i])
 all.equal(my.log.hastings, out$log.hastings)

 library(mcmc)

 x <- seq(0, 10, length = 10001)

 ### sub-exponentially light transformation

 b <- 0.5
 fsub <- morph(b = b)

 y <- unlist(Map(fsub$inverse, x))

 myfsub <- function(x) ifelse(x > 1 / b, exp(b * x) - exp(1) / 3,
     (x * b)^3 * exp(1) / 6 + x * b * exp(1) / 2)
 y2 <- myfsub(x)
 all.equal(y, y2, tolerance = 1e-14)

 z <- unlist(Map(fsub$transform, y))
 all.equal(z, x, tolerance = 1e-14)

 ### exponentially light transformation

 r <- 5 
 p <- 3
 fp3 <- morph(r = r)

 y <- unlist(Map(fp3$inverse, x))

 myfp3 <- function(x) ifelse(x < r, x, x + (x - r)^p)
 y2 <- myfp3(x)
 all.equal(y, y2, tolerance = 1e-14)

 z <- unlist(Map(fp3$transform, y))
 all.equal(z, x, tolerance = 1e-12)

 ### both together

 fboth <- morph(b = b, r = r)

 y <- unlist(Map(fboth$inverse, x))
 y2 <- myfsub(myfp3(x))
 all.equal(y, y2, tolerance = 1e-14)

 z <- unlist(Map(fboth$transform, y))
 all.equal(z, x, tolerance = 1e-12)

 ### exponentially light transformation with p != 3

 r <- 5 
 p <- 2.2
 fpo <- morph(r = r, p = p)

 y <- unlist(Map(fpo$inverse, x))

 myfpo <- function(x) ifelse(x < r, x, x + (x - r)^p)
 y2 <- myfpo(x)
 all.equal(y, y2, tolerance = 1e-14)

 z <- unlist(Map(fpo$transform, y))
 all.equal(z, x, tolerance = 1e-14)

library(mcmc)
isotropic <- mcmc:::isotropic
isotropic.logjacobian <- mcmc:::isotropic.logjacobian

# create identity test function
identity <- function(x) x
d.identity <- function(x) 1

# check that isotropic is length preserving for vectors of lengths 1--1000
all(sapply(1:1000, function(x) length(isotropic(identity)(rep(1, x))) == x))
    
# test that isotropic(identity) is an identity function
all.equal(isotropic(identity)(1:10), 1:10)
x <- seq(0, 1, length.out=200)
all.equal(isotropic(identity)(x), x)

# make sure that isotropic.logjacobian(identity, d.identity) is a 0 function
all.equal(isotropic.logjacobian(identity, d.identity)(1:10), 0)

# make sure that 0 as an input does not cause divide-by-zero errors
all.equal(isotropic(identity)(0), 0)
all.equal(isotropic(identity)(0 * 1:4), rep(0, 4))
all.equal(isotropic.logjacobian(identity, d.identity)(0), 0)
all.equal(isotropic.logjacobian(identity, d.identity)(0 * 1:4), 0)

# try isotropic with f(x) = x^2, then we should get 
# istropic(f)(x) := |x| * x
f <- function(x) x^2
all.equal(isotropic(f)(1), 1)
all.equal(isotropic(f)(c(1, 1)), sqrt(2) * c(1, 1))
all.equal(isotropic(f)(c(1, 0, 1)), sqrt(2) * c(1, 0, 1))

# make sure lazy-loading works properly.
g <- function(x) x^2
g.iso <- isotropic(g)
g <- function(x) x
all.equal(g.iso(2), 2*2)

morph.metrop <- function(obj, initial, nbatch, blen = 1,
    nspac = 1, scale = 1, outfun, debug = FALSE, morph,
    ...)
UseMethod("morph.metrop")

morph.metrop.morph.metropolis <- function(obj, initial, nbatch, blen = 1,
    nspac = 1, scale = 1, outfun, debug = FALSE, morph, ...) {
  if (missing(morph)) {
    morph <- obj$morph
    obj$final <- obj$morph.final
  } else {
    # if the transformation was changed, transform the last state from the
    # original space to be the initial state.
    obj$final <- morph$transform(obj$final)
  }

  if (missing(outfun)) outfun <- obj$outfun
  if (missing(blen)) blen <- obj$blen
  if (missing(nspac)) nspac <- obj$nspac
  if (missing(debug)) debug <- obj$debug
  if (missing(scale)) scale <- obj$scale
  
  morphed.obj <- metrop.metropolis(obj,
                                   nbatch=nbatch,
                                   blen=blen,
                                   nspac=nspac,
                                   scale=scale,
                                   outfun=morph$outfun(outfun),
                                   debug=debug,
                                   ...)
  
  unmorphed.obj <- .morph.unmorph(morphed.obj, morph, outfun)
  return(unmorphed.obj)
}

morph.metrop.function <- function(obj, initial, nbatch, blen = 1,
    nspac = 1, scale = 1, outfun, debug = FALSE, morph, ...) {

  if (missing(morph)) morph <- morph.identity()
  if (missing(outfun)) outfun <- NULL
  
  morphed.obj <- metrop.function(morph$lud(obj),
                                 initial=morph$transform(initial),
                                 nbatch=nbatch,
                                 blen=blen,
                                 scale=scale,
                                 outfun=morph$outfun(outfun),
                                 debug=debug,
                                 ...)
  
  unmorphed.obj <- .morph.unmorph(morphed.obj, morph, outfun)
  return(unmorphed.obj)
}

.morph.unmorph <- function(obj, morph, outfun) {
  obj$morph       <- morph
  obj$morph.final <- obj$final
  obj$final       <- morph$inverse(obj$final)
  obj$outfun      <- outfun
  class(obj) <- c("mcmc", "morph.metropolis")
  return(obj)
}

 library(mcmc)

 set.seed(42)

 data(foo)
 attach(foo)

 out <- glm(y ~ x1 + x2 + x3, family = binomial, x = TRUE)
 summary(out)

 modmat <- out$x

 models <- cbind(rep(0:1, each = 4), rep(rep(0:1, times = 2), each = 2),
               rep(0:1, times = 4))

 exes <- paste("x", 1:3, sep = "")
 models[nrow(models), ]
 beta.initial <- c(nrow(models), out$coefficients)

 neighbors <- matrix(FALSE, nrow(models), nrow(models))
 for (i in 1:nrow(neighbors)) {
     for (j in 1:ncol(neighbors)) {
         foo <- models[i, ]
         bar <- models[j, ]
         if (sum(foo != bar) == 1) neighbors[i, j] <- TRUE
     }
 }
 neighbors

 ludfun <- function(state, log.pseudo.prior, ...) {
     stopifnot(is.numeric(state))
     stopifnot(length(state) == ncol(models) + 2)
     icomp <- state[1]
     stopifnot(icomp == as.integer(icomp))
     stopifnot(1 <= icomp && icomp <= nrow(models))
     stopifnot(is.numeric(log.pseudo.prior))
     stopifnot(length(log.pseudo.prior) == nrow(models))
     beta <- state[-1]
     inies <- c(TRUE, as.logical(models[icomp, ]))
     beta.logl <- beta
     beta.logl[! inies] <- 0
     eta <- as.numeric(modmat %*% beta.logl)
     logp <- ifelse(eta < 0, eta - log1p(exp(eta)), - log1p(exp(- eta)))
     logq <- ifelse(eta < 0, - log1p(exp(eta)), - eta - log1p(exp(- eta)))
     logl <- sum(logp[y == 1]) + sum(logq[y == 0])
     val <- logl - sum(beta^2) / 2 + log.pseudo.prior[icomp]
     return(val)
 }

 qux <- c(25.01, 5.875, 9.028, 0.6959, 11.73,  2.367, 5.864, 0.0)

 out <- temper(ludfun, initial = beta.initial, neighbors = neighbors,
     nbatch = 25, blen = 20, nspac = 5, scale = 0.56789, debug = TRUE,
     log.pseudo.prior = qux)

 names(out)

 apply(out$ibatch, 2, mean)

 ### check decision about within-component or jump/swap

 identical(out$unif.which < 0.5, out$which)

 identical(out$which, out$proposal[ , 1] == out$state[ , 1])

 ### check hastings ratio calculated correctly

 foo <- apply(out$state, 1, ludfun, log.pseudo.prior = qux)
 bar <- apply(out$proposal, 1, ludfun, log.pseudo.prior = qux)
 all.equal(bar - foo, out$log.hastings)

 ### check hastings rejection decided correctly

 identical(out$log.hastings >= 0, is.na(out$unif.hastings))
 all(out$log.hastings < 0 | out$acceptd)
 identical(out$acceptd,
     out$log.hastings >= 0 | out$unif.hastings < exp(out$log.hastings))

 ### check acceptance carried out or not (according to decision) correctly

 before <- out$state
 after <- before
 after[- dim(after)[1], ] <- before[-1, ]
 after[dim(after)[1], ] <- out$final
 my.after <- before
 my.after[out$acceptd, ] <- out$proposal[out$acceptd, ]
 identical(after, my.after)

 ### check within-component proposal

 my.coproposal.within <- out$state[out$which, ]
 proposal.within <- out$proposal[out$which, ]
 my.z <- out$norm[out$which, ]
 my.proposal.within <- my.coproposal.within
 my.proposal.within[ , -1] <- my.coproposal.within[ , -1] + out$scale * my.z
 all.equal(proposal.within, my.proposal.within)

 ### check swap proposal

 coproposal.swap <- out$state[! out$which, ]
 proposal.swap <- out$proposal[! out$which, ]
 unif.choose.swap <- out$unif.choose[! out$which]
 my.i <- coproposal.swap[ , 1]
 nneighbors <- apply(out$neighbors, 1, sum)
 my.nneighbors <- nneighbors[my.i]
 my.k <- floor(my.nneighbors * unif.choose.swap) + 1
 my.j <- my.k
 foo <- seq(1, ncol(out$neighbors))
 for (i in seq(along = my.j)) {
     my.j[i] <- (foo[out$neighbors[my.i[i], ]])[my.k[i]]
 }
 identical(coproposal.swap[ , 1], my.i)
 identical(proposal.swap[ , 1], my.j)

 ### check standard normal and uniform random numbers are as purported

 save.Random.seed <- .Random.seed
 .Random.seed <- out$initial.seed

 nx <- length(out$initial) - 1
 niter <- out$nbatch * out$blen * out$nspac
 my.norm <- matrix(NA, nrow = nrow(out$norm), ncol = ncol(out$norm))
 my.unif.which <- rep(NA, niter)
 my.unif.hastings <- rep(NA, niter)
 my.unif.choose <- rep(NA, niter)
 for (iiter in 1:niter) {
     my.unif.which[iiter] <- runif(1)
     if (out$which[iiter]) {
         my.norm[iiter, ] <- rnorm(nx)
         if (out$log.hastings[iiter] < 0) my.unif.hastings[iiter] <- runif(1) 
     } else {
         my.unif.choose[iiter] <- runif(1)
         if (out$log.hastings[iiter] < 0) my.unif.hastings[iiter] <- runif(1) 
     }
 }
 identical(my.norm, out$norm)
 identical(my.unif.which, out$unif.which)
 identical(my.unif.hastings, out$unif.hastings)
 identical(my.unif.choose, out$unif.choose)

 .Random.seed <- save.Random.seed

 ### check batch means

 my.xstate <- after[ , -1]
 foo <- my.xstate[seq(1, niter) %% out$nspac == 0, ]
 foo <- array(as.vector(foo), dim = c(out$blen, out$nbatch, dim(foo)[2]))
 foo <- apply(foo, c(2, 3), mean)
 all.equal(foo, out$batch)

 ### check ibatch means

 my.istate <- after[ , 1]
 my.istate.matrix <- matrix(0, length(my.istate), nrow(models))
 for (i in 1:nrow(my.istate.matrix))
     my.istate.matrix[i, my.istate[i]] <- 1
 foo <- my.istate.matrix[seq(1, niter) %% out$nspac == 0, ]
 foo <- array(as.vector(foo), dim = c(out$blen, out$nbatch, dim(foo)[2]))
 foo <- apply(foo, c(2, 3), mean)
 all.equal(foo, out$ibatch)

 ### check acceptance rates

 nmodel <- nrow(out$neighbors)

 accept.within <- out$acceptd[out$which]
 my.i.within <- out$state[out$which, 1]
 my.i.within.accept <- my.i.within[accept.within]
 my.acceptx.numer <- tabulate(my.i.within.accept, nbins = nmodel)
 my.acceptx.denom <- tabulate(my.i.within, nbins = nmodel)
 my.acceptx <- my.acceptx.numer / my.acceptx.denom
 identical(my.acceptx, out$acceptx)

 accept.swap <- out$acceptd[! out$which]
 my.i.swap <- out$state[! out$which, 1]
 my.j.swap <- out$proposal[! out$which, 1]
 my.accepti <- matrix(NA, nmodel, nmodel)
 for (i in 1:nmodel) {
     for (j in 1:nmodel) {
         if (out$neighbors[i, j]) {
             my.accepti[i, j] <-
                 mean(accept.swap[my.i.swap == i & my.j.swap == j])
         }
     }
 }
 identical(my.accepti, out$accepti)

 ### check scale vector

 nx <- ncol(models) + 1
 newscale <- rnorm(nx, 0.5, 0.1)

 out <- temper(out, scale = newscale, log.pseudo.prior = qux)

 my.coproposal.within <- out$state[out$which, ]
 proposal.within <- out$proposal[out$which, ]
 my.z <- out$norm[out$which, ]
 my.proposal.within <- my.coproposal.within
 my.proposal.within[ , -1] <- my.coproposal.within[ , -1] +
     sweep(my.z, 2, out$scale, "*")
 all.equal(proposal.within, my.proposal.within)

 ### check scale matrix

 matscale <- matrix(rnorm(nx * nx, 0.0, 0.1), nx, nx)
 diag(matscale) <- 0.56789

 out <- temper(out, scale = matscale, log.pseudo.prior = qux)

 my.coproposal.within <- out$state[out$which, ]
 proposal.within <- out$proposal[out$which, ]
 my.z <- out$norm[out$which, ]
 my.proposal.within <- my.coproposal.within
 my.proposal.within[ , -1] <- my.coproposal.within[ , -1] +
     my.z %*% t(out$scale)
 all.equal(proposal.within, my.proposal.within)

 ### check scale list

 lisztscale <- list(0.56789, newscale, matscale, matscale, newscale,
     0.98765, 0.98765, newscale)

 out <- temper(out, scale = lisztscale, log.pseudo.prior = qux)

 my.coproposal.within <- out$state[out$which, ]
 proposal.within <- out$proposal[out$which, ]
 my.z <- out$norm[out$which, ]
 my.proposal.within <- my.coproposal.within
 for (iiter in 1:nrow(my.z)) {
     my.i <- my.coproposal.within[iiter, 1]
     my.scale <- out$scale[[my.i]]
     if (is.matrix(my.scale)) {
         my.proposal.within[iiter, -1] <- my.coproposal.within[iiter, -1] +
             my.z[iiter, , drop = FALSE] %*% t(my.scale)
     } else {
         my.proposal.within[iiter, -1] <- my.coproposal.within[iiter, -1] +
             my.z[iiter, ] * my.scale
     }
 }
 all.equal(proposal.within, my.proposal.within)

 ### check outfun

 outfun <- function(state, icomp) {
     stopifnot(is.matrix(state))
     stopifnot(is.numeric(state))
     nx <- ncol(initial)
     ncomp <- nrow(initial)
     stopifnot(ncol(state) == nx)
     stopifnot(nrow(state) == ncomp)
     stopifnot(1 <= icomp & icomp <= ncomp)
     foo <- state[icomp, ]
     bar <- foo^2
     return(c(foo, bar))
 }

 ncomp <- nrow(models)
 nx <- length(beta.initial) - 1

 outfun <- function(state, icomp, ...) {
     stopifnot(is.numeric(state))
     stopifnot(length(state) == nx + 1)
     istate <- state[1]
     stopifnot(istate == as.integer(istate))
     stopifnot(1 <= istate && istate <= ncomp)
     stopifnot(1 <= icomp && icomp <= ncomp)
     if (istate == icomp) {
         foo <- state[-1]
     } else {
         foo <- rep(0, nx)
     }
     bar <- foo^2
     return(c(foo, bar))
 }

 out <- temper(ludfun, initial = out$final, neighbors = neighbors,
     nbatch = 25, blen = 20, nspac = 5, scale = 0.56789, debug = TRUE,
     outfun = outfun, log.pseudo.prior = qux, icomp = 4)

 before <- out$state
 after <- before
 after[- dim(after)[1], ] <- before[-1, ]
 after[dim(after)[1], ] <- out$final
 outies <- apply(after, 1, outfun, icomp = 4)
 outies <- t(outies)

 foo <- outies[seq(1, niter) %% out$nspac == 0, ]
 foo <- array(as.vector(foo), dim = c(out$blen, out$nbatch, dim(foo)[2]))
 foo <- apply(foo, c(2, 3), mean)
 all.equal(foo, out$batch)

 if ((! exists("DEBUG")) || (! identical(DEBUG, TRUE))) DEBUG <- FALSE

 library(mcmc)

 options(digits=4) # avoid rounding differences

 set.seed(42)

 save.initial.seed <- .Random.seed

 d <- 3
 witch.which <- 1 - (1 / 2)^(1 / d) * (1 / 4)^(seq(0, 5) / d)
 witch.which

 ncomp <- length(witch.which)

 neighbors <- matrix(FALSE, ncomp, ncomp)
 neighbors[row(neighbors) == col(neighbors) + 1] <- TRUE
 neighbors[row(neighbors) == col(neighbors) - 1] <- TRUE
 neighbors[row(neighbors) == col(neighbors) + 2] <- TRUE
 neighbors[row(neighbors) == col(neighbors) - 2] <- TRUE

 ludfun <- function(state) {
     stopifnot(is.numeric(state))
     stopifnot(length(state) == d + 1)
     icomp <- state[1]
     stopifnot(icomp == as.integer(icomp))
     stopifnot(1 <= icomp && icomp <= ncomp)
     theta <- state[-1]
     if (any(abs(theta) > 1.0)) return(-Inf)
     bnd <- witch.which[icomp]
     if(bnd >= 1.0)
         stop(sprintf("witch.which[%d] >= 1.0", icomp))
     if(bnd <= 0.0)
         stop(sprintf("witch.which[%d] <= 0.0", icomp))
     if (all(abs(theta) > bnd))
         return(- (d + 1) * log(2) - d * log(1 - bnd))
     return(- (d + 1) * log(2) - log1p(- (1 - bnd)^d))
 }

 thetas <- matrix(0, ncomp, d)
 out <- temper(ludfun, initial = thetas, neighbors = neighbors, nbatch = 50,
     blen = 13, nspac = 7, scale = 0.3456789, parallel = TRUE, debug = DEBUG)

 names(out)

 out$acceptx

 out$accepti

 ### check that have prob 1 / 2 for corners

 outfun <- function(state) {
     stopifnot(is.matrix(state))
     ncomp <- nrow(state)
     d <- ncol(state)
     foo <- sweep(abs(state), 1, witch.which)
     bar <- apply(foo > 0, 1, all) 
     return(as.numeric(bar))
 }

 out2 <- temper(out, outfun = outfun)

 colMeans(out2$batch)
 apply(out2$batch, 2, sd) / sqrt(out$nbatch)

 ### try again

 out3 <- temper(out2, blen = 103)

 foo <- cbind(colMeans(out3$batch),
     apply(out3$batch, 2, sd) / sqrt(out$nbatch))
 colnames(foo) <- c("means", "MCSE")
 foo

temper <- function(obj, initial, neighbors, nbatch, blen = 1,
    nspac = 1, scale = 1, outfun, debug = FALSE, parallel = FALSE, ...)
UseMethod("temper")

temper.tempering <- function(obj, initial, neighbors, nbatch, blen = 1,
    nspac = 1, scale = 1, outfun, debug = FALSE, parallel = FALSE, ...)
{
    # like metrop, ignore initial argument
    initial <- obj$final
    # makes no (at least little?) sense to change neighbor structure
    neighbors <- obj$neighbors
    if (missing(nbatch)) nbatch <- obj$nbatch
    if (missing(blen)) blen <- obj$blen
    if (missing(nspac)) nspac <- obj$nspac
    if (missing(scale)) scale <- obj$scale
    if (missing(debug)) debug <- obj$debug
    # makes no sense to change from parallel to serial or vice versa
    # size and shape of state wouldn't even be the same
    parallel <- obj$parallel
    assign(".Random.seed", obj$final.seed, .GlobalEnv)
    if (missing(outfun)) {
        if (is.null(obj$outfun)) {
            temper.function(obj$lud, initial, neighbors, nbatch, blen,
                nspac, scale, debug = debug, parallel = parallel, ...)
        } else {
            temper.function(obj$lud, initial, neighbors, nbatch, blen,
                nspac, scale, obj$outfun, debug = debug, parallel = parallel,
                ...)
        }
    } else {
        temper.function(obj$lud, initial, neighbors, nbatch, blen,
            nspac, scale, outfun, debug = debug, parallel = parallel, ...)
    }
}

temper.function <- function(obj, initial, neighbors, nbatch, blen = 1,
    nspac = 1, scale = 1, outfun, debug = FALSE, parallel = FALSE, ...)
{
    if (! exists(".Random.seed")) runif(1)
    saveseed <- .Random.seed
    func1 <- function(state) obj(state, ...)
    env1 <- environment(fun = func1)
    if (missing(outfun)) {
        func2 <- NULL
        env2 <- NULL
        outfun <- NULL
    } else if (is.function(outfun)) {
        func2 <- function(state) outfun(state, ...)
        env2 <- environment(fun = func2)
    }

    stopifnot(is.numeric(initial))
    storage.mode(initial) <- "double"

    if (is.list(scale)) {
        for (i in 1:length(scale)) {
            stopifnot(is.numeric(scale[[i]]))
            storage.mode(scale[[i]]) <- "double"
        }
    } else {
        stopifnot(is.numeric(scale))
        storage.mode(scale) <- "double"
    }

    out.time <- system.time(
    out <- .Call(C_temper, func1, initial, neighbors, nbatch, blen, nspac,
        scale, func2, debug, parallel, env1, env2)
    )
    result <- structure(c(list(lud = obj,
        neighbors = neighbors, nbatch = nbatch, blen = blen, nspac = nspac,
        scale = scale, outfun = outfun, debug = debug, parallel = parallel,
        initial.seed = saveseed, final.seed = .Random.seed, time = out.time),
        out), class = c("mcmc", "tempering"))
    return(result)
}

\documentclass[11pt]{article}

\usepackage{amsmath}
\usepackage{amsfonts}
\usepackage{indentfirst}
\usepackage{natbib}
\usepackage{url}
\usepackage[utf8]{inputenc}

\newcommand{\real}{\mathbb{R}}

\DeclareMathOperator{\prior}{pri}
\DeclareMathOperator{\posterior}{post}
\DeclareMathOperator{\indicator}{ind}

\newcommand{\fatdot}{\,\cdot\,}

% \VignetteIndexEntry{Bayes Factors via Serial Tempering}

\begin{document}

\title{Bayes Factors via Serial Tempering}
\author{Charles J. Geyer}
\maketitle

<<foo,include=FALSE,echo=FALSE>>=
options(keep.source = TRUE, width = 65)
@

\section{Introduction}

\subsection{Bayes Factors} \label{sec:bayes-factors}

Let $\mathcal{M}$ be a finite or countable set of models (here we only
deal with finite $\mathcal{M}$ but Bayes factors make sense for countable
$\mathcal{M}$).  For each model
$m \in \mathcal{M}$ we have the prior probability of the model $\prior(m)$.
It does not matter if this prior on models is unnormalized.

Each model $m$ has a parameter space $\Theta_m$ and a prior
$$
   g(\theta \mid m), \qquad \theta \in \Theta_m
$$
The spaces $\Theta_m$ can and usually do have different dimensions.  That's
the point.  These within model priors must be normalized proper priors.
The calculations to follow make no sense if these priors are unnormalized
or improper.

Each model $m$ has a data distribution
$$
   f(y \mid \theta, m)
$$
and the observed data $y$ may be either discrete or continuous
(it makes no difference to
the Bayesian who treats $y$ as fixed after it is observed and treats
only $\theta$ and $m$ as random).

The unnormalized posterior for everything
(for models and parameters within models)
is
$$
   f(y \mid \theta, m) g(\theta \mid m) \prior(m)
$$
To obtain the conditional distribution of $y$ given $m$, we must integrate
out the nuisance parameter $\theta$
\begin{align*}
   q(y \mid m)
   & =
   \int_{\Theta_m} f(y \mid \theta, m) g(\theta \mid m) \prior(m) \, d \theta
   \\
   & =
   \prior(m) \int_{\Theta_m} f(y \mid \theta, m) g(\theta \mid m) \, d \theta
\end{align*}
These are the unnormalized posterior probabilities of the models.  The
normalized posterior probabilities are
$$
   \posterior(m \mid y)
   =
   \frac{ q(y \mid m) }{ \sum_{m \in \mathcal{M}} q(y \mid m) }
$$

It is considered useful to define
$$
   b(y \mid m)
   =
   \int_{\Theta_m} f(y \mid \theta, m) g(\theta \mid m) \, d \theta
$$
so
$$
   q(y \mid m) = b(y \mid m) \prior(m)
$$
Then the ratio of posterior probabilities of models $m_1$ and $m_2$ is
$$
   \frac{\posterior(m_1 \mid y)}{\posterior(m_2 \mid y)}
   =
   \frac{q(y \mid m_1)}{q(y \mid m_2)}
   =
   \frac{b(y \mid m_1)}{b(y \mid m_2)}
   \cdot
   \frac{\prior(m_1)}{\prior(m_2)}
$$
This ratio is called the \emph{posterior odds} of the models (a ratio of
probabilities is called an \emph{odds}) of these models.

The \emph{prior odds} is
$$
   \frac{\prior(m_1)}{\prior(m_2)}
$$

The term we have not yet named in
$$
   \frac{\posterior(m_1 \mid y)}{\posterior(m_2 \mid y)}
   =
   \frac{b(y \mid m_1)}{b(y \mid m_2)}
   \cdot
   \frac{\prior(m_1)}{\prior(m_2)}
$$
is called the \emph{Bayes factor}
\begin{equation} \label{eq:factor}
   \frac{b(y \mid m_1)}{b(y \mid m_2)}
\end{equation}
the ratio of posterior odds to prior odds.

The prior odds tells how the prior compares the probability of the models.
The Bayes factor tells us how the data shifts that comparison going from
prior to posterior via Bayes rule.
Bayes factors are the primary tool Bayesians use for model comparison,
the competitor for frequentist $P$-values in frequentist hypothesis
tests of model comparison.

Note that our clumsy multiple letter notation for priors and posteriors
$\prior(m)$ and $\posterior(m \mid y)$ does not matter because neither
is involved in the actual calculation of Bayes factors \eqref{eq:factor}.
Priors and posteriors are involved in motivating Bayes factors but not in
calculating them.

\subsection{Tempering} \label{sec:temper}

Simulated tempering \citep{marinari-parisi,geyer-thompson} is a method of
Markov chain Monte Carlo (MCMC) simulation of many distributions at once.
It was originally invented with the primary aim of speeding up MCMC
convergence, but was also recognized to be useful for sampling multiple
distributions \citep{geyer-thompson}.  In the latter role it is sometimes
referred to as ``umbrella sampling'' which is a term coined
by \citet{torrie-valleau} for sampling multiple distributions via MCMC.

We have a finite set of unnormalized distributions we want to sample,
all related in some way.  The R function \texttt{temper}
in the CRAN package \texttt{mcmc}
requires all to have continuous distributions for random vectors of the same
dimension (all distributions have the same domain $\real^p$).
Let $h_i$, $i \in \mathcal{I}$ denote the unnormalized densities of
these distributions.  Simulated tempering (called ``serial tempering'' by
the \texttt{temper} function to distinguish from a related scheme not used
in this document called ``parallel tempering'' and in either case abbreviated
ST) runs a Markov chain whose
state is a pair $(i, x)$ where $i \in \mathcal{I}$ and $x \in \real^p$.

The unnormalized density of stationary distribution of the ST chain is
\begin{equation} \label{eq:st-joint}
   h(i, x) = h_i(x) c_i
\end{equation}
where the $c_i$ are arbitrary constants chosen by the user (more on this later).

The equilibrium distribution of the ST state $(I, X)$ --- both bits random ---
is such that conditional distribution of $X$ given $I = i$ is the distribution
with unnormalized density $h_i$.  This is obvious from $h(i, x)$ being the
unnormalized conditional density --- the same function thought of as
a function of both variables is the unnormalized joint density and thought
of as a function of just one of the variables is an unnormalized conditional
density --- and $h(i, x)$ thought of as a function of $x$ for fixed $i$ being
proportional to $h_i$.  The equilibrium unnormalized marginal distribution
of $I$ is
\begin{equation} \label{eq:margin}
   \int h(i, x) \, d x = c_i \int h_i(x) \, d x = c_i d_i
\end{equation}
where
$$
   d_i = \int h_i(x) \, d x
$$
is the normalizing constant for $h_i$, that is, $h_i / d_i$ is a normalized
distribution.

It is clear from \eqref{eq:margin} being the unnormalized marginal distribution
that in order for the marginal distribution to be uniform we must choose the
tuning constants $c_i$ to be proportional to $1 / d_i$.  It is not important
that the marginal distribution be exactly uniform, but unless it is
approximately uniform, the sampler will not visit each distribution frequently.
Thus we do need to have the $c_i$ to be approximately proportional to $1 / d_i$.
This is accomplished by trial and
error (one example is done in this document) and is easy for easy problems
and hard for hard problems \citep[have much to say about adjusting
the $c_i$]{geyer-thompson}.  For the rest of this section we will assume
the tuning constants $c_i$ have been so adjusted:
we do not have the $c_i$ exactly proportional to $1 / d_i$ but do have
them approximately proportional to $1 / d_i$.

\subsection{Tempering and Bayes Factors}

Bayes factors are very important in Bayesian inference and many methods have
been invented to calculate them.  No method except the one described here
using ST is anywhere near as accurate and straightforward.  Thus no competitors
will be discussed.

In using ST for Bayes factors we identify the index set $\mathcal{I}$ with
the model set $\mathcal{M}$ and use the integers 1, $\ldots$, $k$ for both.
We would like to identify the within model parameter vector $\theta$ with
the vector $x$ that is the continuous part of the state of the ST Markov
chain, but cannot because the dimension of $\theta$ depends on $m$ and this
is not allowed.  Thus we have to do something a bit more complicated.  We
``pad'' $\theta$ so that it always has the same dimension, doing so in
a way that does not interfere with the Bayes factor calculation.  Write
$\theta = (\theta_{\text{actual}}, \theta_{\text{pad}})$, the dimension
of both parts depending on the model $m$.  Then we insist on the following
conditions:
$$
   f(y \mid \theta, m) = f(y \mid \theta_{\text{actual}}, m)
$$
so the data distribution does not depend on the ``padding'' and
$$
   g(\theta \mid m) = g_{\text{actual}}(\theta_{\text{actual}} \mid m)
   \cdot g_{\text{pad}}(\theta_{\text{pad}} \mid m)
$$
so the two parts are \emph{a priori} independent and both parts of the prior
are normalized proper priors.  This assures that
\begin{equation} \label{eq:unnormalized-bayes-factors}
\begin{split}
   b(y \mid m)
   & =
   \int_{\Theta_m} f(y \mid \theta, m) g(\theta \mid m) \, d \theta
   \\
   & =
   \iint f(y \mid \theta_{\text{actual}}, m)
   g_{\text{actual}}(\theta_{\text{actual}} \mid m)
   g_{\text{pad}}(\theta_{\text{pad}} \mid m)
   \, d \theta_{\text{actual}}
   \, d \theta_{\text{pad}}
   \\
   & =
   \int_{\Theta_m} f(y \mid \theta_{\text{actual}}, m)
   g_{\text{actual}}(\theta_{\text{actual}} \mid m)
   \, d \theta_{\text{actual}}
\end{split}
\end{equation}
so the calculation of the unnormalized Bayes factors is the same whether
or not we ``pad'' $\theta$, and we may then take
\begin{align*}
   h_m(\theta)
   & = 
   f(y \mid \theta, m) g(\theta \mid m)
   \\
   & =
   f(y \mid \theta_{\text{actual}}, m)
   g_{\text{actual}}(\theta_{\text{actual}} \mid m)
   g_{\text{pad}}(\theta_{\text{pad}} \mid m)
\end{align*}
to be the unnormalized densities for the component distributions of the ST
chain, in which case the unnormalized Bayes factors are proportional to the
normalizing constants $d_i$ in Section~\ref{sec:temper}.

\subsection{Tempering and Normalizing Constants}

Let $d$ be the normalizing constant for the joint equilibrium distribution
of the ST chain \eqref{eq:st-joint}.  When we are running the ST chain we know
neither $d$ nor the $d_i$ but we do know the $c_i$, which are constants we
have chosen based on the results of previous runs but are fixed known numbers
for the current run.  Let $(I_t, X_t)$, $t = 1$, 2, $\ldots$ be the sample
path of the ST chain.  Recall that (somewhat annoyingly) we are using the
notation $(i, x)$ for the state vector of a general ST chain and the notation
$(m, \theta)$ for ST chains used to calculate Bayes factors, identifying
$i = m$ and $x = \theta$.

Let $\indicator(\fatdot)$ denote the function that maps logical values to
numerical values, false to zero and true to one.  Normalizing constants are
estimated by averaging the time spent in each model
\begin{equation} \label{eq:st-estimates}
   \hat{\delta}_n(m) = \frac{1}{n} \sum_{t = 1}^n \indicator(I_t = m)
\end{equation}
For the purposes of approximating Bayes factors the $X_t$ are ignored.
The $X_t$ may be useful for other purposes, such as
Bayesian model averaging \citep*{bma}, but this is not discussed here.

The Monte Carlo approximations \eqref{eq:st-estimates} converge
to their expected values under the equilibrium distribution
\begin{equation} \label{eq:st-expectations}
   E\{ \indicator(I_t = m) \}
   =
   \int \frac{h(m, x)}{d} \, d x
   =
   \frac{c_m d_m}{d}
   =
   \delta(m)
\end{equation}
We want to estimate the unnormalized Bayes factors
\eqref{eq:unnormalized-bayes-factors}, which are in this context proportional
to the $d_m$.  The $c_m$ are known, $d$ is unknown but does not matter since
we only need to estimate the $d_m = b(m \mid y)$ up to an overall unknown
constant of proportionality, which cancels out of Bayes factors
\eqref{eq:factor}.

Note that our discussion here applies unchanged to the general problem of
estimating normalizing constants up to an unknown constant of proportionality,
which has applications other than Bayes factors, for example, missing data
maximum likelihood \citep{thompson-guo,geyer,sung-geyer}.
The ST method approximates normalizing constants up to an overall constant of
proportionality with high accuracy regardless of how large or small they are
(whether they are $10^{100}$ or $10^{-100}$), and no other method that does
not use essentially the same idea can do this.

The key is what seems at first sight to be a weakness of ST, the need to
adjust the tuning constants $c_i$ by trial and error.  In this context the
weakness is actually a strength: the adjusted $c_i$ contain most of the
information about the size of the normalizing constants $d_i$ and the
Monte Carlo averages \eqref{eq:st-estimates} add only the finishing touch.
Thus multiple runs of the ST chain with different choices of the $c_i$ used
in each run are needed (the ``trial and error''), but the information from
all are incorporated in the final run used for final approximation of the
normalizing constants (Bayes factors).  It is perhaps surprising that the
Monte Carlo error approximation is trivial.  In the context of the last run
of the ST chain the $c_i$ are known constants and contribute no error.
The Monte Carlo error of the averages \eqref{eq:st-estimates} is
straightforwardly estimated by batch means or competing methods.

\citet{geyer-thompson} note that the $c_i$ enter formally like a prior:
one can think of $h_i(x) c_i$ as likelihood times prior.  But one should
not think of the $c_i$ as representing prior information, informative,
non-informative, or in between.  The $c_i$ are adjusted to make the ST
distribution sample all the models $h_i$, and that is the only criterion
for the adjustment.  For this reason \citet{geyer-thompson} call the
$c_i$ the \emph{pseudoprior}.  This is a special case of a general principle
of MCMC.  When doing MCMC one should forget the statistical motivation
(in this case Bayes factors).  One should set up a Markov chain that does
a good job of simulating the required equilibrium distribution, whatever
it is.  Thinking about the statistical motivation of the equilibrium does
not help and can hurt (if one thinks of the pseudoprior as an actual prior,
one may be tempted to adjust it to represent prior information).

\section{R Package MCMC}

We use the R statistical computing environment \citep{rcore} in our analysis.
It is free software and can be obtained from
\url{http://cran.r-project.org}.  Precompiled binaries
are available for Windows, Macintosh, and popular Linux distributions.
We use the contributed package \verb@mcmc@ \citep{mcmc-R-package}
If R has been installed, but this package has
not yet been installed, do
\begin{verbatim}
install.packages("mcmc")
\end{verbatim}
from the R command line
(or do the equivalent using the GUI menus if on Apple Macintosh
or Microsoft Windows).  This may require root or administrator privileges.

Assuming the \verb@mcmc@ package has been installed, we load it
<<library>>=
library(mcmc)
@
<<baz,include=FALSE,echo=FALSE>>=
baz <- library(help = "mcmc")
baz <- baz$info[[1]]
baz <- baz[grep("Version", baz)]
baz <- sub("^Version: *", "", baz)
bazzer <- paste(R.version$major, R.version$minor, sep = ".")
@
The version of the package used to make this document
is \Sexpr{baz} (which is available on CRAN).
The version of R used to make this document is \Sexpr{bazzer}.

We also set the random number generator seed so that the results are
reproducible.
<<set-seed>>=
set.seed(42)
@
To get different results, change the setting or don't set the seed at all.

\section{Logistic Regression Example}

We use the same logistic regression example used in the \texttt{mcmc}
package vignette for the \texttt{metrop} function (file \texttt{demo.pdf}.
Simulated data for the problem are in the data set \verb@logit@.
There are five variables in the data set, the response \verb@y@
and four predictors, \verb@x1@, \verb@x2@, \verb@x3@, and \verb@x4@.

A frequentist analysis for the problem is done by the following R statements
<<frequentist>>=
data(logit)
out <- glm(y ~ x1 + x2 + x3 + x4, data = logit,
    family = binomial, x = TRUE)
summary(out)
@

But this example isn't about frequentist analysis, we want a Bayesian
analysis.  For our Bayesian analysis we assume the same data model as the
frequentist, and we assume the prior distribution of the five parameters
(the regression coefficients) makes them independent and identically
normally distributed with mean 0 and standard deviation 2.

Moreover, we wish to calculate Bayes factors for the $16 = 2^4$ possible
submodels that include or exclude each of the
predictors, \verb@x1@, \verb@x2@, \verb@x3@, and \verb@x4@.

\subsection{Setup}

We set up a matrix that indicates these models.
<<models>>=
varnam <- names(coefficients(out))
varnam <- varnam[varnam != "(Intercept)"]
nvar <- length(varnam)

models <- NULL
foo <- seq(0, 2^nvar - 1) 
for (i in 1:nvar) {
    bar <- foo %/% 2^(i - 1)
    bar <- bar %% 2
    models <- cbind(bar, models, deparse.level = 0)
}
colnames(models) <- varnam
models
@
In each row, 1 indicates the predictor is in the model and 0 indicates it is
out.

The function \texttt{temper} in the \text{mcmc} package that does tempering
requires a notion of neighbors among models.  It attempts jumps only between
neighboring models.  Here we choose models to be neighbors if they differ
only by one predictor.
<<neighbor>>=
neighbors <- matrix(FALSE, nrow(models), nrow(models))
for (i in 1:nrow(neighbors)) {
    for (j in 1:ncol(neighbors)) {
        foo <- models[i, ]
        bar <- models[j, ]
        if (sum(foo != bar) == 1) neighbors[i, j] <- TRUE
    }
}
@

Now we specify the equilibrium distribution of the ST chain.  Its state vector
is $(i, x)$ or $(m, \theta)$ in our alternative notations, where $i$ is an
integer between $1$ and \verb@nrow(models)@ = \Sexpr{nrow(models)} and
$\theta$ is the parameter vector ``padded'' to always be the same length,
so we take it to be the length of the parameter vector of the full model
which is \verb@length(out$coefficients)@ or \verb@ncol(models) + 1@ which makes
the length of the state of the ST chain \verb@ncol(models) + 2@.
We take the within model priors for the ``padded'' components of the parameter
vector to be the same as those for the ``actual'' components, normal with
mean 0 and standard deviation 2 for all cases.
As is seen in \eqref{eq:unnormalized-bayes-factors} the priors for the
``padded'' components (parameters not in the model for the current state)
do not matter because they drop out of the Bayes factor calculation.
The choice does not matter much for this toy example.
See the discussion section for more on this issue.
It is important that we use normalized log priors,
the term \verb@dnorm(beta, 0, 2, log = TRUE)@ in the function, unlike
when we are simulating only one model as in the \texttt{mcmc} package vignette
where it would be o.~k.\ to use unnormalized log priors \verb@- beta^2 / 8@.
The \texttt{temper} function wants the log unnormalized density of the
equilibrium distribution.
We include an additional argument \texttt{log.pseudo.prior},
which is $\log(c_i)$ in our mathematical development, because this changes
from run to run as we adjust it by trial and error.  Other ``arguments''
are the model matrix of the full model \texttt{modmat}, the matrix
\texttt{models} relating integer indices (the first component of the state
vector of the ST chain) to which predictors are in or out of the model,
and the data vector \texttt{y}, but these are not passed as arguments to our
function and instead are found in the R global environment.
<<ludfun>>=
modmat <- out$x
y <- logit$y

ludfun <- function(state, log.pseudo.prior) {
    stopifnot(is.numeric(state))
    stopifnot(length(state) == ncol(models) + 2)
    icomp <- state[1]
    stopifnot(icomp == as.integer(icomp))
    stopifnot(1 <= icomp && icomp <= nrow(models))
    stopifnot(is.numeric(log.pseudo.prior))
    stopifnot(length(log.pseudo.prior) == nrow(models))
    beta <- state[-1]
    inies <- c(TRUE, as.logical(models[icomp, ]))
    beta.logl <- beta
    beta.logl[! inies] <- 0
    eta <- as.numeric(modmat %*% beta.logl)
    logp <- ifelse(eta < 0, eta - log1p(exp(eta)), - log1p(exp(- eta)))
    logq <- ifelse(eta < 0, - log1p(exp(eta)), - eta - log1p(exp(- eta)))
    logl <- sum(logp[y == 1]) + sum(logq[y == 0])
    logl + sum(dnorm(beta, 0, 2, log = TRUE)) + log.pseudo.prior[icomp]
}
@

\subsection{Trial and Error}

Now we are ready to try it out.  We start in the full model at its MLE,
and we initialize \texttt{log.pseudo.prior} at all zeros, having no idea
\emph{a priori} what it should be.
<<try1>>=
state.initial <- c(nrow(models), out$coefficients)

qux <- rep(0, nrow(models))

out <- temper(ludfun, initial = state.initial, neighbors = neighbors,
    nbatch = 1000, blen = 100, log.pseudo.prior = qux)

names(out)
out$time
@
So what happened?
<<what>>=
ibar <- colMeans(out$ibatch)
ibar
@
The ST chain did not mix well, several models not being visited even once.
So we adjust the pseudo priors to get uniform distribution.
<<adjust>>=
qux <- qux + pmin(log(max(ibar) / ibar), 10)
qux <- qux - min(qux)
qux
@
The new pseudoprior should be proportional to \verb@1 / ibar@ if \texttt{ibar}
is an accurate estimate of \eqref{eq:st-expectations}, but this makes no sense
when the estimates are bad, in particular, when the are exactly zero.  Thus
we put an upper bound, chosen arbitrarily (here 10) on the maximum increase
of the log pseudoprior.  The statement
\begin{verbatim}
qux <- qux - min(qux)
\end{verbatim}
is unnecessary.  An overall arbitrary constant can be added to
the log pseudoprior without changing the equilibrium distribution of the
ST chain.
We do this only to make \texttt{qux} more comparable from
run to run.

Now we repeat this until the log pseudoprior ``converges'' roughly.
Because this loop takes longer than CRAN vingettes are supposed to
take, we save the results to a file
and load the results from this file if it already exists.
<<iterate>>=
lout <- suppressWarnings(try(load("bfst1.rda"), silent = TRUE))
if (inherits(lout, "try-error")) {
    qux.save <- qux
    time.save <- out$time
    repeat{
        out <- temper(out, log.pseudo.prior = qux)
        ibar <- colMeans(out$ibatch)
        qux <- qux + pmin(log(max(ibar) / ibar), 10)
        qux <- qux - min(qux)
        qux.save <- rbind(qux.save, qux, deparse.level = 0)
        time.save <- rbind(time.save, out$time, deparse.level = 0)
        if (max(ibar) / min(ibar) < 2) break
    }
    save(out, qux, qux.save, time.save, file = "bfst1.rda")
} else {
    .Random.seed <- out$final.seed
}
print(qux.save, digits = 3)
print(qux, digits = 3)
apply(time.save, 2, sum)
@

Now that the pseudoprior is adjusted well enough, we need to perhaps
make other adjustments to get acceptance rates near 20\%.
<<accept-i-x>>=
print(out$accepti, digits = 3)
print(out$acceptx, digits = 3)
@
The acceptance rates for swaps seem o. k.
<<accept-i-min>>=
min(as.vector(out$accepti), na.rm = TRUE)
@
and there is nothing simple we can do to adjust them (adjustment is possible,
see the discussion section for more on this issue).  We adjust the
acceptance rates for within model moves by adjusting the scaling.
<<scale>>=
out <- temper(out, scale = 0.5, log.pseudo.prior = qux)
time.save <- rbind(time.save, out$time, deparse.level = 0)
print(out$acceptx, digits = 3)
@
Looks o.~k.\ now.

Inspection of autocorrelation functions for components
of \verb@out$ibatch@ (not shown) says batch length needs to be at least
4 times longer.  We make it 10 times longer for safety.

Because this run takes longer than CRAN vingettes are supposed to
take, we save the results to a file
and load the results from this file if it already exists.
<<try6>>=
lout <- suppressWarnings(try(load("bfst2.rda"), silent = TRUE))
if (inherits(lout, "try-error")) {
    out <- temper(out, blen = 10 * out$blen, log.pseudo.prior = qux)
    save(out, file = "bfst2.rda")
} else {
    .Random.seed <- out$final.seed
}
time.save <- rbind(time.save, out$time, deparse.level = 0)
foo <- apply(time.save, 2, sum)
foo.min <- floor(foo[1] / 60)
foo.sec <- foo[1] - 60 * foo.min
c(foo.min, foo.sec)
@
The total time for all runs of the temper function was
\Sexpr{foo.min} minutes and \Sexpr{round(foo.sec, 1)} seconds.

\subsection{Bayes Factor Calculations}

Now we calculate log 10 Bayes factors relative to the model with the highest
unnormalized Bayes factor.
<<doit>>=
log.10.unnorm.bayes <- (qux - log(colMeans(out$ibatch))) / log(10)
k <- seq(along = log.10.unnorm.bayes)[log.10.unnorm.bayes
    == min(log.10.unnorm.bayes)]
models[k, ]

log.10.bayes <- log.10.unnorm.bayes - log.10.unnorm.bayes[k]
log.10.bayes
@
These are base 10 logarithms of the Bayes factors against the $k$-th
model where $k = \Sexpr{k}$.  For example, the Bayes factor for the $k$-th
model divided by the Bayes factor for the first model is
$10^{\Sexpr{round(log.10.bayes[1], 3)}}$.

Now we calculate Monte Carlo standard errors two different ways.  One is
the way the delta method is usually taught.  To simplify notation, denote
the Bayes factors
$$
   b_m = b(y \mid m)
$$
and their Monte Carlo approximations $\hat{b}_m$.  Then the log Bayes factors
are
$$
   g_i(b) = \log_{10} b_i - \log_{10} b_k
$$
hence we need to apply the delta method with the function $g_i$, which has
derivatives
\begin{align*}
   \frac{\partial g_i(b)}{\partial b_i}
   & =
   \frac{1}{b_i \log_e(10)}
   \\
   \frac{\partial g_i(b)}{\partial b_k}
   & =
   - \frac{1}{b_k \log_e(10)}
   \\
   \frac{\partial g_i(b)}{\partial b_j}
   & =
   0, \qquad \text{$j \neq i$ and $j \neq k$}
\end{align*}
<<doit-se-one>>=
fred <- var(out$ibatch) / out$nbatch
sally <- colMeans(out$ibatch)
mcse.log.10.bayes <- (1 / log(10)) * sqrt(diag(fred) / sally^2 -
    2 * fred[ , k] / (sally * sally[k]) +
    fred[k, k] / sally[k]^2)
mcse.log.10.bayes

foompter <- cbind(models, log.10.bayes, mcse.log.10.bayes)
round(foompter, 5)
@

An alternative calculation of the MCSE replaces the actual function
of the raw Bayes factors with its best linear approximation
$$
   \frac{1}{\log_e(10)} \left(\frac{\hat{b}_i - b_i}{b_i}
   - \frac{\hat{b}_k - b_k}{b_k} \right)
$$
and calculates the standard deviation of this quantity by batch means
<<doit-too>>=
ibar <- colMeans(out$ibatch)
herman <- sweep(out$ibatch, 2, ibar, "/")
herman <- sweep(herman, 1, herman[ , k], "-")
mcse.log.10.bayes.too <- (1 / log(10)) *
    apply(herman, 2, sd) /sqrt(out$nbatch)
all.equal(mcse.log.10.bayes, mcse.log.10.bayes.too)
@

\section{Discussion}

We hope readers are impressed with the power of this method.  The key
to the method is pseudopriors adjusted by trial and error.  The method
could have been invented by any Bayesian who realized that the priors
on models, $\prior(m)$ in our notation in Section~\ref{sec:bayes-factors},
do not affect the Bayes factors and hence are irrelevant to calculating
Bayes factors.  Thus the priors (or pseudopriors in our terminology) should
be chosen for reasons of computational convenience, as we have done,
rather than to incorporate prior information.

The rest of the details of the method are unimportant.  The \texttt{temper}
function in R is convenient to use for this purpose, but there is no reason
to believe that it provides optimal sampling.  Samplers carefully designed
for each particular application would undoubtedly do better.  Our notion
of ``padding'' so that the within model parameters have the same dimension
for all models follows \citet{carlin-chib} but ``reversible jump'' samplers
\citep{green} would undoubtedly do better.  Unfortunately, there seems to
be no way to code up a function like \texttt{temper} that uses ``reversible
jump'' and requires no theoretical work from users that if messed up destroys
the algorithm.  The \texttt{temper} function is foolproof in the sense that
if the log unnormalized density function written by the user
(like our \texttt{ludfun}) is correct, then the ST Markov chain has the
equilibrium distribution is supposed to have.  There is nothing the
user can mess up except this user-written function.  No analog of this
for ``reversible jump'' chains is apparent (to your humble author).

Two issues remain where the text above said ``see the discussion section for
more on this issue.''  The first was about within model priors for the
``padding'' components of within model parameter vectors
$g_{\text{pad}}(\theta_{\text{pad}} \mid m)$ in
the notation in \eqref{eq:unnormalized-bayes-factors}.
Rather than choose these so that they do not depend on the data (as we did),
it would be better (if more trouble) to choose them differently for each
``padding'' component, centering $g_{\text{pad}}(\theta_{\text{pad}} \mid m)$
so the distribution of a component of $\theta_{\text{pad}}$ is near to the
marginal distribution of the same component in neighboring models (according to
the \texttt{neighbors} argument of the \texttt{temper} function).

The other remaining issue is adjusting acceptance rates for jumps.  There
is no way to adjust this other than by changing the number of models and
their definitions.  But the models we have cannot be changed; if we are
to calculate Bayes factors for them, then we must sample them as they are.
But we can insert new models between old models.  For example,
if the acceptance for swaps between model $i$ and model $j$ is too low, then
we can insert distribution $k$ between them that has unnormalized density
$$
   h_k(x) = \sqrt{h_i(x) h_j(x)}.
$$
This idea is inherited from simulated tempering; \citep{geyer-thompson}
have much
discussion of how to insert additional distributions into a tempering network.
It is another key issue in using tempering to speed up sampling.  It is
less obvious in the Bayes factor context, but still an available technique
if needed.


\begin{thebibliography}{}

\bibitem[Carlin and Chib(1995)]{carlin-chib}
Carlin, B.~P. and Chib, S. (1995).
\newblock Bayesian model choice via Markov chain Monte Carlo methods.
\newblock \emph{Journal of the Royal Statistical Society, Series B},
    \textbf{57}, 473--484.

\bibitem[Geyer(1994)]{geyer}
Geyer, C.~J. (1994).
\newblock On the convergence of Monte Carlo maximum likelihood calculations.
\newblock \emph{Journal of the Royal Statistical Society, Series B},
    \textbf{56} 261--274.

\bibitem[Geyer., 2009]{mcmc-R-package}
Geyer., C.~J. (2009).
\newblock \emph{mcmc: Markov Chain Monte Carlo}.
\newblock R package version 0.7-2, available from CRAN.

\bibitem[Geyer and Thompson(1995)]{geyer-thompson}
Geyer, C.~J., and Thompson, E.~A. (1995).
\newblock Annealing Markov chain Monte Carlo with applications to ancestral
    inference.
\newblock \emph{Journal of the American Statistical Association}, \textbf{90},
    909--920.

\bibitem[Green(1995)]{green}
Green, P.~J. (1995).
\newblock Reversible jump {M}arkov chain {M}onte {C}arlo computation and
  {B}ayesian model determination.
\newblock \emph{Biometrika}, \textbf{82}, 711--732.

\bibitem[Hoeting et al.(1999)Hoeting, Madigan, Raftery, and Volinsky]{bma}
Hoeting, J.~A., Madigan, D., Raftery, A.~E. and Volinsky, C.~T. (1999).
\newblock Bayesian model averaging: A tutorial (with discussion).
\newblock \emph{Statical Science}, \textbf{19}, 382--417.
\newblock The version printed in the journal had the equations messed up in
    the production process; a corrected version is available at
    \url{http://www.stat.washington.edu/www/research/online/1999/hoeting.pdf}.

\bibitem[Marinari and Parisi(1992)]{marinari-parisi}
Marinari, E., and Parisi G. (1992).
\newblock Simulated tempering: A new Monte Carlo Scheme.
\newblock \emph{Europhysics Letters}, \textbf{19}, 451--458.

\bibitem[R Development Core Team(2010)]{rcore}
R Development Core Team (2010).
\newblock R: A language and environment for statistical computing.
\newblock R Foundation for Statistical Computing, Vienna, Austria.
\newblock \url{http://www.R-project.org}.

\bibitem[Sung and Geyer(2007)]{sung-geyer}
Sung, Y.~J. and Geyer, C.~J. (2007).
\newblock Monte Carlo likelihood inference for missing data models.
\newblock \emph{Annals of Statistics}, \textbf{35}, 990--1011.

\bibitem[Thompson and Guo(1991)]{thompson-guo}
Thompson, E. A. and Guo, S. W. (1991).
\newblock Evaluation of likelihood ratios for complex genetic models.
\newblock \emph{IMA J. Math. Appl. Med. Biol.}, \textbf{8}, 149--169.

\bibitem[Torrie and Valleau(1977)]{torrie-valleau}
Torrie, G.~M., and Valleau, J.~P. (1977).
\newblock Nonphysical sampling distributions in Monte Carlo free-energy
  estimation: Umbrella sampling.
\newblock \emph{Journal of Computational Physics}, \textbf{23}, 187--199.

\end{thebibliography}

\end{document}

\documentclass{article}

\usepackage{amstext}

% \VignetteIndexEntry{Debugging MCMC Code}

\begin{document}

<<foo,include=FALSE,echo=FALSE>>=
options(keep.source = TRUE, width = 60)
foo <- packageDescription("mcmc")
@

\title{Debugging MCMC Code}
\author{Charles J. Geyer}
\maketitle

\section{Introduction}

This document discusses debugging Markov chain Monte Carlo code
using the R contributed package \texttt{mcmc} (Version \Sexpr{foo$Version})
for examples.  It also documents the debugging output of the functions
\texttt{mcmc} and \texttt{temper}.

Debugging MCMC code if the code is taken as a black box is basically
impossible.  In interesting examples, the only thing one knows about
the equilibrium distribution of an MCMC sampler is what one learns
from the samples.  This obviously doesn't help with testing.  If the
sampler is buggy, then the only thing you know about the equilibrium
distribution is wrong, but if you don't know it is buggy, then you don't
know it is wrong.  So you don't know anything.  There is no way to tell
whether random output has the correct distribution when you don't know
anything about the distribution it is supposed to have.

The secret to debugging MCMC code lies in two principles:
\begin{itemize}
\item take the randomness out, and
\item expose the innards.
\end{itemize}
The first slogan means consider the algorithm a deterministic function
of the elementary pseudo-random numbers that are trusted (for example,
the outputs of the R random number generators, which you aren't responsible
for debugging and are also well tested).
The second slogan means output, at least for debugging purposes, enough
intermediate state so that testing is straightforward.

For a Gibbs sampler, this means outputting all of the trusted elementary
pseudo-random numbers used, the state before and after each elementary
Gibbs update, and which update is being done if a random scan is used.
Also one needs to output the initial seeds of the pseudo-random number
generator (this is true for all situations and will not be mentioned again).

For a Metropolis-Hastings sampler, this means outputting all of the trusted
elementary
pseudo-random numbers used, the state before and after each elementary
Metropolis-Hastings update, the proposal for that update, the Hastings ratio
for that update, decision (accept or reject) in that update.

For more complicated MCMC samplers, there is more ``innards'' to ``expose''
(see the discussion of the \texttt{temper} function below), but you get the
idea.  You can't output too much debugging information.

\section{The Metrop Function}

The R function \texttt{metrop} in the \texttt{mcmc} package has an argument
\verb@debug = FALSE@ that when \verb@TRUE@ causes extra debugging information
to be output.
Let \texttt{niter} be the number of iterations
\verb@nbatch * blen * nspac@, and let \texttt{d} be the dimension of the state
vector.  The result of invoking \texttt{metrop} is a list.  When
\verb@debug = TRUE@ it has the following additional components
\begin{itemize}
\item \texttt{current}, an \texttt{niter} by \texttt{d} matrix
    of mode \verb@"numeric"@, the state before iteration \texttt{i}
    is \verb@current[i, ]@
\item \texttt{proposal}, an \texttt{niter} by \texttt{d} matrix
    of mode \verb@"numeric"@, the proposal for iteration \texttt{i}
    is \verb@proposal[i, ]@
\item \texttt{z}, an \texttt{niter} by \texttt{d} matrix
    of mode \verb@"numeric"@, the vector of standard normal random variates
    used to generate the proposal for iteration \texttt{i}
    is \verb@z[i, ]@
\item \texttt{log.green}, a vector of length \texttt{niter}
    and mode \verb@"numeric"@, the logarithm of the Hastings ratio
    for each iteration
\item \texttt{u}, a vector of length \texttt{niter}
    and mode \verb@"numeric"@, the $\text{Uniform}(0, 1)$ random variate
    compared to the Hastings ratio for each iteration or \texttt{NA} if
    none is needed (when the log Hastings ratio is nonnegative)
\item \texttt{debug.accept}, a vector of length \texttt{niter}
    and mode \verb@"logical"@, the decision for each iteration,
    accept the proposal (\texttt{TRUE}) or reject it (\texttt{FALSE})
\end{itemize}
(The components \texttt{z} and \texttt{debug.accept} were added in
version 0.7-3 of the \texttt{mcmc} package.  Before that only the others
were output.)

Two components of the list returned by the \texttt{metrop} function always
(whether \verb@debug = TRUE@ or \verb@debug = FALSE@) are also necessary
for debugging.  They are
\begin{itemize}
\item \texttt{initial.seed} the value of the variable \texttt{.Random.seed}
    that contains the seeds of the R random number generator system before
    invocation of the \texttt{metrop} function
\item \texttt{final}, a vector of length \texttt{d} and mode \verb@"numeric"@,
    the state after the last iteration
\end{itemize}

All of the files in the \texttt{tests} directory of the source for the
package (not installed but found in the source tarball on CRAN) test
the \texttt{metrop} function except those beginning \texttt{temp},
which test the \texttt{temper} function.  Since these tests were written
many years ago, are spread out over many files, and are not commented,
we will not describe them in detail.  Suffice it to say that they check
every aspect of the functioning of the \texttt{metrop} function.

\section{The Temper Function}

The R function \texttt{temper} in the \texttt{mcmc} package has an argument
\verb@debug = FALSE@ that when \verb@TRUE@ causes extra debugging information
to be output.
Let \texttt{niter} be the number of iterations
\verb@nbatch * blen * nspac@, let \texttt{d} be the dimension of the state
vector, and let \texttt{ncomp} be the number of components of the tempering
mixture.
The result of invoking \texttt{temper} is a list.  When
\verb@debug = TRUE@ and \verb@parallel = TRUE@ it has the following additional
components
% which
% unif.which
% state
% log.hastings
% unif.hastings
% proposal
% acceptd
% norm
% unif.choose
% coproposal
\begin{itemize}
\item \texttt{which}, a vector of length \texttt{niter}
    and mode \verb@"logical"@ the type of update for each iteration,
    within component (\texttt{TRUE}) or swap components (\texttt{FALSE}).
\item \texttt{unif.which}, a vector of length \texttt{niter}
    and mode \verb@"numeric"@, the $\text{Uniform}(0, 1)$ random variate
    used to decide which type of update is done.
\item \texttt{state}, an \texttt{niter} by \texttt{ncomp} by \texttt{d}
    array of mode \verb@"numeric"@, the state before iteration \texttt{i}
    is \verb@state[i, , ]@
\item \texttt{proposal}, an \texttt{niter} by \verb@d + 1@
    matrix of mode \verb@"numeric"@, the proposal for iteration \texttt{i}
    is \verb@proposal[i, ]@ (explanation below)
\item \texttt{coproposal}, an \texttt{niter} by \verb@d + 1@
    matrix of mode \verb@"numeric"@, the proposal for iteration \texttt{i}
    is \verb@coproposal[i, ]@ (explanation below)
\item \texttt{log.hastings}, a vector of length \texttt{niter}
    and mode \verb@"numeric"@, the logarithm of the Hastings ratio for
    each iteration
\item \texttt{unif.hastings}, a vector of length \texttt{niter}
    and mode \verb@"numeric"@, the $\text{Uniform}(0, 1)$ random variate
    compared to the Hastings ratio for each iteration or \texttt{NA} if
    none is needed (when the log Hastings ratio is nonnegative)
\item \texttt{acceptd}, a vector of length \texttt{niter}
    and mode \verb@"logical"@, the decision for each iteration,
    accept the proposal (\texttt{TRUE}) or reject it (\texttt{FALSE})
\item \texttt{norm}, an \texttt{niter} by \texttt{d} matrix
    of mode \verb@"numeric"@, the vector of standard normal random variates
    used to generate the proposal for iteration \texttt{i} is \verb@z[i, ]@
    unless none are needed (for swap updates) when it is \texttt{NA} 
\item \texttt{unif.choose}, an \texttt{niter} by 2 matrix
    of mode \verb@"numeric"@, the vector of $\text{Uniform}(0, 1)$
    random variates used to choose the components to update in iteration
    \texttt{i} is \verb@unif.choose[i, ]@; in a swap update two are used;
    in a within-component update only one is used and the second is \texttt{NA}
\end{itemize}

In a within-component update, one component say \texttt{j} is chosen for
update.  The \emph{coproposal} is the current value of the state for this
component, which is a vector of length \verb@d + 1@, the first
component of which is \texttt{j} and the rest of which is \verb@state[i, j, ]@
if we are in iteration \texttt{i}.
The \emph{proposal} is a similar vector, the first
component of which is again \texttt{j} and the rest of which is a multivariate
normal random vector centered at \verb@state[i, j, ]@.
The coproposal is the current state; the proposal is the possible value
(if accepted) of the state at the next time.

In a swap update, two components say \texttt{j1} and \texttt{j2} are chosen for
update.  Strictly, speaking the coproposal is the pair of vectors
\verb@c(j1, state[i, j1, ])@ and \verb@c(j2, state[i, j2, ])@
and the proposal is these swapped, that is, the pair of vectors
\verb@c(j2, state[i, j1, ])@ and \verb@c(j1, state[i, j2, ])@
if we are in iteration \texttt{i}.
Since, however, there is a lot of redundant information here,
the vector \verb@c(j1, state[i, j1, ])@ is output as \verb@coproposal[i, ]@
and the vector \verb@c(j2, state[i, j2, ])@ is output as \verb@proposal[i, ]@.

When \verb@debug = TRUE@ and \verb@parallel = FALSE@
the result of invoking \texttt{temper} is a list having
the following additional components
% which
% unif.which
% state
% log.hastings
% unif.hastings
% proposal
% acceptd
% norm
% unif.choose
\begin{itemize}
\item \texttt{which}, a vector of length \texttt{niter}
    and mode \verb@"logical"@ the type of update for each iteration,
    within component (\texttt{TRUE}) or jump from one component to
    another (\texttt{FALSE}).
\item \texttt{unif.which}, a vector of length \texttt{niter}
    and mode \verb@"numeric"@, the $\text{Uniform}(0, 1)$ random variate
    used to decide which type of update is done.
\item \texttt{state}, an \texttt{niter} by \verb@d + 1@
    matrix of mode \verb@"numeric"@, the state before iteration \texttt{i}
    is \verb@state[i, ]@
\item \texttt{proposal}, an \texttt{niter} by \verb@d + 1@
    matrix of mode \verb@"numeric"@, the proposal for iteration \texttt{i}
    is \verb@proposal[i, ]@
\item \texttt{log.hastings}, a vector of length \texttt{niter}
    and mode \verb@"numeric"@, the logarithm of the Hastings ratio for
    each iteration
\item \texttt{unif.hastings}, a vector of length \texttt{niter}
    and mode \verb@"numeric"@, the $\text{Uniform}(0, 1)$ random variate
    compared to the Hastings ratio for each iteration or \texttt{NA} if
    none is needed (when the log Hastings ratio is nonnegative)
\item \texttt{acceptd}, a vector of length \texttt{niter}
    and mode \verb@"logical"@, the decision for each iteration,
    accept the proposal (\texttt{TRUE}) or reject it (\texttt{FALSE})
\item \texttt{norm}, an \texttt{niter} by \texttt{d} matrix
    of mode \verb@"numeric"@, the vector of standard normal random variates
    used to generate the proposal for iteration \texttt{i} is \verb@z[i, ]@
    unless none are needed (for jump updates) when it is \texttt{NA} 
\item \texttt{unif.choose}, a vector of length \texttt{niter}
    and mode \verb@"numeric"@, the $\text{Uniform}(0, 1)$
    random variates used to choose the component to update in iteration
    \texttt{i} is \verb@unif.choose[i, ]@; in a jump update one is used;
    in a within-component update none is used and \texttt{NA} is output
\end{itemize}

All of the files in the \texttt{tests} directory of the source for the
package (not installed but found in the source tarball on CRAN)
beginning \texttt{temp} test the \texttt{temper} function.
They check every aspect of the functioning of the \texttt{temper} function.

In the file \texttt{temp-par.R} in the \texttt{tests} directory, the following
checks are made according to the comments in that file
\begin{enumerate}
\item check decision about within-component or jump/swap
\item check proposal and coproposal are actually current state or part thereof
\item check hastings ratio calculated correctly
\item check hastings rejection decided correctly
\item check acceptance carried out or not (according to decision) correctly
\item check within-component proposal
\item check swap proposal
\item check standard normal and uniform random numbers are as purported
\item check batch means
\item check acceptance rates
\item check scale vector
\item check scale matrix
\item check scale list
\item check outfun
\end{enumerate}
In the file \texttt{temp-ser.R} in the \texttt{tests} directory, the all of
the same checks are made according to the comments in that file except for
check number 2 above, which would make no sense because there is no
\texttt{coproposal} component in the serial (\verb@parallel = FALSE@) case.

\end{document}

\documentclass{article}

\usepackage{natbib}
\usepackage{graphics}
\usepackage{amsmath}
\usepackage{indentfirst}
\usepackage{url}
\usepackage[utf8]{inputenc}

\DeclareMathOperator{\var}{var}
\DeclareMathOperator{\cov}{cov}

% \VignetteIndexEntry{MCMC Example}

\begin{document}

<<foo,include=FALSE,echo=FALSE>>=
options(keep.source = TRUE, width = 60)
@

\title{MCMC Package Example (Version \Sexpr{packageVersion("mcmc")})}
\author{Charles J. Geyer}
\maketitle

\section{The Problem}

This is an example of using the \verb@mcmc@ package in R.  The problem comes
from a take-home question on a (take-home) PhD qualifying exam
(School of Statistics, University of Minnesota).

Simulated data for the problem are in the dataset \verb@logit@.
There are five variables in the data set, the response \verb@y@
and four predictors, \verb@x1@, \verb@x2@, \verb@x3@, and \verb@x4@.

A frequentist analysis for the problem is done by the following R statements
<<frequentist>>=
library(mcmc)
data(logit)
out <- glm(y ~ x1 + x2 + x3 + x4, data = logit,
    family = binomial, x = TRUE)
summary(out)
@

But this problem isn't about that frequentist analysis, we want a Bayesian
analysis.  For our Bayesian analysis we assume the same data model as the
frequentist, and we assume the prior distribution of the five parameters
(the regression coefficients) makes them independent and identically
normally distributed with mean 0 and standard deviation 2.

The log unnormalized posterior density (log likelihood plus log prior)
for this model is calculated by
the following R function.  In order to avoid using either global variables
or \verb@...@ arguments, we use the function factory pattern
(\citealp[Section~10.3.1]{wickham};
\citealp[Section~7.5, especially Subsection~7.5.4]{basic};
see also Appendix~\ref{sec:versus} below).
<<log.unnormalized.posterior>>=
lupost_factory <- function(x, y) function(beta) {
    eta <- as.numeric(x %*% beta)
    logp <- ifelse(eta < 0, eta - log1p(exp(eta)), - log1p(exp(- eta)))
    logq <- ifelse(eta < 0, - log1p(exp(eta)), - eta - log1p(exp(- eta)))
    logl <- sum(logp[y == 1]) + sum(logq[y == 0])
    return(logl - sum(beta^2) / 8)
}

lupost <- lupost_factory(out$x, out$y)
@
The tricky calculation of the log likelihood avoids overflow and catastrophic
cancellation in calculation of $\log(p)$ and $\log(q)$ where
\begin{align*}
   p & = \frac{\exp(\eta)}{1 + \exp(\eta)} = \frac{1}{1 + \exp(- \eta)}
   \\
   q & = \frac{1}{1 + \exp(\eta)} = \frac{\exp(- \eta)}{1 + \exp(- \eta)}
\end{align*}
so taking logs gives
\begin{align*}
   \log(p) & = \eta - \log(1 + \exp(\eta)) = - \log(1 + \exp(- \eta))
   \\
   \log(q) & = - \log(1 + \exp(\eta)) = - \eta - \log(1 + \exp(- \eta))
\end{align*}
To avoid overflow, we always chose the case where the argument of $\exp$
is negative.  We have also avoided catastrophic cancellation when
$\lvert\eta\rvert$ is large.  If $\eta$ is large and positive, then
\begin{align*}
   p & \approx 1
   \\
   q & \approx 0
   \\
   \log(p) & \approx - \exp(- \eta)
   \\
   \log(q) & \approx - \eta - \exp(- \eta)
\end{align*}
and our use of the R function \texttt{log1p}, which calculates the
function $x \mapsto \log(1 + x)$
correctly for small $x$ avoids all problems.  The case where $\eta$ is large
and negative is similar.

\section{Beginning MCMC}

With those definitions in place, the following code runs the Metropolis
algorithm to simulate the posterior.
<<metropolis-try-1>>=
set.seed(42)    # to get reproducible results
beta.init <- as.numeric(coefficients(out))

out <- metrop(lupost, beta.init, 1e3)
names(out)
out$accept
@

The arguments to the \verb@metrop@ function used here (there are others
we don't use) are
\begin{itemize}
\item an R function (here \verb@lupost@) that evaluates the log unnormalized
    density of the desired stationary distribution of the Markov chain
    (here a posterior distribution).  Note that (although this example
    does not exhibit the phenomenon) that the unnormalized density may
    be zero, in which case the log unnormalized density is \verb@-Inf@.
\item an initial state (here \verb@beta.init@) of the Markov chain.
\item a number of batches (here \verb@1e3@) for the Markov chain.
    This combines with batch length and spacing (both 1 by default)
    to determine the number of iterations done.
\item additional arguments (here \verb@x@ and \verb@y@) supplied to
    provided functions (here \verb@lupost@).
\item there is no ``burn-in'' argument, although burn-in is easily
    accomplished, if desired (more on this below).
\end{itemize}

The output is in the component \verb@out$batch@ returned by the \verb@metrop@
function.  We'll look at it presently, but first we need to adjust the
proposal to get a higher acceptance rate (\verb@out$accept@).  It is generally
accepted \citep*{grg} that an acceptance rate of about 20\% is right, although
this recommendation is based on the asymptotic analysis of a toy problem
(simulating a multivariate normal distribution) for which one would never
use MCMC and is very unrepresentative of difficult MCMC applications.

\citet{geyer-temp} came to a similar conclusion,
that a 20\% acceptance rate is about right, in a very different situation.
But they also warned that a 20\% acceptance rate could be very wrong
and produced
an example where a 20\% acceptance rate was impossible and attempting to
reduce the acceptance rate below 70\% would keep the sampler from ever
visiting part of the state space.  So the 20\% magic number must be
considered like other rules of thumb we teach in intro courses
(like $n > 30$ means means normal approximation is valid).
We know these rules of thumb can fail.
There are examples in the literature where
they do fail.  We keep repeating them because we want something simple to
tell beginners, and they are all right for some problems.

Be that as it may, we try for 20\%.
<<metropolis-try-2>>=
out <- metrop(out, scale = 0.1)
out$accept
out <- metrop(out, scale = 0.3)
out$accept
out <- metrop(out, scale = 0.5)
out$accept
out <- metrop(out, scale = 0.4)
out$accept
@

Here the first argument to each instance of the \verb@metrop@ function is
the output of a previous invocation.  The Markov chain continues where
the previous run stopped, doing just what it would have done if it had
kept going, the initial state and random seed being the final state and
final random seed of the previous invocation.  Everything stays the same
except for the arguments supplied (here \verb@scale@).
\begin{itemize}
\item The argument \verb@scale@ controls the size of the Metropolis
    ``normal random walk'' proposal.  The default is \verb@scale = 1@.
    Big steps give lower acceptance rates.  Small steps give higher.
    We want something about 20\%.  It is also possible to make \verb@scale@
    a vector or a matrix.  See \verb@help(metrop)@.
\end{itemize}

Because each run starts where the last one stopped (when the first argument
to \verb@metrop@ is the output of the previous invocation), each run serves
as ``burn-in'' for its successor (assuming that any part of that run was
worth anything at all).

\section{Diagnostics}

O.~K.  That does it for the acceptance rate.  So let's do a longer run
and look at the results.
<<label=metropolis-try-3>>=
out <- metrop(out, nbatch = 1e4)
t.test(out$accept.batch)$conf.int
out$time
@
Here we do a Monte Carlo confidence interval
for the true unknown acceptance rate
(what we would see with an infinite Monte Carlo sample size).

Figure~\ref{fig:fig1} (page~\pageref{fig:fig1})
shows the time series plot made by the R statement
<<label=fig1too,include=FALSE>>=
plot(ts(out$batch))
@
\begin{figure}
\begin{center}
<<label=fig1,fig=TRUE,echo=FALSE>>=
<<fig1too>>
@
\end{center}
\caption{Time series plot of MCMC output.}
\label{fig:fig1}
\end{figure}

Another way to look at the output is an autocorrelation plot.
Figure~\ref{fig:fig2} (page~\pageref{fig:fig2})
shows the time series plot made by the R statement
<<label=fig2too,include=FALSE>>=
acf(out$batch)
@
\begin{figure}
\begin{center}
<<label=fig2,fig=TRUE,echo=FALSE>>=
<<fig2too>>
@
\end{center}
\caption{Autocorrelation plot of MCMC output.}
\label{fig:fig2}
\end{figure}

As with any multiplot plot, these are a bit hard to read.  Readers are
invited to make the separate plots to get a better picture.
As with all ``diagnostic'' plots in MCMC, these don't ``diagnose''
subtle problems.
\begin{quotation}
The purpose of regression diagnostics is to find obvious, gross,
embarrassing problems that jump out of simple plots \citep{bogosity}.
\end{quotation}
The time series plots will show \emph{obvious} nonstationarity.
They will not show \emph{nonobvious} nonstationarity.  They
provide no guarantee whatsoever that your Markov chain is sampling
anything remotely resembling the correct stationary distribution
(with log unnormalized density \verb@lupost@).  In this very easy
problem, we do not expect any convergence difficulties and so believe
what the diagnostics seem to show, but one is a fool to trust such
diagnostics in difficult problems.

The autocorrelation plots seem to show that the
the autocorrelations are negligible after about lag 25.
This diagnostic inference is reliable if the sampler is actually
working (has nearly reached equilibrium) and worthless otherwise.
Thus batches of length 25 should be sufficient, but let's use
length 100 to be safe.

A more judicious discussion of asymptotics is found
in \citet[Section~1.11.5]{intro}, which says
\begin{quotation}
Many [diagnostics] come with theorems, but the theorems never prove
the property you really want a diagnostic to have. These theorems say
that if the chain converges, then the diagnostic will probably say that
the chain converged, but they do not say that if the chain pseudo-converges,
then the diagnostic will probably say that the chain did not converge.
\end{quotation}

\section{Monte Carlo Estimates and Standard Errors}

<<label=metropolis-try-4>>=
out <- metrop(out, nbatch = 100, blen = 100,
    outfun = function(z) c(z, z^2))
t.test(out$accept.batch)$conf.int
out$time
@

We have added an argument \verb@outfun@ that gives the functional
of the Markov chain \citep[Section~1.6]{intro} we want to average.
For this problem we are interested
in both posterior mean and variance.  Mean is easy, just average the
variables in question.  But variance is a little tricky.  We need to
use the identity
$$
   \var(X) = E(X^2) - E(X)^2
$$
to write variance as a function of two things that can be estimated
by simple averages.  Hence we want to average the state itself and
the squares of each component.  Hence our \verb@outfun@ returns
\verb@c(z, z^2)@ for an argument (the state vector) \verb@z@.

\subsection{Simple Means}

The grand means (means of batch means) are
<<label=metropolis-batch>>=
apply(out$batch, 2, mean)
@
The first 5 numbers are the Monte Carlo estimates of the posterior means.
The second 5 numbers are the Monte Carlo estimates of the posterior
ordinary second moments.  We get the posterior variances by
<<label=metropolis-batch-too>>=
foo <- apply(out$batch, 2, mean)
mu <- foo[1:5]
sigmasq <- foo[6:10] - mu^2
mu
sigmasq
@

Monte Carlo standard errors (MCSE) are calculated from the batch means.
This is simplest for the means.
<<label=metropolis-mcse-mu>>=
mu.mcse <- apply(out$batch[ , 1:5], 2, sd) / sqrt(out$nbatch)
mu.mcse
@
The extra factor \verb@sqrt(out$nbatch)@ arises because the batch means
have variance $\sigma^2 / b$ where $b$ is the batch length, which is
\verb@out$blen@,
whereas the overall means \verb@mu@ have variance $\sigma^2 / n$ where
$n$ is the total number of iterations, which is \verb@out$blen * out$nbatch@.

\subsection{Functions of Means}

To get the MCSE for the posterior variances we apply the delta method.
Let $u_i$ denote the sequence of batch means of the first kind for one
parameter and $\bar{u}$ the grand mean (the estimate of the posterior mean
of that parameter),
let $v_i$ denote the sequence of batch means of the second kind for the
same parameter and $\bar{v}$ the grand mean (the estimate of the posterior
second absolute moment of that parameter), and let $\mu = E(\bar{u})$ and
$\nu = E(\bar{v})$.  Then the delta method linearizes the nonlinear function
$$
   g(\mu, \nu) = \nu - \mu^2
$$
as
$$
   \Delta g(\mu, \nu) = \Delta \nu - 2 \mu \Delta \mu
$$
saying that
$$
   g(\bar{u}, \bar{v}) - g(\mu, \nu)
$$
has the same asymptotic normal distribution as
$$
   (\bar{v} - \nu) - 2 \mu (\bar{u} - \mu)
$$
which, of course, has variance \verb@1 / nbatch@ times that of
$$
   (v_i - \nu) - 2 \mu (u_i - \mu)
$$
and this variance is estimated by
$$
   \frac{1}{n_{\text{batch}}} \sum_{i = 1}^{n_{\text{batch}}}
   \bigl[ (v_i - \bar{v}) - 2 \bar{u} (u_i - \bar{u}) \bigr]^2
$$
So
<<label=metropolis-mcse-sigmasq>>=
u <- out$batch[ , 1:5]
v <- out$batch[ , 6:10]
ubar <- apply(u, 2, mean)
vbar <- apply(v, 2, mean)
deltau <- sweep(u, 2, ubar)
deltav <- sweep(v, 2, vbar)
foo <- sweep(deltau, 2, ubar, "*")
sigmasq.mcse <- sqrt(apply((deltav - 2 * foo)^2, 2, mean) / out$nbatch)
sigmasq.mcse
@
does the MCSE for the posterior variance.

Let's just check that this complicated \verb@sweep@ and \verb@apply@ stuff
does do the right thing.
<<label=metropolis-mcse-sigmasq-too>>=
sqrt(mean(((v[ , 2] - vbar[2]) - 2 * ubar[2] * (u[ , 2] - ubar[2]))^2) /
    out$nbatch)
@

\paragraph{Comment} Through version 0.5 of this vignette it contained
an incorrect procedure for calculating this MCSE, justified by a handwave
(which was incorrect).
Essentially, it said to use the standard deviation of the batch means called
\verb@v@ here, which appears to be very conservative.

\subsection{Functions of Functions of Means}

If we are also interested in the posterior standard deviation
(a natural question, although not asked on the exam problem),
the delta method gives its standard error in terms of that
for the variance
<<label=metropolis-mcse-sigma>>=
sigma <- sqrt(sigmasq)
sigma.mcse <- sigmasq.mcse / (2 * sigma)
sigma
sigma.mcse
@

\section{A Final Run}

So that's it.  The only thing left to do is a little more precision
(the exam problem directed ``use a long enough run of your Markov chain
sampler so that the MCSE are less than 0.01'')
<<label=metropolis-try-5>>=
out <- metrop(out, nbatch = 500, blen = 400)
t.test(out$accept.batch)$conf.int
out$time
<<metropolis-batch-too>>
<<metropolis-mcse-mu>>
<<metropolis-mcse-sigmasq>>
<<metropolis-mcse-sigma>>
@
and some nicer output, which is presented in three tables
constructed from the R variables defined above
using the R \verb@xtable@ command in the \verb@xtable@ library.
\begin{table}[ht]
\caption{Posterior Means}
\label{tab:mu}
\begin{center}
<<label=tab1,echo=FALSE,results=tex>>=
foo <- rbind(mu, mu.mcse)
dimnames(foo) <- list(c("estimate", "MCSE"),
    c("constant", paste("$x_", 1:4, "$", sep = "")))
library(xtable)
print(xtable(foo, digits = rep(4, 6),
    align = c("l", rep("c", 5))), floating = FALSE,
    caption.placement = "top",
    sanitize.colnames.function = function(x) return(x))
@
\end{center}
\end{table}
\begin{table}[ht]
\caption{Posterior Variances}
\label{tab:sigmasq}
\begin{center}
<<label=tab1,echo=FALSE,results=tex>>=
foo <- rbind(sigmasq, sigmasq.mcse)
dimnames(foo) <- list(c("estimate", "MCSE"),
    c("constant", paste("$x_", 1:4, "$", sep = "")))
library(xtable)
print(xtable(foo, digits = rep(4, 6),
    align = c("l", rep("c", 5))), floating = FALSE,
    caption.placement = "top",
    sanitize.colnames.function = function(x) return(x))
@
\end{center}
\end{table}
\begin{table}[ht]
\caption{Posterior Standard Deviations}
\label{tab:sigma}
\begin{center}
<<label=tab1,echo=FALSE,results=tex>>=
foo <- rbind(sigma, sigma.mcse)
dimnames(foo) <- list(c("estimate", "MCSE"),
    c("constant", paste("$x_", 1:4, "$", sep = "")))
library(xtable)
print(xtable(foo, digits = rep(4, 6),
    align = c("l", rep("c", 5))), floating = FALSE,
    caption.placement = "top",
    sanitize.colnames.function = function(x) return(x))
@
\end{center}
\end{table}

Note for the record that the all the results presented in the tables
are from ``one long run'' where long here took only
<<label=time,echo=FALSE,results=tex>>=
cat(out$time[1], "\n")
@
seconds (on whatever computer it was run on).

\section{New Variance Estimation Functions}

R function \texttt{initseq} (added in version 0.6 of this package)
estimates variances in the Markov chain
central limit theorem (CLT) following the methodology introduced by
\citet[Section~3.3]{practical}.  These methods only apply to scalar-valued
functionals of
reversible Markov chains, but the Markov chains produced by the \texttt{metrop}
function satisfy this condition, even, as we shall see below, when batching
is used.

Rather than redo the Markov chains in the preceding material, we just look
at a toy problem, an AR(1) time series, which can be simulated in one line
of R.  This is the example on the help page for \texttt{initseq}.
<<x>>=
n <- 2e4
rho <- 0.99
x <- arima.sim(model = list(ar = rho), n = n)
@
The time series \texttt{x} is a reversible Markov chain and trivially
a scalar-valued functional of a Markov chain.

Define
\begin{equation} \label{eq:little}
   \gamma_k = \cov(X_i, X_{i + k})
\end{equation}
where the covariances refer to the stationary Markov chain having the
same transition probabilities as \texttt{x}.  Then the variance in the CLT
is
$$
   \sigma^2 = \gamma_0 + 2 \sum_{k = 1}^\infty \gamma_k
$$
\citep[Theorem~2.1]{practical}, that is,
$$
   \bar{x}_n \approx \text{Normal}\left(\mu, \frac{\sigma^2}{n}\right),
$$
where $\mu = E(X_i)$ is the quantity being estimated by MCMC (in this
toy problem we know $\mu = 0$).

Naive estimates of $\sigma^2$ obtained by plugging in empirical
estimates of the gammas do not provide consistent estimation
\citep[Section~3.1]{practical}.  Thus the scheme implemented
by the R function \texttt{initseq}.  Define
\begin{equation} \label{eq:big}
   \Gamma_k = \gamma_{2 k} + \gamma_{2 k + 1}
\end{equation}
\citet[Theorem~3.1]{practical} says that $\Gamma_k$ considered as a function
of $k$ is strictly positive, strictly decreasing, and strictly convex
(provided we are, as stated above, working with a reversible Markov chain).
Thus it makes sense to use estimators that use these properties.
The estimators implemented by the R function \texttt{initseq} and
described by \citet[Section~3.3]{practical} are conservative-consistent
in the sense of Theorem~3.2 of that section.

Figure~\ref{fig:gamma} (page~\pageref{fig:gamma})
shows the time series plot made by the R statement
<<label=figgamtoo,include=FALSE>>=
out <- initseq(x)
plot(seq(along = out$Gamma.pos) - 1, out$Gamma.pos,
        xlab = "k", ylab = expression(Gamma[k]), type = "l")
lines(seq(along = out$Gamma.dec) - 1, out$Gamma.dec, lty = "dotted")
lines(seq(along = out$Gamma.con) - 1, out$Gamma.con, lty = "dashed")
@
\begin{figure}
\begin{center}
<<label=figgam,fig=TRUE,echo=FALSE>>=
<<figgamtoo>>
@
\end{center}
\caption{Plot ``Big Gamma'' defined by \eqref{eq:little} and \eqref{eq:big}.
Solid line, initial positive sequence estimator.
Dotted line, initial monotone sequence estimator.
Dashed line, initial convex sequence estimator.}
\label{fig:gamma}
\end{figure}
One can use whichever curve one chooses, but now that
the \texttt{initseq} function makes the computation trivial, it makes
sense to use the initial convex sequence.

Of course, one is not interested in Figure~\ref{fig:gamma}, except
perhaps when explaining the methodology.  What is actually important
is the estimate of $\sigma^2$, which is given by
<<assvar>>=
out$var.con
(1 + rho) / (1 - rho) * 1 / (1 - rho^2)
@
where for comparison we have given the exact theoretical value of $\sigma^2$,
which, of course, is never available in a non-toy problem.

These initial sequence estimators seem, at first sight to be a competitor
for the method of batch means.  However, appearances can be deceiving.
The two methods are complementary.  The sequence of batch means is itself
a scalar-valued functional of a reversible Markov chain.  Hence the
initial sequence estimators can be applied to it.
<<batx>>=
blen <- 5
x.batch <- apply(matrix(x, nrow = blen), 2, mean)
bout <- initseq(x.batch)
@
Because the batch length is too short, the variance of the batch means
does not estimate $\sigma^2$.  We must account for the autocorrelation
of the batches, shown in Figure~\ref{fig:gambat}.
<<label=figgambattoo,include=FALSE>>=
plot(seq(along = bout$Gamma.con) - 1, bout$Gamma.con,
        xlab = "k", ylab = expression(Gamma[k]), type = "l")
@
\begin{figure}
\begin{center}
<<label=figgambat,fig=TRUE,echo=FALSE>>=
<<figgambattoo>>
@
\end{center}
\caption{Plot ``Big Gamma'' defined by \eqref{eq:little} and \eqref{eq:big}
for the sequence of batch means (batch length \Sexpr{blen}).
Only initial convex sequence estimator is shown.}
\label{fig:gambat}
\end{figure}
Because the the variance is proportional to one over the batch length,
we need to multiply by the batch length to estimate the $\sigma^2$
for the original series.
<<compvar>>=
out$var.con
bout$var.con * blen
@
Another way to look at this is that the MCMC estimator of $\mu$ is
either \texttt{mean(x)} or \texttt{mean(x.batch)}.  And the variance
must be divided by the sample size to give standard errors.  So either
<<ci-con>>=
mean(x) + c(-1, 1) * qnorm(0.975) * sqrt(out$var.con / length(x))
mean(x.batch) + c(-1, 1) * qnorm(0.975) * sqrt(bout$var.con / length(x.batch))
@
is an asymptotic 95\% confidence interval for $\mu$.  Just divide by
the relevant sample size.

\appendix

\section{Dot-dot-dot Versus Global Variables Versus Closures}
\label{sec:versus}

This appendix deals with three ways to pass information to a function
being passed to another R function (a higher-order function), for example when
\begin{itemize}
\item the function being passed is the objective function for an optimization
    done by the higher-order function, which optimizes
    (R function \texttt{optim} for example),
\item the function being passed is the integrand for an integration
    done by the higher-order function, which integrates
    (R function \texttt{integrate} for example),
\item the function being passed is the estimator of a parameter for a
    bootstrap done by the higher-order function, which simulates
    the (bootstrap approximation) of the sampling distribution of the
    estimator (R function \texttt{boot} in R package \texttt{boot},
    for example),
\item the function being passed is the log unnormalized density function
    for a simulation
    done by the higher-order function, which simulates the distribution
    having that unnormalized density
    (R function \texttt{metrop} in this package for example),
\end{itemize}
These ways are
\begin{itemize}
\item using dot-dot-dot (R syntax \verb@...@),
\item using global variables,
\item using closures, also called the function factory pattern.
\end{itemize}
The main body of this vignette uses the third option.  This appendix
explains them all and the virtues and vices of each.

\subsection{Dot-dot-dot}

The dot-dot-dot mechanism is fairly easy to use when only one function is passed
to the higher-order function, but does require care and more work to use.
It is even harder to deal with when more than one function is passed to
the higher-order function.  R functions \texttt{metrop} and \texttt{temper} in
this package can be passed two functions: the log unnormalized density
function and the output function.

\subsection{Only One Function Argument}

Previous versions of this vignette used the dot-dot-dot mechanism everywhere.
In those versions the log unnormalized density function was defined by
<<log.unnormalized.posterior-dot-dot-dot>>=
lupost <- function(beta, x, y) {
     eta <- as.numeric(x %*% beta)
     logp <- ifelse(eta < 0, eta - log1p(exp(eta)), - log1p(exp(- eta)))
     logq <- ifelse(eta < 0, - log1p(exp(eta)), - eta - log1p(exp(- eta)))
     logl <- sum(logp[y == 1]) + sum(logq[y == 0])
     return(logl - sum(beta^2) / 8)
}
@
rather than the way it is now done in the main text above.  Note that
everything is the same in the two definitions except here we have
\begin{verbatim}
lupost <- function(beta, x, y) {
\end{verbatim}
where in the main text we now have
\begin{verbatim}
lupost_factory <- function(x, y) function(beta) {
\end{verbatim}
and then we have to execute the function factory \verb@lupost_factory@
to make the \texttt{lupost} function.

But the main difference is that R function \texttt{lupost}
(the user written function specifying the log unnormalized posterior density)
\begin{itemize}
\item here has 3 arguments, \texttt{beta}, \texttt{x},
    and \texttt{y}, and the latter two must be passed via the dot-dot-dot
    mechanism, but
\item there has 1 argument, \texttt{beta}, and it
    just knows about \texttt{x} and \texttt{y} --- they are in its closure.
\end{itemize}

So to use this \texttt{lupost} function, we have to add arguments
\texttt{x} and \texttt{y} to each call to R function \texttt{metrop}.
For example
<<metropolis-try-dot-dot-dot>>=
out <- glm(y ~ x1 + x2 + x3 + x4, data = logit,
    family = binomial, x = TRUE)
x <- out$x
y <- out$y

out <- metrop(lupost, beta.init, 1e3, x = x, y = y)
out$accept
out <- metrop(out, scale = 0.1, x = x, y = y)
out$accept
out <- metrop(out, scale = 0.3, x = x, y = y)
out$accept
out <- metrop(out, scale = 0.5, x = x, y = y)
out$accept
out <- metrop(out, scale = 0.4, x = x, y = y)
out$accept
@
and so forth.

This method has the benefit that we do not have to explain function factories
and has the drawback that we need to keep remembering to add
\verb@x = x, y = y@ to each invocation of R function \texttt{metrop}.
It is unclear to me which pattern is more mysterious to naive users.

\subsection{More Than One Function Argument}

The situation becomes more complicated (see the Warning section of the help
pages for R functions \texttt{metrop} and \texttt{temper}) when more than
one function argument is passed to the higher-order function.  Then
\emph{they all must handle the \emph{same} dot-dot-dot arguments} whether
or not they want them.

So now we must define the output function as
<<outfun-dot-dot-dot>>=
outfun <- function(z, ...) c(z, z^2)
@
The \verb@...@ argument in the function signature is essential because
this function is going to be passed dot-dot-dot arguments \texttt{x}
and \texttt{y}, which it does not need and does not want, so it has to
allow for them (and then not use them).

Then we can continue
<<outfun-try-dot-dot-dot>>=
out <- metrop(out, nbatch = 100, blen = 100, outfun = outfun,
    x = x, y = y)
out$accept
@
and this works.  But we would have gotten an error about unused arguments
if we had defined the output function without the dot-dot-dot as we did
in the main text.

Your humble author was himself confused about this for years and so doubts
that naive R users will find it intuitive.

\subsection{Global Variables}

As every programmer knows global variables are easy to use and evil
\citep{c2}.  They are part of the way of R (or perhaps part of one of
the ways of R).  They are OK for ``very small or one-off programs'' \citep{c2}.
If you are going to throw the code away after one use, fine.
If you are going to give your code to other people or even use it
yourself six months later (after you have long forgotten the details),
then this method is evil and should be avoided.

In this method we define both functions passed to the higher-order function
without \verb@...@ and without using a function factory.  We already
in the preceding section of this appendix defined R objects \texttt{x} and
\texttt{y} as global variables (in the R global environment \verb@.GlobalEnv@).
They are global variables and we use them as such, defining
<<functions-global>>=
lupost <- function(beta) {
     eta <- as.numeric(x %*% beta)
     logp <- ifelse(eta < 0, eta - log1p(exp(eta)), - log1p(exp(- eta)))
     logq <- ifelse(eta < 0, - log1p(exp(eta)), - eta - log1p(exp(- eta)))
     logl <- sum(logp[y == 1]) + sum(logq[y == 0])
     return(logl - sum(beta^2) / 8)
}
outfun <- function(z) c(z, z^2)
@
Then the following works
<<doit-global>>=
out <- metrop(lupost, beta.init, 1e3)
out$accept
out <- metrop(out, scale = 0.1)
out$accept
out <- metrop(out, scale = 0.3)
out$accept
out <- metrop(out, scale = 0.5)
out$accept
out <- metrop(out, scale = 0.4)
out$accept
out <- metrop(out, nbatch = 100, blen = 100, outfun = outfun)
out$accept
@

We get the best of both worlds.  We don't need \verb@x = x, y = y@
and we don't need a function factory.

But if we change the name of the global variables to say
\texttt{modmat} and \texttt{resp} instead of \texttt{x} and \texttt{y},
then our code breaks.  R function \texttt{lupost} is looking up
global variables \texttt{x} and \texttt{y} \emph{under those names}
not under any other names.  So your code using this method is rigid and brittle
and probably unusable by others.

Note that if we are using either of the other methods, renaming is no problem.
Using dot-dot-dot we do
\begin{verbatim}
out <- metrop(out, scale = 0.1, x = modmat, y = resp)
\end{verbatim}
Using the function factory we do
\begin{verbatim}
lupost <- lupost_factory(modmat, resp)
\end{verbatim}
See Section~7.5.3 of \citet{basic} for more about this.

\subsection{Function Factory}

The terminology ``function factory pattern'' is apparently due to
\citet[Section~10.3.1]{wickham}.  But it is just a special case about
how closures work (in R and all other languages that have them).
Compare
<<fred>>=
fred <- function(y) function(x) x + y
fred(2)(3)
@
\citep[Section~6.5]{basic}) to
<<currying>>=
lupost_factory <- function(x, y) function(beta) {
    eta <- as.numeric(x %*% beta)
    logp <- ifelse(eta < 0, eta - log1p(exp(eta)), - log1p(exp(- eta)))
    logq <- ifelse(eta < 0, - log1p(exp(eta)), - eta - log1p(exp(- eta)))
    logl <- sum(logp[y == 1]) + sum(logq[y == 0])
    return(logl - sum(beta^2) / 8)
}
lupost <- lupost_factory(x, y)
lupost(beta.init)
@
We could also do the same calculation treating \verb@lupost_factory@
as just an ordinary curried function, like R function \texttt{fred}
in the preceding example,
<<currying-too>>=
lupost_factory(x, y)(beta.init)
@

So there is nothing mysterious about function factories.  They are just
one particular use of
the essence of functional programming (closures).
If you really understand functional programming, then you must understand this.

Long ago, when I switched from S to R,
I would not have considered that being an ``knowledgeable user'' of R
required knowledge of closures.  Now I do.  This is partly because other
functional languages like Scheme, Javascript, F\#, Clojure, and Haskell
have become
very popular for general computer programming.  So now we want to emphasize
that we can do in R what we can do in them.
But it is also partly because I now understand more about functional
programming.  I can now see that if you really understand closures,
then you don't need most other features of these programming languages
(including R).  Following \citet{crockford} we can say that all programming
languages (including R) have their good parts and their bad parts and we
should use the good features and avoid the bad features.  The best feature
is closures.  Crockford calls this the best idea ever put in a programming
language.  So we should use it a lot.

If I were starting to write the \texttt{mcmc} package today, I might leave
out the dot-dot-dot arguments of R functions \texttt{metrop},
\texttt{morph.metrop}, and \texttt{temper}.  That would mean that
users could not use the dot-dot-dot pattern.  They would be forced to
use the function factory pattern or the global variables pattern.
And if they had accepted that the global variables pattern is evil,
then they would have to use the function factory pattern.
(If you like the dot-dot-dot pattern, don't worry.  It is not going
to be removed from this package.  Backward compatibility trumps everything.)

\begin{thebibliography}{}

\bibitem[C2 Wiki(2013)]{c2}
C2 Wiki Contributors (2013).
\newblock Global Variables Are Bad.
\newblock \url{http://wiki.c2.com/?GlobalVariablesAreBad}.

\bibitem[Crockford(2008)]{crockford}
Crockford, D. (2008).
\newblock \emph{JavaScript: The Good Parts}.
\newblock O'Reilly, Sebastopol CA.

\bibitem[Gelman et al.(1996)Gelman, Roberts, and Gilks]{grg}
Gelman, A., G.~O. Roberts, and W.~R. Gilks (1996).
\newblock Efficient Metropolis jumping rules.
\newblock In \emph{Bayesian Statistics, 5 (Alicante, 1994)}, pp.~599--607.
  Oxford University Press.

\bibitem[Geyer(1992)]{practical}
Geyer, C.~J. (1992).
\newblock Practical Markov chain Monte Carlo (with discussion).
\newblock \emph{Statistical Science}, 7, 473--511.

\bibitem[Geyer(2006)]{bogosity}
Geyer, C. J. (2006).
\newblock On the bogosity of MCMC diagnostics.
\newblock \url{http://users.stat.umn.edu/~geyer/mcmc/diag.html}.

\bibitem[Geyer(2011)]{intro}
Geyer, C. J. (2011).
\newblock Introduction to Markov chain Monte Carlo.
\newblock In \emph{Handbook of Markov Chain Monte Carlo}, edited by
    Brooks, S., Gelman, A., Jones, G., and Meng, X.-L., pp.~3--48.
\newblock Chapman \& Hall/CRC, Boca Raton, FL.

\bibitem[Geyer(2018)]{basic}
Geyer, C.~J. (2018).
\newblock Stat 3701 lecture notes: Basics of R.
\newblock \url{http://www.stat.umn.edu/geyer/3701/notes/basic.html}.

\bibitem[Geyer and Thompson(1995)]{geyer-temp}
Geyer, C.~J. and E.~A. Thompson (1995).
\newblock Annealing Markov chain Monte Carlo with applications to
    ancestral inference.
\newblock \emph{Journal of the American Statistical Association}, 90, 909--920.

\bibitem[Wickham(2014)]{wickham}
Wickham, H. (2014).
\newblock \emph{Advanced R}.
\newblock Chapman \& Hall/CRC, Boca Raton, FL.
\newblock Also available on the web \url{http://adv-r.had.co.nz/}.

\end{thebibliography}

\end{document}

\documentclass{article}

\usepackage{natbib}
\usepackage{graphics}
\usepackage{amsmath,amssymb}
\usepackage{indentfirst}
\usepackage[utf8]{inputenc}
\usepackage[tableposition=top]{caption}
\usepackage{url}

\DeclareMathOperator{\var}{var}
\DeclareMathOperator{\cov}{cov}
\DeclareMathOperator{\E}{E}
\newcommand{\inner}[1]{\langle #1 \rangle}

% \VignetteIndexEntry{MCMC Morph Example}

\begin{document}

<<foo,include=FALSE,echo=FALSE>>=
options(keep.source = TRUE, width = 60)
foo <- packageDescription("mcmc")
@

\title{Morphometric MCMC (mcmc Package Ver.~\Sexpr{foo$Version})}
% $ (Just to make emacs syntax highlighting work properly)
\author{Leif T. Johnson \and Charles J. Geyer}
\maketitle

\section{Overview}

This is an example how to use morphometric Markov chains as implemented in
the \verb@mcmc@ package in R.

Let $X$ be an $\mathbb{R}^k$ valued random variable with probability density
function, $f_X$.  Let $g$ be a diffeomorphism, and $Y=g(X)$.  Then the
probability density function of $Y$, $f_Y$ is given by
\begin{equation}\label{eq:def-fy}
  f_Y(y) = f_X\bigl(g^{-1}(y)\bigr) \det\bigl( \nabla g^{-1}(y) \bigr).
\end{equation}
Since $g$ is a diffeomorphism, we can draw inference about $X$ from information
about $Y$ (and vice versa).

It is not unusual for $f_X$ to either be known only up to a normalizing
constant, or to be analytically intractable in other ways --- such as
being high dimensional.
A common solution to this problem is to use Markov chain
Monte Carlo (MCMC) methods to learn about $f_X$.

When using MCMC, a primary concern of the practitioner should be the question
``Does the Markov chain converge fast enough to be useful?''  One very useful
convergence rate is called \emph{geometrically ergodic}
\citep[Chapter~1]{johnson-thesis}.

The \texttt{mcmc} package implements the Metropolis random-walk algorithm for
arbitrary log unnormalized probability densities.  But the Metropolis
random-walk algorithm does not always perform well.  As is demonstrated in
\citet{johnson-geyer}, for $f_X$ and $f_Y$ related by diffeomorphism as in
\eqref{eq:def-fy}, a Metropolis random-walk for $f_Y$ can be geometrically
ergodic
even though a Metropolis random-walk for $f_X$ is not.
Since the transformation is
one-to-one, inference about $f_X$ can be drawn from the Markov chain for $f_Y$.

The \texttt{morph.metrop} and \texttt{morph} functions in the \texttt{mcmc}
package provide this functionality, and this vignette gives a demonstration
on how to use them.

\section{T Distribution} \label{sec:toy}

We start with a univariate example, which is a Student $t$ distribution
with three degrees of freedom.
Of course, one doesn't need MCMC to simulate this distribution
(the R function \texttt{rt} does that), so this is just a toy problem.
But it does illustrate some aspects of using variable transformation.

A necessary condition for geometric ergodicity of a random-walk Metropolis
algorithm is that the target density $\pi$ have a moment generating function
\citep{jarner-tweedie}.
For a univariate target density, which we have in this section,
a sufficient condition for geometric ergodicity of a random-walk Metropolis
algorithm is that the target density $\pi$ be exponentially light
\citet{mengersen-tweedie}.
Thus if we do not use variable transformation,
the Markov chain simulated by the \texttt{metrop} function will not
be geometrically ergodic.
\citet[Example 4.2]{johnson-geyer} show that a $t$ distribution is
sub-exponentially light.  Hence using the transformations
described in their Corollaries~1 and~2 will induce a target density
$\pi_\gamma$ for which a Metropolis random-walk will be geometrically
ergodic.
using the transformation described as $h_2$ in
\citet[Corollary~2]{johnson-geyer} will induce a target density for which a
Metropolis random-walk will be geometrically ergodic.

Passing a positive value for \texttt{b} to \texttt{morph} function will
create the aforementioned transformation, $h_2$.  It's as simple as
<<>>=
library(mcmc)
h2 <- morph(b=1)
@
We can now see the induced density.  Note that \texttt{morph} works for
log unnormalized densities, so we need exponentiate the induced density to
plot it on the usual scale.
<<>>=
lud <- function(x) dt(x, df=3, log=TRUE)
lud.induced <- h2$lud(lud)
@
We can plot the two densities,
<<fig=TRUE>>=
curve(exp(Vectorize(lud.induced)(x)), from = -3, to = 3, lty = 2,
    xlab = "t", ylab = "density")
curve(exp(lud(x)), add = TRUE)
legend("topright", c("t density", "induced density"), lty=1:2)
@

The \texttt{Vectorize} in this example is necessary because
the function \texttt{lud.induced} is not vectorized.
Instead, it treats any vector passed as a single input, which
is rescaled (using the specified diffeomorphism) and passed to
\texttt{lud}.  Compare the behavior of \texttt{lud} and
\texttt{lud.induced} in the following example.
<<>>=
lud(1:4)
lud(1)
foo <- try(lud.induced(1:4))
class(foo)
cat(foo, "\n")
lud.induced(1)
@
Because the function \texttt{dt} is vectorized, the function \texttt{lud}
is also vectorized, mapping vectors to vectors,
whereas the function \texttt{lud.induced} is not vectorized,
mapping vectors to scalars.

Before we start using random numbers, we set the seed of the random number
generator so this document always produces the same results.
<<set-seed>>=
set.seed(42)
@
To change the results, change the seed or delete the \texttt{set.seed}
statement.

Running a Markov chain for the induced density is done with
\texttt{morph.metrop}.
<<>>=
out <- morph.metrop(lud, 0, blen=100, nbatch=100, morph=morph(b=1))
@
The content of \texttt{out\$batch} is on the scale of used by
\texttt{lud}.  Once the transformation has been set, no adjustment is
needed (unless you want to change transformations).  We start by adjusting
the scale.
<<>>=
# adjust scale to find a roughly 20% acceptance rate
out$accept
@
An acceptance rate of \Sexpr{round(100 * out$accept, 1)}\%
%$ to fix emacs highlighting
is probably too high.  By increasing the scale of the proposal distribution
we can bring it down towards 20\%.
<<>>=
out <- morph.metrop(out, scale=4)
out$accept
@
We now use this Markov chain to estimate the expectation of the target
distribution.
But first we need to check whether our batch length is good.
The following code
<<label=fig0too,include=FALSE>>=
acf(out$batch)
@
makes the autocorrelation plot (Figure~\ref{fig:fig0}).
\begin{figure}
\begin{center}
<<label=fig0,fig=TRUE,echo=FALSE>>=
<<fig0too>>
@
\end{center}
\caption{Autocorrelation plot for the sequence of batch means.}
\label{fig:fig0}
\end{figure}
It looks like there is no significant autocorrelation among the batches
so the following produces a valid confidence interval for the true
unknown mean of the target distribution (since this is a toy problem
we actually know the true ``unknown'' mean is zero, but we pretend we
don't know that for the purposes of the toy problem)
<<>>=
t.test(out$batch)
@
If we want a point estimate and a Monte Carlo standard error, those are
<<>>=
colMeans(out$batch)
apply(out$batch, 2, sd) / sqrt(out$nbatch)
@
If a shorter confidence interval is desired, the Markov chain can be run
longer (increase either the number of batches or the batch length, or both).

Note that when calculating our estimate and the Monte Carlo standard error
we are not concerned with what was happening on the transformed scale.  The
\texttt{morph.metrop} function seamlessly does this for us.

\subsection{Comparison of Morphed and Unmorphed}

To show the utility of the transformation, we will study the behavior
of the Markov chain with and without the transformation for the same
problem as in the preceding section.
We will consider two different estimation methods.
\begin{enumerate}
\item \label{enum:rw} Estimate the mean of the target distribution
  using a random-walk Metropolis algorithm implemented by the \texttt{metrop}
  function.  \citet{jarner-roberts} demonstrate that a central limit
  theorem does not hold for these estimates.
\item \label{enum:rw-induced} Estimate the mean of the target distribution
  using a random-walk Metropolis algorithm implemented by the
  \texttt{morph.metrop} function with argument \texttt{morph = morph(b=1)}.
  \citet{johnson-geyer} demonstrate that a central limit
  theorem does hold for these estimates.
\end{enumerate}

For the former, we need to adjust the scale.
<<unmorph-metrop-adjust>>=
out.unmorph <- metrop(lud, 0, blen=1000, nbatch=1)
out.unmorph$accept
out.unmorph <- metrop(out.unmorph, scale=4)
out.unmorph$accept
out.unmorph <- metrop(out.unmorph, scale=6)
out.unmorph$accept
@
A scale of 6 appears to be about right.  Now we do a long run for
this sampler.
Because this run takes longer than CRAN vingettes are supposed to
take, we save the results to a file
and load the results from this file if it already exists.
<<unmorph-metrop-t-long-run>>=
lout <- suppressWarnings(try(load("morph1.rda"), silent = TRUE))
if (inherits(lout, "try-error")) {
    out.unmorph <- metrop(out.unmorph, blen = 1e5, nbatch = 1e3)
    save(out.unmorph, file = "morph1.rda")
} else {
    .Random.seed <- out.unmorph$final.seed
}
out.unmorph$accept
@

Let's look at the distribution of batch means.
The following code
<<label=fig4too,include=FALSE>>=
foo <- as.vector(out.unmorph$batch)
qqnorm(foo)
qqline(foo)
@
makes a Q-Q plot of the batch means (Figure~\ref{fig:fig4}).
\begin{figure}
\begin{center}
<<label=fig4,fig=TRUE,echo=FALSE>>=
<<fig4too>>
@
\end{center}
\caption{Q-Q plot of batch means (batch length \Sexpr{out.unmorph$blen})
for the unmorphed chain.}
\label{fig:fig4}
\end{figure}
We see bad behavior of the unmorphed chain.  These batch means
(or at least some batch means for sufficiently long batch length) should
look normally distributed, and these don't.  Not even close.
We do a formal test just to check our interpretation of the plot
<<shapiro-wilk>>=
shapiro.test(foo)
@

Now for comparison, we check the morphed chain.
<<morph-metrop-t-long-run>>=
lout <- suppressWarnings(try(load("morph2.rda"), silent = TRUE))
if (inherits(lout, "try-error")) {
    out.morph <- morph.metrop(out, blen = 1e5, nbatch = 1e3)
    save(out.morph, file = "morph2.rda")
} else {
    .Random.seed <- out.morph$final.seed
}
out.morph$accept
@

The following code
<<label=fig5too,include=FALSE>>=
foo <- as.vector(out.morph$batch)
qqnorm(foo)
qqline(foo)
@
makes a Q-Q plot of the batch means (Figure~\ref{fig:fig5}).
\begin{figure}
\begin{center}
<<label=fig5,fig=TRUE,echo=FALSE>>=
<<fig5too>>
@
\end{center}
\caption{Q-Q plot of batch means (batch length \Sexpr{out.unmorph$blen})
for the morphed chain.}
\label{fig:fig5}
\end{figure}
We see good behavior of the morphed chain.  These batch means do
look normally distributed.
We do a formal test just to check our interpretation of the plot
<<shapiro-wilk>>=
shapiro.test(foo)
@

\section{Binomial Distribution with a Conjugate Prior}

We demonstrate a morphometric Markov chain using the \texttt{UCBAdmisions}
data set included in \texttt{R}, (use \texttt{help(UCBAdmissions)} to see
details of this data set).  We will model the probability of a student
being admitted or rejected, using the sex of the student and the department
that the student applied to as predictor variables.  For our prior, we
naively assume that 30\% of all students are admitted, independent of sex
or department.  As this is a naive prior, we will only add 5 students to
each gender-department combination.  This will not give the prior much
weight, most of the information in the posterior distribution will be from
the data.

If we have $L$ observations from a multinomial distribution, then using a
multinomial logit-link, with model matrices $M^1,\dots,M^L$, regression
parameter $\beta$, observed counts $Y^1,\dots,Y^N$ with observed sample
sizes $N^1,\dots,N^L$ and prior probabilities $\xi^1, \dots, \xi^L$ and
prior ``sample sizes'' $\nu^1,\dots,\nu^L$ then the posterior distribution
of $\beta$ is given by \citep[Sec. 5.1.2]{johnson-thesis}
\begin{equation}\label{eq:mult-post-conj-complicated}
\pi(\beta|y,n,\xi,\nu) \propto \exp\biggl\{ \sum_{l=1}^L \inner{y^l + \xi^l
    \nu^l, M^l \beta} - (n^l + \nu^l) \log\bigl(
    \sum_j e^{M_{j\cdot} \beta} \bigr) \biggr\}
\end{equation}
where $\inner{a, b}$ denotes the usual inner product between vectors $a$
and $b$.  For our application, we can simplify this in two ways.

First, we use the posterior counts instead of the sum of the prior and data
counts, i.e. use $y^{*l} = y^l + \xi^l \nu^l$ and $n^{*l} = n^l + \nu^l$.

Second, to avoid having a direction of recession in $\pi(\beta|\cdot)$, we
need to fix the elements of $\beta$ that correspond with one of the
response categories.  Since we are going to fitting a binomial response, if
we set these elements of $\beta$ to be $0$, we may then replace the
sequence of model matrices with a single model matrix; $M$ instead of
$M^1,\dots,M^L$.  The $l$-th row of $M$ will correspond to $M^l$.  Label
the two response categories $A$ and $B$.  Without loss of generality, we
will fix the elements of $\beta$ corresponding to category $B$ to 0.

Let $x_1,\dots,x_L$ represent the posterior counts of category $A$, and
$\beta^*$ represent the corresponding elements of $\beta$ --- these are the
elements of $\beta$ we did not fix as 0.  The meaning of
$n^{*1},\dots,n^{*L}$ is unchanged.  Then our simplified unnormalized
posterior density is
\begin{equation}\label{eq:simplified-posterior}
  \pi(\beta|x,n^*) \propto
  \exp\biggl\{
    \inner{x, M \beta^*}
    -
    \sum_{l=1}^L n^{*l} \log\bigl(1 + e^{(M \beta^*)_l}\bigr)
  \biggr\}.
\end{equation}
This can be computed with a very simple \texttt{R} function, we implement
it in log form.
<<def-posterior-binom>>=
lud.binom <- function(beta, M, x, n) {
  MB <- M %*% beta
  sum(x * MB) - sum(n * log(1 + exp(MB)))
}
@

Now that we have a function to calculate a log-unnormalized posterior
density, we can run the Markov chain.  To that, we need the model matrix.
First we convert the \texttt{UCAdmissions} data to a \texttt{data.frame}.
<<convert>>=
dat <- as.data.frame(UCBAdmissions)
dat.split <- split(dat, dat$Admit)
dat.split <- lapply(dat.split,
                    function(d) {
                      val <- as.character(d$Admit[1])
                      d["Admit"] <- NULL
                      names(d)[names(d) == "Freq"] <- val
                      d
                    })
dat <- merge(dat.split[[1]], dat.split[[2]])
@

Next we build the model matrix.  Our model specification allows for an
interaction between gender and department, even though our prior assumes
that they are independent.
<<build-model-matrix>>=
formula <- cbind(Admitted, Rejected) ~ (Gender + Dept)^2
mf <- model.frame(formula, dat)
M <- model.matrix(formula, mf)
@

As stated above, we will take $\nu = 5$ and $\xi=0.30$.  That is, we will
add 5 students to each gender-department combination, where each
combination has a 30\% acceptance rate.
<<>>=
xi <- 0.30
nu <- 5
@

<<lud-binom>>=
lud.berkeley <- function(B)
  lud.binom(B, M, dat$Admitted + xi * nu, dat$Admitted + dat$Rejected + nu)
@

This function is suitable for passing to \texttt{metrop} or
\texttt{morph.metrop}.  We know that using \texttt{morph.metrop} with
\texttt{morph=morph(p=3)} will run a geometrically ergodic Markov chain
\citep{johnson-geyer}.
<<>>=
berkeley.out <- morph.metrop(lud.berkeley, rep(0, ncol(M)), blen=1000,
                             nbatch=1, scale=0.1, morph=morph(p=3))
berkeley.out$accept
berkeley.out <- morph.metrop(berkeley.out, scale=0.05)
berkeley.out$accept
berkeley.out <- morph.metrop(berkeley.out, scale=0.02)
berkeley.out$accept
berkeley.out <- morph.metrop(berkeley.out, blen=10000)
berkeley.out$accept
@

<<>>=
berkeley.out <- morph.metrop(berkeley.out, blen=1, nbatch=100000)
@

Estimate the posterior mean acceptance probabilities for each
gender-department combination.
<<>>=
beta <- setNames(colMeans(berkeley.out$batch), colnames(M))
MB <- M %*% beta
dat$p <- dat$Admitted / (dat$Admitted + dat$Rejected)
dat$p.post <- exp(MB) / (1 + exp(MB))
dat
@
The small difference between the data and posterior probabilities is
expected, our prior was given very little weight.  Using
\texttt{morph.metrop} with the setting \texttt{morph=morph(p=3)} in this
setting is an efficient way of sampling from the posterior distribution.

We can also compare the posterior distribution of admittance
probability for each gender-department combination.
Table~\ref{tab:post-quant} gives the 5\% and 95\% quantiles for the
posterior distribution of the admittance probabilities for each
gender-department combination.  Figure~\ref{fig:posterior-probs} gives
the same quantiles, plus the mean posterior-probability for each
gender-department combination.  From these we can see that for each
department, there is considerable overlap of the distributions of
probabilities for males and females.

<<calculate-posterior-probabilities>>=
posterior.probabilities <-
  t(apply(berkeley.out$batch, 1,
          function(r) {
            eMB <- exp(M %*% r)
            eMB / (1 + eMB)
          }))
quants <- apply(posterior.probabilities, 2, quantile, prob=c(0.05, 0.95))
quants.str <- matrix(apply(quants, 2,
                           function(r) sprintf("[%0.2f, %0.2f]", r[1], r[2])),
                     nrow=2, byrow=TRUE)

@

\begin{table}[ht]
  \caption{5\% and 95\% posterior quantiles for admittance probability
    for each gender-department combination}
  \begin{center}
\begin{tabular}{|l|c|c|c|c|c|c|}
  \hline
 Gender & Dept. A & Dept. B & Dept. C & Dept. D & Dept. E. & Dept. F \\
 \hline
 Female & \Sexpr{paste(quants.str[1, 1:6], collapse=" & ")} \\
 Male & \Sexpr{paste(quants.str[2, 1:6], collapse=" & ")} \\
 \hline
\end{tabular}
\label{tab:post-quant}
\end{center}
\end{table}

\begin{figure}
\begin{center}
<<label=fig1,fig=TRUE,echo=FALSE>>=
x <- (0:5) * 2 + 1
plot(x[c(1, 6)] + 0.5 * c(-1, 1), 0:1,
     xlab="Department", ylab="Probability", xaxt="n", type="n")
axis(1, x, LETTERS[1:6])
for(i in 1:6) {
  lines((x[i]-0.25)*c(1, 1), quants[1:2, i], lwd=2, col="gray")
  lines((x[i] + 0.25) * c(1, 1), quants[1:2, i + 6], lwd=2, col="gray")
  points(x[i] + 0.25 * c(-1, 1), dat$p.post[i + c(0, 6)], pch=c("F", "M"))
}
@
\end{center}
\caption{Posterior 5\% and 95\% quantiles and mean, by department and gender.}
\label{fig:posterior-probs}
\end{figure}

\section{Cauchy Location-Scale Model}

We are going to do a Cauchy location-scale family objective Bayesianly.

\subsection{Data}

First we generate some data.
<<cauchy-data>>=
n <- 15
mu0 <- 50
sigma0 <- 10
x <- rcauchy(n, mu0, sigma0)
round(sort(x), 1)
@
\texttt{mu0} and \texttt{sigma0} are the true unknown parameter values
(since the data are simulated we actually know these ``unknown'' parameter
values, but we must pretend we don't know them and estimate them).

\subsection{Prior}

The standard objective prior distribution for this situation
(insofar as any prior distribution can be said to be an objective standard)
is the improper prior
$$
   g(\mu, \sigma) = \frac{1}{\sigma}
$$
which is right Haar measure for the location-scale group, and is the
standard prior that comes from the group invariance argument
\citep[Section~3.2]{kass-wasserman}.

\subsection{Log Unnormalized Posterior}

We need a function whose argument is a two-vector
<<cauchy-log-unnormalized-posterior>>=
lup <- function(theta) {
    if (any(is.na(theta)))
        stop("NA or NaN in input to log unnormalized density function")
    mu <- theta[1]
    sigma <- theta[2]
    if (sigma <= 0) return(-Inf)
    if (any(! is.finite(theta))) return(-Inf)
    result <- sum(dcauchy(x, mu, sigma, log = TRUE)) - log(sigma)
    if (! is.finite(result)) {
        warning(paste("Oops!  mu = ", mu, "and sigma =", sigma))
    }
    return(result)
}
@

\subsection{Laplace Approximation}

To have some idea what we are doing, we first maximize the log unnormalized
posterior.  To do it helps to have good starting points for the optimization.
Robust estimators of location and scale are
<<cauchy-robust>>=
mu.twiddle <- median(x)
sigma.twiddle <- IQR(x)
c(mu.twiddle, sigma.twiddle)
@
The posterior mode is
<<cauchy-posterior-mode>>=
oout <- optim(c(mu.twiddle, sigma.twiddle), lup,
    control = list(fnscale = -1), hessian = TRUE)
stopifnot(oout$convergence == 0)
mu.hat <- oout$par[1]
sigma.hat <- oout$par[2]
c(mu.hat, sigma.hat)
@
and the hessian evaluated at the posterior mode (calculated by
\texttt{optim} using finite differences) is
<<cauchy-hessian>>=
oout$hessian
@
The hessian is nearly diagonal and one can check that theoretically
is exactly diagonal.  Thus approximate (asymptotic) posterior standard
deviations are
<<cauchy-se>>=
sqrt(- 1 / diag(oout$hessian))
@

\subsection{Theory}

To use the theory in \citet{johnson-geyer} we must verify that the
target distribution (the unnormalized posterior) is everywhere positive,
and it isn't (it is zero for $\sigma \le 0$).  We tried making $\log(\sigma)$
the parameter but this didn't work either because $\log(\sigma)$ goes to
infinity so slowly that this stretches out the tails so much that the
transformations introduced by \citet{johnson-geyer} can't pull them back
in again.  We do know \citep[Example~3.4]{johnson-geyer} that if we fix
$\sigma$ this is a sub-exponentially light target distribution.  Letting
$\sigma$ vary can only make this worse.  Thus, if we don't do anything
and just use the \texttt{metrop} function, then performance will be very
bad.  So we are going to use the transformations and the \texttt{morph.metrop}
function, even though the theory that motivates them does not hold.

\subsection{Morph}

We want to center the transformation at the posterior mode, and use a
radius $r$ that doesn't transform until several approximate standard deviations
<<cauchy-doit>>=
moo <- morph(b = 0.5, r = 7, center = c(mu.hat, sigma.hat))
mout <- morph.metrop(lup, c(mu.hat, sigma.hat), 1e4,
    scale = 3, morph = moo)
mout$accept
mout <- morph.metrop(mout)
@
Good enough.  An attempt to increase the scale led to error when the
transformation functions overflowed.  Can't take steps too big with this
stuff.
The following code
<<label=cfig1too,include=FALSE>>=
acf(mout$batch)
@
makes an autocorrelation plot (Figure~\ref{fig:cfig1}).
\begin{figure}
\begin{center}
<<label=cfig1,fig=TRUE,echo=FALSE>>=
<<cfig1too>>
@
\end{center}
\caption{Autocorrelation plot.  First component is $\mu$, second is $\sigma$.}
\label{fig:cfig1}
\end{figure}
It looks like lag 10 to 15 is enough to get near independence.

Now we want to make marginal density plots.
If we just feed our MCMC output to the R function \texttt{density}
it undersmooths because it expects independent and identically distributed
data rather than autocorrelated
data.  Thus we feed it subsampled, nearly uncorrelated data to select
the bandwidth and then use that bandwidth on the full data.  Here's
how that works.
The following code
<<label=cfig2too,include=FALSE>>=
mu <- mout$batch[ , 1]
i <- seq(1, mout$nbatch, by = 15)
out.sub <- density(mu[i])
out <- density(mu, bw = out.sub$bw)
plot(out)
@
makes the density plot (Figure~\ref{fig:cfig2}).
\begin{figure}
\begin{center}
<<label=cfig2,fig=TRUE,echo=FALSE>>=
<<cfig2too>>
@
\end{center}
\caption{Density plot for the marginal posterior for $\mu$.}
\label{fig:cfig2}
\end{figure}
And a similar plot for $\sigma$ (Figure~\ref{fig:cfig3})
\begin{figure}
\begin{center}
<<label=cfig3,fig=TRUE,echo=FALSE>>=
sigma <- mout$batch[ , 2]
out.sub <- density(sigma[i])
out <- density(sigma, bw = out.sub$bw)
plot(out)
@
\end{center}
\caption{Density plot for the marginal posterior for $\sigma$.}
\label{fig:cfig3}
\end{figure}

\begin{thebibliography}{}

\bibitem[Jarner and Roberts(2007)]{jarner-roberts}
Jarner, S.F., and G.O. Roberts (2007).
\newblock Convergence of heavy-tailed Monte Carlo Markov chain algorithms.
\newblock \emph{Scandinavian Journal of Statistics}, 34, 781--815.

\bibitem[Jarner and Tweedie(2003)]{jarner-tweedie}
Jarner, S.~F., and Tweedie, R.~L. (2003).
\newblock Necessary conditions for geometric and polynomial ergodicity of
    random-walk-type Markov chains.
\newblock \emph{Bernoulli}, 9, 559--578.

\bibitem[Johnson(2011)]{johnson-thesis}
Johnson, L.~T. (2011).
\newblock Geometric Ergodicity of a Random-Walk Metropolis Algorithm via
  Variable Transformation and Computer Aided Reasoning in Statistics.
\newblock Ph.~D. thesis.  University of Minesota.
  \url{http://purl.umn.edu/113140}

\bibitem[Johnson and Geyer(submitted)]{johnson-geyer}
Johnson, L.~T., and C.~J. Geyer (submitted).
\newblock Variable Transformation to Obtain Geometric Ergodicity
    in the Random-walk Metropolis Algorithm.
\newblock Revised and resubmitted to \emph{Annals of Statistics}.

\bibitem[Kass and Wasserman(1996)]{kass-wasserman}
Kass, R.~E., and Wasserman, L. (1996).
\newblock Formal rules for selecting prior distributions: A review and
    annotated bibliography.
\newblock \emph{Journal of the American Statistical Association},
    435, 1343--1370.

\bibitem[Mengersen and Tweedie(1996)]{mengersen-tweedie}
  Mengersen, K.L., ad R. L. Tweedie (1996).
\newblock Rates of convergence of the Hastings and Metropolis algorithms.
\newblock \emph{Annals of Statistics}, 24, 101--121.

\end{thebibliography}

\end{document}

\name{morph}
\encoding{UTF-8}
\alias{morph}
\alias{morph.identity}
\title{Variable Transformation}
\description{
  Utility functions for variable transformation.
}
\usage{
morph(b, r, p, center)
morph.identity()
}
\arguments{
  \item{b}{Positive real number.  May be missing.}
  \item{r}{Non-negative real number.  May be missing.  If \code{p} is
    specified, \code{r} defaults to 0.}
  \item{p}{Real number strictly greater than 2.  May be missing.  If
    \code{r} is specified, \code{p} defaults to 3.}
  \item{center}{Real scalar or vector.  May be missing.  If
    \code{center} is a vector it should be the same length of the state
    of the Markov chain, \code{center} defaults to 0}
}
\section{Warning}{
  The equations for the returned \code{transform} function (see below)
  do not have a general analytical solution when \code{p} is not equal
  to 3.  This implementation uses numerical approximation to calculate
  \code{transform} when \code{p} is not equal to 3.  If computation
  speed is a factor, it is advisable to use \code{p=3}.  This is not a
  factor when using \code{\link{morph.metrop}}, as \code{transform} is
  only called once during setup, and not at all while running the Markov chain.
}
\details{
  The \code{morph} function facilitates using variable transformations
  by providing functions to (using \eqn{X} for the original random
  variable with the pdf \eqn{f_X}{f.X}, and \eqn{Y} for the transformed
  random variable with the pdf \eqn{f_Y}{f.Y}):
  \itemize{
    \item Calculate the log unnormalized probability density for \eqn{Y}
    induced by the transformation.
    \item Transform an arbitrary function of \eqn{X} to a function of
    \eqn{Y}.
    \item Transform values of \eqn{X} to values of \eqn{Y}.
    \item Transform values of \eqn{Y} to values of \eqn{X}
       (the inverse transformation).
  }
  for a select few transformations.

  \code{morph.identity} implements the identity transformation,
  \eqn{Y=X}.
  
  The parameters \code{r}, \code{p}, \code{b} and \code{center} specify the
  transformation function.  In all cases, \code{center} gives the center
  of the transformation, which is the value \eqn{c} in the equation
  \deqn{Y = f(X - c).}  If no parameters are specified, the identity
  transformation, \eqn{Y=X}, is used.
  
  The parameters \code{r}, \code{p} and \code{b} specify a function
  \eqn{g}, which is a monotonically increasing bijection from the
  non-negative reals to the non-negative reals.  Then
  \deqn{f(X) = g\bigl(|X|\bigr) \frac{X}{|X|}}{f(X) = g(|X|) * X / |X|}
  where \eqn{|X|} represents the Euclidean norm of the vector \eqn{X}.
  The inverse function is given by
  \deqn{f^{-1}(Y) = g^{-1}\bigl(|Y|\bigr) \frac{Y}{|Y|}.}{f^{-1}(Y) = g^{-1}(|Y|) * Y / |Y|.}

  The parameters \code{r} and \code{p} are used to define the function
  \deqn{g_1(x) = x + (x-r)^p I(x > r)}{g1(x) = x + (x-r)^p * I(x > r)}
  where \eqn{I( \cdot )}{I(•)}  is the indicator
  function.  We require that \code{r} is non-negative and \code{p} is
  strictly greater than 2.  The parameter \code{b} is used to define the
  function
  \deqn{g_2(x) = \bigl(e^{bx} - e / 3\bigr) I(x > \frac{1}{b}) + 
    \bigl(x^3 b^3 e / 6 + x b e / 2\bigr) I(x \leq
    \frac{1}{b})}{
    g2(x) = (exp(b * x) - exp(1) / 3) * I(x > 1 / b) +
            (x^3 * b^3 exp(1) / 6 + x * b * exp(1) / 2) * I(x <= 1 / b).}
  We require that \eqn{b} is positive.

  The parameters \code{r}, \code{p} and \code{b} specify \eqn{f^{-1}} in
  the following manner:
  \itemize{
    \item  If one or both of \code{r} and \code{p} is specified, and \code{b}
    is not specified, then \deqn{f^{-1}(X) = g_1(|X|)
      \frac{X}{|X|}.}{f^{-1}(X) = g1(|X|) * X / |X|.}  If only
    \code{r} is specified, \code{p = 3} is used.  If only \code{p} is specified,
    \code{r = 0} is used.

    \item If only \code{b} is specified, then \deqn{f^{-1}(X) = g_2(|X|)
      \frac{X}{|X|}.}{f^{-1}(X) = g2(|X|) * X / |X|.}

    \item If one or both of \code{r} and \code{p} is specified, and \code{b} is
    also specified, then \deqn{f^{-1}(X) = g_2(g_1(|X|))
      \frac{X}{|X|}.}{f^{-1}(X) = g2(g1(|X|)) * X / |X|.}
  }
}
\value{
  a list containing the functions
  \itemize{
  \item \code{outfun(f)}, a function that operates on functions.
  \code{outfun(f)} returns the function \code{function(state, ...)
    f(inverse(state), ...)}.
  \item \code{inverse}, the inverse transformation function.
  \item \code{transform}, the transformation function.
  \item \code{lud}, a function that operates on functions.  As input,
  \code{lud} takes a function that calculates a log unnormalized
  probability density, and returns a function that calculates the
  log unnormalized density by transforming a random variable using the
  \code{transform} function.  \code{lud(f) = function(state, ...)
  f(inverse(state), ...) + log.jacobian(state)}, where
  \code{log.jacobian} represents the function that calculate the log
  Jacobian of the transformation.  \code{log.jacobian} is not returned.
}
}
\examples{

# use an exponential transformation, centered at 100.
b1 <- morph(b=1, center=100)
# original log unnormalized density is from a t distribution with 3
# degrees of freedom, centered at 100.
lud.transformed <- b1$lud(function(x) dt(x - 100, df=3, log=TRUE))
d.transformed <- Vectorize(function(x) exp(lud.transformed(x)))
\dontrun{
curve(d.transformed, from=-3, to=3, ylab="Induced Density")
}
}
\seealso{
  \code{\link{morph.metrop}}
}
\keyword{misc}

\name{foo}
\docType{data}
\alias{foo}
\title{Simulated logistic regression data.}
\description{
  Like it says
}
\usage{data(foo)}
\format{
  A data frame with variables
   \describe{
    \item{x1}{quantitative predictor.}
    \item{x2}{quantitative predictor.}
    \item{x3}{quantitative predictor.}
    \item{y}{Bernoulli response.}
  }
}
\examples{
library(mcmc)
data(foo)
out <- glm(y ~ x1 + x2 + x3, family = binomial, data = foo)
summary(out)
}
\keyword{datasets}

\name{morph.metrop}
\alias{morph.metrop}
\alias{morph.metrop.function}
\alias{morph.metrop.morph.metropolis}
\title{Morphometric Metropolis Algorithm}
\description{
  Markov chain Monte Carlo for continuous random vector using a
  Metropolis algorithm for an induced density.
}
\usage{
morph.metrop(obj, initial, nbatch, blen = 1, nspac = 1, scale = 1,
  outfun, debug = FALSE, morph, ...)
}
\arguments{
  \item{obj}{see \code{\link{metrop}}.}
  \item{initial}{see \code{\link{metrop}}.}
  \item{nbatch}{see \code{\link{metrop}}.}
  \item{blen}{see \code{\link{metrop}}.}
  \item{nspac}{see \code{\link{metrop}}.}
  \item{scale}{see \code{\link{metrop}}.}
  \item{outfun}{unlike for \code{\link{metrop}} must be a function or missing;
    if missing the identity function, \code{function(x) x}, is used.}
  \item{debug}{see \code{\link{metrop}}.}
  \item{morph}{morph object used for transformations.  See \code{\link{morph}}.}
  \item{...}{see \code{\link{metrop}}.}
}
\details{
  \code{morph.metrop} implements morphometric methods for Markov
  chains.  The caller specifies a log unnormalized probability density
  and a transformation.  The transformation specified by the
  \code{morph} parameter is used to induce a new log unnormalized
  probability   density, a Metropolis algorithm is
  run for the induced density.  The Markov chain is transformed back to
  the original scale.  Running the Metropolis algorithm for the induced
  density, instead of the original density, can result in a Markov chain
  with better convergence properties.  For more details see Johnson and Geyer
  (submitted).  Except for \code{morph}, all parameters are
  passed to \code{\link{metrop}}, transformed when necessary.  The
  \code{scale} parameter is \emph{not} transformed.

  If \eqn{X} is a real vector valued continuous random variable, and
  \eqn{Y = f(X)} where \eqn{f} is a diffeomorphism, then the pdf of
  \eqn{Y} is given by \deqn{f_Y(y) = f_X(f^{-1}(y)) | \nabla f^{-1}(y)
  |}{ fY(y) = fX(f^{-1}(y)) | del f^{-1}(y) |} where \eqn{f_X}{fX} is
  the pdf of \eqn{X} and \eqn{\nabla f^{-1}}{del f^{-1}} is the Jacobian
  of \eqn{f^{-1}}.  Because \eqn{f} is a diffeomorphism, a Markov chain
  for \eqn{f_Y}{fY} may be transformed into a Markov chain for
  \eqn{f_X}{fX}.  Furthermore, these Markov chains are isomorphic
  (Johnson and Geyer, submitted) and have the same convergence rate.
  The \code{\link{morph}} variable provides a diffeomorphism,
  \code{morph.metrop} uses this diffeomorphism to induce the log
  unnormalized density, \eqn{\log f_Y}{log fY} based on the user
  supplied log unnormalized density, \eqn{\log f_X}{log fX}.
  \code{morph.metrop} runs a Metropolis algorithm for \eqn{\log f_Y}{log
  fY} and transforms the resulting Markov chain into a Markov chain for
  \eqn{f_X}{fX}.  The user accessible output components are the same as
  those that come from \code{\link{metrop}}, see the documentation for
  \code{\link{metrop}} for details.

  Subsequent calls of \code{morph.metrop} may change to the
  transformation by specifying a new value for \code{morph}.

  Any of the other parameters to \code{morph.metrop} may also be
  modified in subsequent calls.  See \code{\link{metrop}} for more details.

  The general idea is that a random-walk Metropolis sampler
  (what \code{\link{metrop}} does) will not be geometrically
  ergodic unless the tails of the unnormalized density decrease
  superexponentially fast (so the tails of the log unnormalized density
  decrease faster than linearly).  It may not be geometrically ergodic
  even then (see Johnson and Geyer, submitted, for the complete theory).
  The transformations used by this function (provided by \code{\link{morph}})
  can produce geometrically ergodic chains when the tails of the log
  unnormalized density are too light for \code{\link{metrop}} to do so.

  When the tails of the unnormalized density are exponentially light but
  not superexponentially light (so the tails of the log unnormalized density
  are asymptotically linear, as in the case of exponential family models
  when conjugate priors are used, for example logistic regression, Poisson
  regression with log link, or log-linear models for categorical data), one
  should use \code{\link{morph}} with \code{b = 0} (the default), which
  produces a transformation of the form \eqn{g_1}{g1} in the notation
  used in the details section of the help for \code{\link{morph}}.
  This will produce a geometrically ergodic sampler if other features of the
  log unnormalized density are well behaved.  For example it will do so
  for the exponential family examples mentioned above.
  (See Johnson and Geyer, submitted, for the complete theory.)

  The transformation \eqn{g_1}{g1} behaves like a shift transformation
  on a ball of radius \code{r} centered at \code{center}, so these arguments
  to \code{\link{morph}} should be chosen so that a sizable proportion of
  the probability under the original (untransformed) unnormalized density
  is contained in this ball.  This function will work when \code{r = 0} and
  \code{center = 0} (the defaults) are used, but may not work as well as when
  \code{r} and \code{center} are well chosen.

  When the tails of the unnormalized density are not exponentially light
  (so the tails of the log unnormalized density decrease sublinearly, as
  in the case of univariate and multivariate \eqn{t} distributions), one
  should use \code{\link{morph}} with \code{r > 0} and \code{p = 3}, which
  produces a transformation of the form \eqn{g_2}{g2} composed
  with \eqn{g_1}{g1} in the notation
  used in the details section of the help for \code{\link{morph}}.
  This will produce a geometrically ergodic sampler if other features of the
  log unnormalized density are well behaved.  For example it will do so
  for the \eqn{t} examples mentioned above.
  (See Johnson and Geyer, submitted, for the complete theory.)
}
\value{
  an object of class \code{mcmc}, subclass \code{morph.metropolis}.
  This object is a list containing all of the elements from an object
  returned by \code{\link{metrop}}, plus at least the following
  components:
  \item{morph}{the morph object used for the transformations.}
  \item{morph.final}{the final state of the Markov chain on the
    transformed scale.}
}
\examples{
out <- morph.metrop(function(x) dt(x, df=3, log=TRUE), 0, blen=100,
  nbatch=100, morph=morph(b=1))
# change the transformation.
out <- morph.metrop(out, morph=morph(b=2))
out$accept
# accept rate is high, increase the scale.
out <- morph.metrop(out, scale=4)
# close to 0.20 is about right.
out$accept
}
\references{
Johnson, L. T. and Geyer, C. J. (submitted)
Variable Transformation to Obtain Geometric Ergodicity
    in the Random-walk Metropolis Algorithm.
}
\seealso{
  \code{\link{metrop}}, \code{\link{morph}}.
}
\keyword{misc}

\name{logit}
\docType{data}
\alias{logit}
\title{Simulated logistic regression data.}
\description{
  Like it says
}
\usage{data(logit)}
\format{
  A data frame with variables
   \describe{
    \item{x1}{quantitative predictor.}
    \item{x2}{quantitative predictor.}
    \item{x3}{quantitative predictor.}
    \item{x4}{quantitative predictor.}
    \item{y}{Bernoulli response.}
  }
}
\examples{
library(mcmc)
data(logit)
out <- glm(y ~ x1 + x2 + x3 + x4, family = binomial, data = logit)
summary(out)
}
\keyword{datasets}

\name{metrop}
\alias{metrop}
\alias{metrop.function}
\alias{metrop.metropolis}
\title{Metropolis Algorithm}
\description{
    Markov chain Monte Carlo for continuous random vector using a Metropolis
    algorithm.
}
\usage{
metrop(obj, initial, nbatch, blen = 1, nspac = 1, scale = 1, outfun,
    debug = FALSE, ...)
\method{metrop}{function}(obj, initial, nbatch, blen = 1, nspac = 1,
    scale = 1, outfun, debug = FALSE, ...)
\method{metrop}{metropolis}(obj, initial, nbatch, blen = 1, nspac = 1,
    scale = 1, outfun, debug = FALSE, ...)
}
\arguments{
  \item{obj}{Either an \R function or an object of class \code{"metropolis"}
      from a previous invocation of this function.

      If a function, it evaluates the log unnormalized probability
      density of the desired equilibrium distribution of the Markov chain.
      Its first argument is the state vector of the Markov chain.  Other
      arguments arbitrary and taken from the \code{...} arguments of this
      function.
      It should return \code{-Inf} for points of the state space having
      probability zero under the desired equilibrium distribution.
      See also Details and Warning.

      If an object of class \code{"metropolis"}, any missing arguments
      (including the log unnormalized density function) are taken from
      this object.  Also \code{initial} is ignored and the initial state
      of the Markov chain is the final state from the run recorded in
      \code{obj}.
  }
  \item{initial}{a real vector, the initial state of the Markov chain.
      Must be feasible, see Details.  Ignored if \code{obj} is of
      class \code{"metropolis"}.}
  \item{nbatch}{the number of batches.}
  \item{blen}{the length of batches.}
  \item{nspac}{the spacing of iterations that contribute to batches.}
  \item{scale}{controls the proposal step size.  If scalar or
          vector, the proposal is \code{x + scale * z} where \code{x} is
          the current state and \code{z} is a standard normal random vector.
          If matrix, the proposal is \code{x + scale \%*\% z}.}
  \item{outfun}{controls the output.  If a function, then the batch means
          of \code{outfun(state, ...)} are returned.  If a numeric
          or logical vector, then the batch means of \code{state[outfun]}
          (if this makes sense).  If missing, the the batch means
          of \code{state} are returned.}
  \item{debug}{if \code{TRUE} extra output useful for testing.}
  \item{...}{additional arguments for \code{obj} or \code{outfun}.}
}
\details{
Runs a \dQuote{random-walk} Metropolis algorithm, terminology introduced
by Tierney (1994), with multivariate normal proposal
producing a Markov chain with equilibrium distribution having a specified
unnormalized density.  Distribution must be continuous.  Support of the
distribution is the support of the density specified by argument \code{obj}.
The initial state must satisfy \code{obj(state, ...) > -Inf}.
Description of a complete MCMC analysis (Bayesian logistic regression)
using this function can be found in the vignette
\code{vignette("demo", "mcmc")}.

Suppose the function coded by the log unnormalized function (either
\code{obj} or \code{obj$lud}) is actually a log unnormalized density,
that is, if \eqn{w} denotes that function, then \eqn{e^w}{exp(w)} integrates
to some value strictly between zero and infinity.  Then the \code{metrop}
function always simulates a reversible, Harris ergodic Markov chain having
the equilibrium distribution with this log unnormalized density.
The chain is not guaranteed to be geometrically ergodic.  In fact it cannot
be geometrically ergodic if the tails of the log unnormalized density are
sufficiently heavy.  The \code{\link{morph.metrop}} function deals with this
situation.
}
\value{
  an object of class \code{"mcmc"}, subclass \code{"metropolis"},
  which is a list containing at least the following components:
  \item{accept}{fraction of Metropolis proposals accepted.}
  \item{batch}{\code{nbatch} by \code{p} matrix, the batch means, where
      \code{p} is the dimension of the result of \code{outfun}
      if \code{outfun} is a function, otherwise the dimension of
      \code{state[outfun]} if that makes sense, and the dimension
      of \code{state} when \code{outfun} is missing.}
  \item{accept.batch}{a vector of length \code{nbatch}, the batch means
      of the acceptances.}
  \item{initial}{value of argument \code{initial}.}
  \item{final}{final state of Markov chain.}
  \item{initial.seed}{value of \code{.Random.seed} before the run.}
  \item{final.seed}{value of \code{.Random.seed} after the run.}
  \item{time}{running time of Markov chain from \code{system.time()}.}
  \item{lud}{the function used to calculate log unnormalized density,
  either \code{obj} or \code{obj$lud} from a previous run.}
  \item{nbatch}{the argument \code{nbatch} or \code{obj$nbatch}.}
  \item{blen}{the argument \code{blen} or \code{obj$blen}.}
  \item{nspac}{the argument \code{nspac} or \code{obj$nspac}.}
  \item{outfun}{the argument \code{outfun} or \code{obj$outfun}.}
  Description of additional output when \code{debug = TRUE} can be
  found in the vignette \code{debug} (\url{../doc/debug.pdf}).
}
\section{Warning}{
If \code{outfun} is missing or not a function, then the log unnormalized
density can be defined without a \ldots argument and that works fine.
One can define it starting \code{ludfun <- function(state)} and that works
or \code{ludfun <- function(state, foo, bar)}, where \code{foo} and \code{bar}
are supplied as additional arguments to \code{metrop}.

If \code{outfun} is a function, then both it and the log unnormalized
density function can be defined without \ldots arguments \emph{if they
have exactly the same arguments list} and that works fine.  Otherwise it
doesn't work.  Define these functions by
\preformatted{
ludfun <- function(state, foo)
outfun <- function(state, bar)
}
and you get an error about unused arguments.  Instead define these functions by
\preformatted{
ludfun <- function(state, foo, \ldots)
outfun <- function(state, bar, \ldots)
}
and supply \code{foo} and \code{bar} as additional arguments to \code{metrop},
and that works fine.

In short, the log unnormalized density function and \code{outfun} need
to have \ldots in their arguments list to be safe.  Sometimes it works
when \ldots is left out and sometimes it doesn't.

Of course, one can avoid this whole issue by always defining the log
unnormalized density function and \code{outfun} to have only one argument
\code{state} and use global variables (objects in the \R global environment) to
specify any other information these functions need to use.  That too
follows the \R way.  But some people consider that bad programming practice.

A third option is to define either or both of these functions using a function
factory.  This is demonstrated in the vignette for this package named
\code{demo}, which is shown by \code{vignette("demo", "mcmc")}.
}
\section{Philosophy of MCMC}{
This function follows the philosophy of MCMC explained
the introductory chapter of the
\emph{Handbook of Markov Chain Monte Carlo} (Geyer, 2011).

This function automatically does batch means in order to reduce
the size of output and to enable easy calculation of Monte Carlo standard
errors (MCSE), which measure error due to the Monte Carlo sampling (not
error due to statistical sampling --- MCSE gets smaller when you run the
computer longer, but statistical sampling variability only gets smaller
when you get a larger data set).  All of this is explained in the package
vignette \code{vignette("demo", "mcmc")} and in Section 1.10 of Geyer (2011).

This function does not apparently
do \dQuote{burn-in} because this concept does not actually help with MCMC
(Geyer, 2011, Section 1.11.4) but the re-entrant property of this
function does allow one to do \dQuote{burn-in} if one wants.
Assuming \code{ludfun}, \code{start.value}, \code{scale}
have been already defined
and are, respectively, an \R function coding the log unnormalized density
of the target distribution, a valid state of the Markov chain,
and a useful scale factor,
\preformatted{
out <- metrop(ludfun, start.value, nbatch = 1, blen = 1e5, scale = scale)
out <- metrop(out, nbatch = 100, blen = 1000)
}
throws away a run of 100 thousand iterations before doing another run of
100 thousand iterations that is actually useful for analysis, for example,
\preformatted{
apply(out$batch, 2, mean)
apply(out$batch, 2, sd) / sqrt(out$nbatch)
}
give estimates of posterior means and their MCSE assuming the batch length
(here 1000) was long enough to contain almost all of the significant
autocorrelation (see Geyer, 2011, Section 1.10, for more on MCSE).
The re-entrant property of this function (the second run starts
where the first one stops) assures that this is really \dQuote{burn-in}.

The re-entrant property allows one to do very long runs without having to
do them in one invocation of this function.
\preformatted{
out2 <- metrop(out)
out3 <- metrop(out2)
batch <- rbind(out$batch, out2$batch, out3$batch)
}
produces a result as if the first run had been three times as long.
}
\section{Tuning}{
The \code{scale} argument must be adjusted so that the acceptance rate
is not too low or too high to get reasonable performance.  The rule of
thumb is that the acceptance rate should be about 25\%.
But this recommendation (Gelman, et al., 1996) is justified by analysis
of a toy problem (simulating a spherical multivariate normal distribution)
for which MCMC is unnecessary.  There is no reason to believe this is optimal
for all problems (if it were optimal, a stronger theorem could be proved).
Nevertheless, it is clear that at very low acceptance rates the chain makes
little progress (because in most iterations it does not move) and that at
very high acceptance rates the chain also makes little progress (because
unless the log unnormalized density is nearly constant, very high acceptance
rates can only be achieved by very small values of \code{scale} so the
steps the chain takes are also very small).

Even in the Gelman, et al. (1996) result, the optimal rate for spherical
multivariate normal depends on dimension.  It is 44\% for \eqn{d = 1}
and 23\% for \eqn{d = \infty}{d = infinity}.
Geyer and Thompson (1995) have an example, admittedly for
simulated tempering (see \code{\link{temper}}) rather than random-walk
Metropolis, in which no acceptance rate less than 70\% produces an ergodic
Markov chain.  Thus 25\% is merely a rule of thumb.  We only know we don't
want too high or too low.  Probably 1\% or 99\% is very inefficient.
}
\references{
Gelman, A., Roberts, G. O., and Gilks, W. R. (1996)
Efficient Metropolis jumping rules.
In \emph{Bayesian Statistics 5: Proceedings of the Fifth Valencia
    International Meeting}.  Edited by J. M. Bernardo,
    J. O. Berger, A. P. Dawid, and A. F. M. Smith.
Oxford University Press, Oxford, pp. 599--607. 

Geyer, C. J. (2011)
Introduction to MCMC.
In \emph{Handbook of Markov Chain Monte Carlo}. Edited by S. P. Brooks,
A. E. Gelman, G. L. Jones, and X. L. Meng.
Chapman & Hall/CRC, Boca Raton, FL, pp. 3--48.

Geyer, C. J. and Thompson, E. A. (1995)
Annealing Markov chain Monte Carlo with applications to ancestral inference.
\emph{Journal of the American Statistical Association} \bold{90} 909--920.

Tierney, L. (1994)
Markov chains for exploring posterior distributions (with discussion).
\emph{Annals of Statistics} \bold{22} 1701--1762.
}
\seealso{
\code{\link{morph.metrop}} and \code{\link{temper}}
}
\examples{
h <- function(x) if (all(x >= 0) && sum(x) <= 1) return(1) else return(-Inf)
out <- metrop(h, rep(0, 5), 1000)
out$accept
# acceptance rate too low
out <- metrop(out, scale = 0.1)
out$accept
t.test(out$accept.batch)$conf.int
# acceptance rate o. k. (about 25 percent)
plot(out$batch[ , 1])
# but run length too short (few excursions from end to end of range)
out <- metrop(out, nbatch = 1e4)
out$accept
plot(out$batch[ , 1])
hist(out$batch[ , 1])
acf(out$batch[ , 1], lag.max = 250)
# looks like batch length of 250 is perhaps OK
out <- metrop(out, blen = 250, nbatch = 100)
apply(out$batch, 2, mean) # Monte Carlo estimates of means
apply(out$batch, 2, sd) / sqrt(out$nbatch) # Monte Carlo standard errors
t.test(out$accept.batch)$conf.int
acf(out$batch[ , 1]) # appears that blen is long enough
}
\keyword{misc}

\name{initseq}
\alias{initseq}
\title{Initial Sequence Estimators}
\description{
    Variance of sample mean of functional of reversible Markov chain
    using methods of Geyer (1992).
}
\usage{
initseq(x)
}
\arguments{
  \item{x}{a numeric vector that is a scalar-valued functional of a reversible
      Markov chain.}
}
\details{
Let
\deqn{\gamma_k = \textrm{cov}(X_i, X_{i + k})}{gamma[k] = cov(x[i], x[i + k])}
considered as a function of the lag \eqn{k} be
the autocovariance function of the input time series.
Define
\deqn{\Gamma_k = \gamma_{2 k} + \gamma_{2 k + 1}}{Gamma[k] = gamma[2 k] + gamma[2 k + 1]}
the sum of consecutive pairs of autocovariances.  Then Theorem 3.1 in
Geyer (1992) says that \eqn{\Gamma_k}{Gamma[k]} considered as a function of
\eqn{k} is strictly positive, strictly decreasing, and strictly convex,
assuming the input time series is a scalar-valued functional of a reversible Markov
chain.  All of the MCMC done by this package is reversible.
This \R function estimates the \dQuote{big gamma} function,
\eqn{\Gamma_k}{Gamma[k]} considered as a function of
\eqn{k}, subject to three different constraints, (1) nonnegative,
(2) nonnegative and nonincreasing, and (3) nonnegative, nonincreasing,
and convex.  It also estimates the variance in the Markov chain central
limit theorem (CLT)
\deqn{\gamma_0 + 2 \sum_{k = 1}^\infty \gamma_k = - \gamma_0 + 2 \sum_{k = 0}^\infty \Gamma_k}{- gamma0 + 2 * sum(gamma) = - gamma0 + 2 * sum(Gamma)}

\strong{Note:} The batch means provided by \code{\link{metrop}} are also
scalar functionals of a reversible Markov chain.  Thus these initial sequence
estimators applied to the batch means give valid standard errors for the
mean of the match means even when the batch length is too short to provide
a valid estimate of asymptotic variance.  One does, of course, have to
multiply the asymptotic variance of the batch means by the batch length
to get the asymptotic variance for the unbatched chain.
}
\value{
a list containing the following components:

  \item{gamma0}{the scalar \eqn{\gamma_0}{gamma[0]}, the marginal variance
  of \code{x}.}
  \item{Gamma.pos}{the vector \eqn{\Gamma}{Gamma}, estimated so as to be nonnegative,
  where, as always, \R uses one-origin indexing so \code{Gamma.pos[1]} is 
  \eqn{\Gamma_0}{Gamma[0]}.}
  \item{Gamma.dec}{the vector \eqn{\Gamma}{Gamma}, estimated so as to be nonnegative
  and nonincreasing, where, as always,
  \R uses one-origin indexing so \code{Gamma.dec[1]} is 
  \eqn{\Gamma_0}{Gamma[0]}.}
  \item{Gamma.con}{the vector \eqn{\Gamma}{Gamma}, estimated so as to be nonnegative
  and nonincreasing and convex, where, as always,
  \R uses one-origin indexing so \code{Gamma.con[1]} is 
  \eqn{\Gamma_0}{Gamma[0]}.}
  \item{var.pos}{the scalar \code{- gamma0 + 2 * sum(Gamma.pos)}, which is
  the asymptotic variance in the Markov chain CLT.  Divide by \code{length(x)}
  to get the approximate variance of the sample mean of \code{x}.}
  \item{var.dec}{the scalar \code{- gamma0 + 2 * sum(Gamma.dec)}, which is
  the asymptotic variance in the Markov chain CLT.  Divide by \code{length(x)}
  to get the approximate variance of the sample mean of \code{x}.}
  \item{var.con}{the scalar \code{- gamma0 + 2 * sum(Gamma.con)}, which is
  the asymptotic variance in the Markov chain CLT.  Divide by \code{length(x)}
  to get the approximate variance of the sample mean of \code{x}.}
}
\section{Bugs}{
Not precisely a bug, but \code{var.pos}, \code{var.dec}, and \code{var.con}
can be negative.  This happens only when the chain is way too short to estimate
the variance, and even then rarely.  But it does happen.
}
\references{
Geyer, C. J. (1992)
Practical Markov Chain Monte Carlo.
\emph{Statistical Science} \bold{7} 473--483.
}
\seealso{
\code{\link{metrop}}
}
\examples{
n <- 2e4
rho <- 0.99
x <- arima.sim(model = list(ar = rho), n = n)
out <- initseq(x)
\dontrun{
plot(seq(along = out$Gamma.pos) - 1, out$Gamma.pos,
   xlab = "k", ylab = expression(Gamma[k]), type = "l")
lines(seq(along = out$Gamma.dec) - 1, out$Gamma.dec, col = "red")
lines(seq(along = out$Gamma.con) - 1, out$Gamma.con, col = "blue")
}
# asymptotic 95\% confidence interval for mean of x
mean(x) + c(-1, 1) * qnorm(0.975) * sqrt(out$var.con / length(x))
# estimated asymptotic variance
out$var.con
# theoretical asymptotic variance
(1 + rho) / (1 - rho) * 1 / (1 - rho^2)
# illustrating use with batch means
bm <- apply(matrix(x, nrow = 5), 2, mean)
initseq(bm)$var.con * 5
}
\keyword{ts}

\name{temper}
\alias{temper}
\alias{temper.function}
\alias{temper.tempering}
\title{Simulated Tempering and Umbrella Sampling}
\description{
    Markov chain Monte Carlo (MCMC) for continuous random vectors using
    parallel or serial tempering, the latter also called umbrella
    sampling and simulated tempering.
    The chain simulates \eqn{k} different distributions on the
    same state space.  In parallel tempering, all the distributions are
    simulated in each iteration.  In serial tempering, only one of the
    distributions is simulated (a random one).  In parallel tempering,
    the state is a \eqn{k \times p}{k by p} matrix, each row of which
    is the state for one of the distributions.
    In serial tempering the state of the Markov chain is a pair \eqn{(i, x)},
    where \eqn{i} is an integer between 1 and \eqn{k} and \eqn{x} is a vector
    of length \eqn{p}.  This pair is represented as a single real vector
    \code{c(i, x)}.  The variable \eqn{i} says which distribution \eqn{x}
    is a simulation for.
}
\usage{
temper(obj, initial, neighbors, nbatch, blen = 1, nspac = 1, scale = 1,
    outfun, debug = FALSE, parallel = FALSE, \dots)
\method{temper}{function}(obj, initial, neighbors, nbatch,
    blen = 1, nspac = 1, scale = 1,
    outfun, debug = FALSE, parallel = FALSE, \dots)
\method{temper}{tempering}(obj, initial, neighbors, nbatch,
    blen = 1, nspac = 1, scale = 1,
    outfun, debug = FALSE, parallel = FALSE, \dots)
}
\arguments{
  \item{obj}{either an \R function or an object of class \code{"tempering"}
    from a previous run.

    If a function, it should evaluate the log unnormalized
    density \eqn{\log h(i, x)}{log h(i, x)} of the desired equilibrium
    distribution of the Markov chain for serial tempering (the same function
    is used for both serial and parallel tempering, see Details below for
    further explanation).

    If an object of class \code{"tempering"},
    the log unnormalized density function
    is \code{obj$lud}, and missing arguments of \code{temper} are
    obtained from the corresponding elements of \code{obj}.

    The first argument of the log unnormalized density function is the
    is an \R vector \code{c(i, x)}, where \code{i} says which distribution
    \code{x} is supposed to be a simulation for.
    Other arguments are arbitrary and taken from
    the \code{\dots} arguments of \code{temper}.  The log unnormalized density
    function should return \code{-Inf} in regions of the state space having
    probability zero.}
  \item{initial}{for serial tempering, a real vector \code{c(i, x)} as
    described above.  For parallel tempering, a real
    \eqn{k \times p}{k by p} matrix as described above.  In either case,
    the initial state of the Markov chain.
    Ignored if \code{obj} has class \code{"tempering"}.}
  \item{neighbors}{a logical symmetric matrix of dimension \code{k}
    by \code{k}.  Elements that are \code{TRUE} indicate jumps
    or swaps attempted by the Markov chain.
    Ignored if \code{obj} has class \code{"tempering"}.}
  \item{nbatch}{the number of batches.}
  \item{blen}{the length of batches.}
  \item{nspac}{the spacing of iterations that contribute to batches.}
  \item{scale}{controls the proposal step size for real elements of the state
    vector.  For serial tempering, proposing a new value for the \eqn{x}
    part of the state \eqn{(i, x)}.  For parallel tempering, proposing
    a new value for the \eqn{x_i}{x[i]} part of the state
    \eqn{(x_1, \ldots, x_k)}{(x[1], \ldots, x[k])}.  In either case, the proposal
    is a real vector of length \eqn{p}.  If scalar or vector, the proposal
    is \code{x + scale * z} where \code{x} is the part \eqn{x} or
    \eqn{x_i}{x[i]} of the state the proposal may replace.
    If matrix, the proposal is
    \code{x + scale \%*\% z}.  If list, the length must be \code{k},
    and each element must be scalar, vector, or matrix, and operate as
    described above.  The \eqn{i}-th component of the list is used to update
    \eqn{x} when the state is \eqn{(i, x)} or \eqn{x_i}{x[i]} otherwise.}
  \item{outfun}{controls the output.  If a function, then the batch means
      of \code{outfun(state, \dots)} are returned.  The argument \code{state}
      is like the argument \code{initial} of this function.  If missing, the
      batch means of the real part of the state vector or matrix are returned,
      and for serial tempering the batch means of a multivariate Bernoulli
      indicating the current component are returned.}
  \item{debug}{if \code{TRUE} extra output useful for testing.}
  \item{parallel}{if \code{TRUE} does parallel tempering, if \code{FALSE} does
      serial tempering.
      Ignored if \code{obj} has class \code{"tempering"}.}
  \item{...}{additional arguments for \code{obj} or \code{outfun}.}
}
\details{
Serial tempering simulates a mixture of distributions of a continuous random
vector.  The number of components of the mixture is \code{k}, and the dimension
of the random vector is \code{p}.  Denote the state \eqn{(i, x)}, where \eqn{i}
is a positive integer between 1 and \eqn{k}, and let \eqn{h(i, x)} denote
the unnormalized joint density of their equilibrium distribution.
The logarithm of this function is what \code{obj} or \code{obj$lud} calculates.
The mixture distribution is the marginal for \eqn{x} derived from
the equilibrium distribution \eqn{h(i, x)}, that is,
\deqn{h(x) = \sum_{i = 1}^k h(i, x)}{h(x) = sum[i = 1 to k] h(i, x)}

Parallel tempering simulates a product of distributions of a continuous random
vector.  Denote the state \eqn{(x_1, \ldots, x_k)}{(x[1], \ldots, x[k])},
then the unnormalized joint density of the equilibrium distribution is
\deqn{h(x_1, \ldots, x_k) = \prod_{i = 1}^k h(i, x_i)}{h(x[1], \dots, x[k]) = prod[i = 1 to k] h(i, x[i])}

The update mechanism of the Markov chain combines two kinds of elementary
updates: jump/swap updates (jump for serial tempering, swap for parallel
tempering) and within-component updates.  Each iteration of the Markov chain
one of these elementary updates is done.  With probability 1/2 a jump/swap
update is done, and with probability 1/2 a with-component update is done.

Within-component updates are the same for both serial and parallel tempering.
They are \dQuote{random-walk} Metropolis updates with multivariate normal
proposal, the proposal distribution being determined by the argument
\code{scale}.  In serial tempering, the \eqn{x} part of the current state
\eqn{(i, x)} is updated preserving \eqn{h(i, x)}.
In parallel tempering, an index \eqn{i} is chosen at random and the part
of the state \eqn{x_i}{x[i]} representing that component is updated,
again preserving \eqn{h(i, x)}.

Jump updates choose uniformly at random a neighbor of the current component:
if \eqn{i} indexes the current component, then it chooses uniformly at random
a \eqn{j} such that \code{neighbors[i, j] == TRUE}.  It then does does a
Metropolis-Hastings update for changing the current state from \eqn{(i, x)}
to \eqn{(j, x)}.

Swap updates choose a component uniformly at random and a neighbor of that
component uniformly at random: first an index \eqn{i} is chosen uniformly
at random between 1 and \eqn{k}, then an index \eqn{j} is chosen
uniformly at random such that \code{neighbors[i, j] == TRUE}.  It then does
does a Metropolis-Hastings update for swapping the states of the
two components: interchanging \eqn{x_i}{x[i, ]} and \eqn{x_j}{x[j, ]}
while preserving \eqn{h(x_1, \ldots, x_k)}{h(x[1], \dots, x[k])}.

The initial state must satisfy \code{lud(initial, ...) > - Inf} for serial
tempering or must satisfy \code{lud(initial[i, ], ...) > - Inf} for each
\code{i} for parallel tempering, where \code{lud} is either \code{obj}
or \code{obj$lud}.
That is, the initial state must have positive probability.
}
\value{
  an object of class \code{"mcmc"}, subclass \code{"tempering"},
  which is a list containing at least the following components:
  \item{batch}{the batch means of the continuous part of the state.
    If \code{outfun} is missing, an \code{nbatch} by \code{k} by \code{p}
    array.  Otherwise, an \code{nbatch} by \code{m} matrix, where \code{m}
    is the length of the result of \code{outfun}.}
  \item{ibatch}{(returned for serial tempering only) an \code{nbatch}
    by \code{k} matrix giving batch means for the multivariate Bernoulli
    random vector that is all zeros except for a 1 in the \code{i}-th place
    when the current state is \eqn{(i, x)}.}
  \item{acceptx}{fraction of Metropolis within-component proposals accepted.
    A vector of length \code{k} giving the acceptance rate for each component.}
  \item{accepti}{fraction of Metropolis jump/swap proposals accepted.
    A \code{k} by \code{k} matrix giving the acceptance rate for each allowed
    jump or swap component.  \code{NA} for elements such that the corresponding
    elements of \code{neighbors} is \code{FALSE}.}
  \item{initial}{value of argument \code{initial}.}
  \item{final}{final state of Markov chain.}
  \item{initial.seed}{value of \code{.Random.seed} before the run.}
  \item{final.seed}{value of \code{.Random.seed} after the run.}
  \item{time}{running time of Markov chain from \code{system.time()}.}
  \item{lud}{the function used to calculate log unnormalized density,
  either \code{obj} or \code{obj$lud} from a previous run.}
  \item{nbatch}{the argument \code{nbatch} or \code{obj$nbatch}.}
  \item{blen}{the argument \code{blen} or \code{obj$blen}.}
  \item{nspac}{the argument \code{nspac} or \code{obj$nspac}.}
  \item{outfun}{the argument \code{outfun} or \code{obj$outfun}.}
  Description of additional output when \code{debug = TRUE} can be
  found in the vignette \code{debug}, which is shown by
  \code{vignette("debug", "mcmc")}.
}
\section{Warning}{
If \code{outfun} is missing, then the log unnormalized
density function can be defined without a \ldots argument and that works fine.
One can define it starting \code{ludfun <- function(state)} and that works
or \code{ludfun <- function(state, foo, bar)}, where \code{foo} and \code{bar}
are supplied as additional arguments to \code{temper} and that works too.

If \code{outfun} is a function, then both it and the log unnormalized
density function can be defined without \ldots arguments \emph{if they
have exactly the same arguments list} and that works fine.  Otherwise it
doesn't work.  Define these functions by
\preformatted{
ludfun <- function(state, foo)
outfun <- function(state, bar)
}
and you get an error about unused arguments.  Instead define these functions by
\preformatted{
ludfun <- function(state, foo, \ldots)
outfun <- function(state, bar, \ldots)
}
and supply \code{foo} and \code{bar} as additional arguments to \code{temper},
and that works fine.

In short, the log unnormalized density function and \code{outfun} need
to have \ldots in their arguments list to be safe.  Sometimes it works
when \ldots is left out and sometimes it doesn't.

Of course, one can avoid this whole issue by always defining the log
unnormalized density function and \code{outfun} to have only one argument
\code{state} and use global variables (objects in the \R global environment) to
specify any other information these functions need to use.  That too
follows the \R way.  But some people consider that bad programming practice.

A third option is to define either or both of these functions using a function
factory.  This is demonstrated in the vignette for this package named
\code{demo}, which is shown by \code{vignette("demo", "mcmc")}.
}
\section{Philosophy of MCMC}{
This function follows the philosophy of MCMC explained
the introductory chapter of the
\emph{Handbook of Markov Chain Monte Carlo} (Geyer, 2011a)
and in the chapter of that book on tempering and related subjects
(Geyer, 2011b).
See also the section on philosophy of \code{\link{metrop}}.
}
\section{Tuning}{
The \code{scale} argument must be adjusted so that the acceptance rate
for within-component proposals (component \code{acceptx} of the result
returned by this function)
is not too low or too high to get reasonable performance.
The log unnormalized density function must be chosen so that the acceptance rate
for jump/swap proposals (component \code{accepti} of the result
returned by this function)
is not too low or too high to get reasonable performance.
The former is a vector and the latter is a matrix, and
all these rates must be adjusted to be reasonable.

The rates in in \code{accepti} are influenced by the number of components
of the tempering mixture distribution, by what those components are (how
far apart they are in some unspecified metric on probability distributions),
and by the chosen normalizing constants for those distributions.

For examples of tuning tempering, see Geyer (2011b) and also the vignette
of this package shown by \code{vignette("bfst", "mcmc")}.
The help for R function \code{\link{metrop}} also gives advice on tuning
its sampler, which is relevant for tuning the \code{acceptx} rates.

See also Geyer (1991) and Geyer and Thompson (1995) for the general
theory of tuning parallel and serial tempering.
}

\references{
Geyer, C. J. (1991)
Markov chain Monte Carlo maximum likelihood.
\emph{Computing Science and Statistics: Proc. 23rd Symp. Interface}, 156--163.
\url{http://hdl.handle.net/11299/58440}.

Geyer, C. J. (2011a)
Introduction to MCMC.
In \emph{Handbook of Markov Chain Monte Carlo}. Edited by S. P. Brooks,
A. E. Gelman, G. L. Jones, and X. L. Meng.
Chapman & Hall/CRC, Boca Raton, FL, pp. 3--48.

Geyer, C. J. (2011b)
Importance Sampling, Simulated Tempering, and Umbrella Sampling.
In \emph{Handbook of Markov Chain Monte Carlo}. Edited by S. P. Brooks,
A. E. Gelman, G. L. Jones, and X. L. Meng.
Chapman & Hall/CRC, Boca Raton, FL, pp. 295--312.

Geyer, C. J. and Thompson, E. A. (1995)
Annealing Markov chain Monte Carlo with applications to ancestral inference.
\emph{Journal of the American Statistical Association} \bold{90} 909--920.
}
\examples{
d <- 9
witch.which <- c(0.1, 0.3, 0.5, 0.7, 1.0)
ncomp <- length(witch.which)

neighbors <- matrix(FALSE, ncomp, ncomp)
neighbors[row(neighbors) == col(neighbors) + 1] <- TRUE
neighbors[row(neighbors) == col(neighbors) - 1] <- TRUE

ludfun <- function(state, log.pseudo.prior = rep(0, ncomp)) {
    stopifnot(is.numeric(state))
    stopifnot(length(state) == d + 1)
    icomp <- state[1]
    stopifnot(icomp == as.integer(icomp))
    stopifnot(1 <= icomp && icomp <= ncomp)
    stopifnot(is.numeric(log.pseudo.prior))
    stopifnot(length(log.pseudo.prior) == ncomp)
    theta <- state[-1]
    if (any(theta > 1.0)) return(-Inf)
    bnd <- witch.which[icomp]
    lpp <- log.pseudo.prior[icomp]
    if (any(theta > bnd)) return(lpp)
    return(- d * log(bnd) + lpp)
}

# parallel tempering
thetas <- matrix(0.5, ncomp, d)
out <- temper(ludfun, initial = thetas, neighbors = neighbors, nbatch = 20,
    blen = 10, nspac = 5, scale = 0.56789, parallel = TRUE, debug = TRUE)

# serial tempering
theta.initial <- c(1, rep(0.5, d))
# log pseudo prior found by trial and error
qux <- c(0, 9.179, 13.73, 16.71, 20.56)

out <- temper(ludfun, initial = theta.initial, neighbors = neighbors,
    nbatch = 50, blen = 30, nspac = 2, scale = 0.56789,
    parallel = FALSE, debug = FALSE, log.pseudo.prior = qux)
}
\keyword{misc}

\name{olbm}
\alias{olbm}
\title{Overlapping Batch Means}
\description{
    Variance of sample mean of time series calculated using overlapping
    batch means.
}
\usage{
olbm(x, batch.length, demean = TRUE)
}
\arguments{
  \item{x}{a matrix or time series object.  Each column of \code{x} is
      treated as a scalar time series.}
  \item{batch.length}{length of batches.}
  \item{demean}{when \code{demean = TRUE} (the default) the sample mean
      is subtracted from each batch mean when estimating the variance.
      Using \code{demean = FALSE} would essentially assume the true mean
      is known to be zero, which might be useful in a toy problem where
      the answer is known.}
}
\value{
  The estimated variance of the sample mean.
}
\seealso{
  \code{\link{ts}}
}
\examples{
h <- function(x) if (all(x >= 0) && sum(x) <= 1) return(1) else return(-Inf)
out <- metrop(h, rep(0, 5), 1000)
out <- metrop(out, scale = 0.1)
out <- metrop(out, nbatch = 1e4)
foo <- olbm(out$batch, 150)
# monte carlo estimates (true means are same by symmetry)
apply(out$batch, 2, mean)
# monte carlo standard errors (true s. d. are same by symmetry)
sqrt(diag(foo))
# check that batch length is reasonable
acf(out$batch, lag.max = 200)
}
\keyword{ts}
